@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
@@ -10,30 +10,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -60,28 +58,36 @@
 #include <private/qv4functionobject_p.h>
 #include <private/qv4objectproto_p.h>
 #include <private/qv4scopedvalue_p.h>
+#include <private/qv4objectiterator_p.h>
 
-using namespace QV4;
+static void initResources()
+{
+#ifdef QT_STATIC
+    Q_INIT_RESOURCE(qmake_QtQuick_LocalStorage);
+#endif
+}
+
+QT_BEGIN_NAMESPACE
 
 #define V4THROW_SQL(error, desc) { \
-    QV4::Scoped<String> v(scope, ctx->engine->newString(desc)); \
-    QV4::Scoped<Object> ex(scope, ctx->engine->newErrorObject(v)); \
-    ex->put(QV4::ScopedString(scope, ctx->engine->newIdentifier(QStringLiteral("code"))), QV4::ScopedValue(scope, Primitive::fromInt32(error))); \
-    ctx->throwError(ex); \
+    QV4::ScopedString v(scope, scope.engine->newString(desc)); \
+    QV4::ScopedObject ex(scope, scope.engine->newErrorObject(v)); \
+    ex->put(QV4::ScopedString(scope, scope.engine->newIdentifier(QStringLiteral("code"))).getPointer(), QV4::ScopedValue(scope, Primitive::fromInt32(error))); \
+    ctx->engine()->throwError(ex); \
     return Encode::undefined(); \
 }
 
 #define V4THROW_SQL2(error, desc) { \
-    QV4::Scoped<String> v(scope, ctx->engine->newString(desc)); \
-    QV4::Scoped<Object> ex(scope, ctx->engine->newErrorObject(v)); \
-    ex->put(QV4::ScopedString(scope, ctx->engine->newIdentifier(QStringLiteral("code"))), QV4::ScopedValue(scope, Primitive::fromInt32(error))); \
-    args->setReturnValue(ctx->throwError(ex)); \
+    QV4::ScopedString v(scope, scope.engine->newString(desc)); \
+    QV4::ScopedObject ex(scope, scope.engine->newErrorObject(v)); \
+    ex->put(QV4::ScopedString(scope, scope.engine->newIdentifier(QStringLiteral("code"))).getPointer(), QV4::ScopedValue(scope, Primitive::fromInt32(error))); \
+    args->setReturnValue(scope.engine->throwError(ex)); \
     return; \
 }
 
 #define V4THROW_REFERENCE(string) { \
-    QV4::Scoped<String> v(scope, ctx->engine->newString(string)); \
-    ctx->throwReferenceError(v); \
+    QV4::ScopedString v(scope, scope.engine->newString(string)); \
+    ctx->engine()->throwReferenceError(v); \
     return Encode::undefined(); \
 }
 
@@ -89,73 +95,86 @@ using namespace QV4;
 class QQmlSqlDatabaseData : public QV8Engine::Deletable
 {
 public:
-    QQmlSqlDatabaseData(QV8Engine *engine);
+    QQmlSqlDatabaseData(QV4::ExecutionEngine *engine);
     ~QQmlSqlDatabaseData();
 
-    PersistentValue databaseProto;
-    PersistentValue queryProto;
-    PersistentValue rowsProto;
+    QV4::PersistentValue databaseProto;
+    QV4::PersistentValue queryProto;
+    QV4::PersistentValue rowsProto;
 };
 
-V8_DEFINE_EXTENSION(QQmlSqlDatabaseData, databaseData)
+V4_DEFINE_EXTENSION(QQmlSqlDatabaseData, databaseData)
+
+namespace QV4 {
+
+namespace Heap {
+    struct QQmlSqlDatabaseWrapper : public Object {
+        enum Type { Database, Query, Rows };
+        QQmlSqlDatabaseWrapper()
+        {
+            type = Database;
+        }
+
+        Type type;
+        QSqlDatabase database;
+
+        QString version; // type == Database
+
+        bool inTransaction; // type == Query
+        bool readonly;   // type == Query
+
+        QSqlQuery sqlQuery; // type == Rows
+        bool forwardOnly; // type == Rows
+    };
+}
 
 class QQmlSqlDatabaseWrapper : public Object
 {
-    V4_OBJECT
-
 public:
-    enum Type { Database, Query, Rows };
-    QQmlSqlDatabaseWrapper(QV8Engine *e)
-        : Object(QV8Engine::getV4(e)), type(Database), inTransaction(false), readonly(false), forwardOnly(false)
+    V4_OBJECT2(QQmlSqlDatabaseWrapper, Object)
+    V4_NEEDS_DESTROY
+
+    static Heap::QQmlSqlDatabaseWrapper *create(QV4::ExecutionEngine *engine)
     {
-        setVTable(staticVTable());
+        return engine->memoryManager->allocObject<QQmlSqlDatabaseWrapper>();
     }
 
     ~QQmlSqlDatabaseWrapper() {
     }
 
-    static ReturnedValue getIndexed(Managed *m, uint index, bool *hasProperty);
-    static void destroy(Managed *that) {
-        static_cast<QQmlSqlDatabaseWrapper *>(that)->~QQmlSqlDatabaseWrapper();
-    }
-
-    Type type;
-    QSqlDatabase database;
-
-    QString version; // type == Database
-
-    bool inTransaction; // type == Query
-    bool readonly;   // type == Query
-
-    QSqlQuery sqlQuery; // type == Rows
-    bool forwardOnly; // type == Rows
+    static ReturnedValue getIndexed(const Managed *m, uint index, bool *hasProperty);
 };
 
-DEFINE_REF(QQmlSqlDatabaseWrapper, Object);
-DEFINE_OBJECT_VTABLE(QQmlSqlDatabaseWrapper);
+}
+
+using namespace QV4;
+
+DEFINE_OBJECT_VTABLE(QV4::QQmlSqlDatabaseWrapper);
+
+
 
 static ReturnedValue qmlsqldatabase_version(CallContext *ctx)
 {
     QV4::Scope scope(ctx);
-    QV4::Scoped<QQmlSqlDatabaseWrapper> r(scope, ctx->callData->thisObject.as<QQmlSqlDatabaseWrapper>());
-    if (!r || r->type != QQmlSqlDatabaseWrapper::Database)
+    QV4::Scoped<QQmlSqlDatabaseWrapper> r(scope, ctx->thisObject().as<QQmlSqlDatabaseWrapper>());
+    if (!r || r->d()->type != Heap::QQmlSqlDatabaseWrapper::Database)
         V4THROW_REFERENCE("Not a SQLDatabase object");
 
-    return Encode(ctx->engine->newString(r->version));
+    return Encode(scope.engine->newString(r->d()->version));
 }
 
 static ReturnedValue qmlsqldatabase_rows_length(CallContext *ctx)
 {
     QV4::Scope scope(ctx);
-    QV4::Scoped<QQmlSqlDatabaseWrapper> r(scope, ctx->callData->thisObject.as<QQmlSqlDatabaseWrapper>());
-    if (!r || r->type != QQmlSqlDatabaseWrapper::Rows)
+    QV4::Scoped<QQmlSqlDatabaseWrapper> r(scope, ctx->thisObject().as<QQmlSqlDatabaseWrapper>());
+    if (!r || r->d()->type != Heap::QQmlSqlDatabaseWrapper::Rows)
         V4THROW_REFERENCE("Not a SQLDatabase::Rows object");
 
-    int s = r->sqlQuery.size();
+    int s = r->d()->sqlQuery.size();
     if (s < 0) {
         // Inefficient
-        if (r->sqlQuery.last()) {
-            s = r->sqlQuery.at() + 1;
+        if (r->d()->sqlQuery.last()) {
+            s = r->d()->sqlQuery.at() + 1;
         } else {
             s = 0;
         }
@@ -166,22 +185,22 @@ static ReturnedValue qmlsqldatabase_rows_length(CallContext *ctx)
 static ReturnedValue qmlsqldatabase_rows_forwardOnly(CallContext *ctx)
 {
     QV4::Scope scope(ctx);
-    QV4::Scoped<QQmlSqlDatabaseWrapper> r(scope, ctx->callData->thisObject.as<QQmlSqlDatabaseWrapper>());
-    if (!r || r->type != QQmlSqlDatabaseWrapper::Rows)
+    QV4::Scoped<QQmlSqlDatabaseWrapper> r(scope, ctx->thisObject().as<QQmlSqlDatabaseWrapper>());
+    if (!r || r->d()->type != Heap::QQmlSqlDatabaseWrapper::Rows)
         V4THROW_REFERENCE("Not a SQLDatabase::Rows object");
-    return Encode(r->sqlQuery.isForwardOnly());
+    return Encode(r->d()->sqlQuery.isForwardOnly());
 }
 
 static ReturnedValue qmlsqldatabase_rows_setForwardOnly(CallContext *ctx)
 {
     QV4::Scope scope(ctx);
-    QV4::Scoped<QQmlSqlDatabaseWrapper> r(scope, ctx->callData->thisObject.as<QQmlSqlDatabaseWrapper>());
-    if (!r || r->type != QQmlSqlDatabaseWrapper::Rows)
+    QV4::Scoped<QQmlSqlDatabaseWrapper> r(scope, ctx->thisObject().as<QQmlSqlDatabaseWrapper>());
+    if (!r || r->d()->type != Heap::QQmlSqlDatabaseWrapper::Rows)
         V4THROW_REFERENCE("Not a SQLDatabase::Rows object");
-    if (ctx->callData->argc < 1)
-        return ctx->throwTypeError();
+    if (ctx->argc() < 1)
+        return ctx->engine()->throwTypeError();
 
-    r->sqlQuery.setForwardOnly(ctx->callData->args[0].toBoolean());
+    r->d()->sqlQuery.setForwardOnly(ctx->args()[0].toBoolean());
     return Encode::undefined();
 }
 
@@ -189,38 +208,37 @@ QQmlSqlDatabaseData::~QQmlSqlDatabaseData()
 {
 }
 
-static QString qmlsqldatabase_databasesPath(QV8Engine *engine)
+static QString qmlsqldatabase_databasesPath(QV4::ExecutionEngine *engine)
 {
-    return engine->engine()->offlineStoragePath() +
+    return engine->qmlEngine()->offlineStoragePath() +
            QDir::separator() + QLatin1String("Databases");
 }
 
-static void qmlsqldatabase_initDatabasesPath(QV8Engine *engine)
+static void qmlsqldatabase_initDatabasesPath(QV4::ExecutionEngine *engine)
 {
     QString databasesPath = qmlsqldatabase_databasesPath(engine);
     if (!QDir().mkpath(databasesPath))
         qWarning() << "LocalStorage: can't create path - " << databasesPath;
 }
 
-static QString qmlsqldatabase_databaseFile(const QString& connectionName, QV8Engine *engine)
+static QString qmlsqldatabase_databaseFile(const QString& connectionName, QV4::ExecutionEngine *engine)
 {
     return qmlsqldatabase_databasesPath(engine) + QDir::separator() + connectionName;
 }
 
-static ReturnedValue qmlsqldatabase_rows_index(QQmlSqlDatabaseWrapperRef r, ExecutionEngine *v4, quint32 index, bool *hasProperty = 0)
+static ReturnedValue qmlsqldatabase_rows_index(const QQmlSqlDatabaseWrapper *r, ExecutionEngine *v4, quint32 index, bool *hasProperty = 0)
 {
     Scope scope(v4);
-    QV8Engine *v8 = v4->v8Engine;
 
-    if (r->sqlQuery.at() == (int)index || r->sqlQuery.seek(index)) {
-        QSqlRecord record = r->sqlQuery.record();
+    if (r->d()->sqlQuery.at() == (int)index || r->d()->sqlQuery.seek(index)) {
+        QSqlRecord record = r->d()->sqlQuery.record();
         // XXX optimize
-        Scoped<Object> row(scope, v4->newObject());
+        ScopedObject row(scope, v4->newObject());
         for (int ii = 0; ii < record.count(); ++ii) {
             QVariant v = record.value(ii);
             ScopedString s(scope, v4->newIdentifier(record.fieldName(ii)));
-            ScopedValue val(scope, v.isNull() ? Encode::null() : v8->fromVariant(v));
-            row->put(s, val);
+            ScopedValue val(scope, v.isNull() ? Encode::null() : v4->fromVariant(v));
+            row->put(s.getPointer(), val);
         }
         if (hasProperty)
             *hasProperty = true;
@@ -232,43 +250,50 @@ static ReturnedValue qmlsqldatabase_rows_index(QQmlSqlDatabaseWrapperRef r, Exec
     }
 }
 
-ReturnedValue QQmlSqlDatabaseWrapper::getIndexed(Managed *m, uint index, bool *hasProperty)
+ReturnedValue QQmlSqlDatabaseWrapper::getIndexed(const Managed *m, uint index, bool *hasProperty)
 {
-    QV4::Scope scope(m->engine());
-    QV4::Scoped<QQmlSqlDatabaseWrapper> r(scope, m->as<QQmlSqlDatabaseWrapper>());
-    if (!r || r->type != QQmlSqlDatabaseWrapper::Rows)
+    Q_ASSERT(m->as<QQmlSqlDatabaseWrapper>());
+    const QQmlSqlDatabaseWrapper *r = static_cast<const QQmlSqlDatabaseWrapper *>(m);
+    if (!r || r->d()->type != Heap::QQmlSqlDatabaseWrapper::Rows)
         return Object::getIndexed(m, index, hasProperty);
 
-    return qmlsqldatabase_rows_index(r, m->engine(), index, hasProperty);
+    return qmlsqldatabase_rows_index(r, r->engine(), index, hasProperty);
 }
 
 static ReturnedValue qmlsqldatabase_rows_item(CallContext *ctx)
 {
     QV4::Scope scope(ctx);
-    QV4::Scoped<QQmlSqlDatabaseWrapper> r(scope, ctx->callData->thisObject.as<QQmlSqlDatabaseWrapper>());
-    if (!r || r->type != QQmlSqlDatabaseWrapper::Rows)
+    QV4::Scoped<QQmlSqlDatabaseWrapper> r(scope, ctx->thisObject().as<QQmlSqlDatabaseWrapper>());
+    if (!r || r->d()->type != Heap::QQmlSqlDatabaseWrapper::Rows)
         V4THROW_REFERENCE("Not a SQLDatabase::Rows object");
 
-    return qmlsqldatabase_rows_index(r, ctx->engine, ctx->callData->argc ? ctx->callData->args[0].toUInt32() : 0);
+    return qmlsqldatabase_rows_index(r, scope.engine, ctx->argc() ? ctx->args()[0].toUInt32() : 0);
+}
+
+static QVariant toSqlVariant(QV4::ExecutionEngine *engine, const QV4::ScopedValue &value)
+{
+    // toVariant() maps a null JS value to QVariant(VoidStar), but the SQL module
+    // expects a null variant. (this is because of QTBUG-40880)
+    if (value->isNull())
+        return QVariant();
+    return engine->toVariant(value, /*typehint*/-1);
 }
 
 static ReturnedValue qmlsqldatabase_executeSql(CallContext *ctx)
 {
     QV4::Scope scope(ctx);
-    QV4::Scoped<QQmlSqlDatabaseWrapper> r(scope, ctx->callData->thisObject.as<QQmlSqlDatabaseWrapper>());
-    if (!r || r->type != QQmlSqlDatabaseWrapper::Query)
+    QV4::Scoped<QQmlSqlDatabaseWrapper> r(scope, ctx->thisObject().as<QQmlSqlDatabaseWrapper>());
+    if (!r || r->d()->type != Heap::QQmlSqlDatabaseWrapper::Query)
         V4THROW_REFERENCE("Not a SQLDatabase::Query object");
 
-    QV8Engine *engine = ctx->engine->v8Engine;
-
-    if (!r->inTransaction)
+    if (!r->d()->inTransaction)
         V4THROW_SQL(SQLEXCEPTION_DATABASE_ERR,QQmlEngine::tr("executeSql called outside transaction()"));
 
-    QSqlDatabase db = r->database;
+    QSqlDatabase db = r->d()->database;
 
-    QString sql = ctx->callData->argc ? ctx->callData->args[0].toQString() : QString();
+    QString sql = ctx->argc() ? ctx->args()[0].toQString() : QString();
 
-    if (r->readonly && !sql.startsWith(QLatin1String("SELECT"),Qt::CaseInsensitive)) {
+    if (r->d()->readonly && !sql.startsWith(QLatin1String("SELECT"),Qt::CaseInsensitive)) {
         V4THROW_SQL(SQLEXCEPTION_SYNTAX_ERR, QQmlEngine::tr("Read-only Transaction"));
     }
 
@@ -278,15 +303,16 @@ static ReturnedValue qmlsqldatabase_executeSql(CallContext *ctx)
     ScopedValue result(scope, Primitive::undefinedValue());
 
     if (query.prepare(sql)) {
-        if (ctx->callData->argc > 1) {
-            ScopedValue values(scope, ctx->callData->args[1]);
-            if (values->asArrayObject()) {
+        if (ctx->argc() > 1) {
+            ScopedValue values(scope, ctx->args()[1]);
+            if (values->as<ArrayObject>()) {
                 ScopedArrayObject array(scope, values);
                 quint32 size = array->getLength();
                 QV4::ScopedValue v(scope);
-                for (quint32 ii = 0; ii < size; ++ii)
-                    query.bindValue(ii, engine->toVariant((v = array->getIndexed(ii)), -1));
-            } else if (values->asObject()) {
+                for (quint32 ii = 0; ii < size; ++ii) {
+                    query.bindValue(ii, toSqlVariant(scope.engine, (v = array->getIndexed(ii))));
+                }
+            } else if (values->as<Object>()) {
                 ScopedObject object(scope, values);
                 ObjectIterator it(scope, object, ObjectIterator::WithProtoChain|ObjectIterator::EnumerableOnly);
                 ScopedValue key(scope);
@@ -295,34 +321,34 @@ static ReturnedValue qmlsqldatabase_executeSql(CallContext *ctx)
                     key = it.nextPropertyName(val);
                     if (key->isNull())
                         break;
-                    QVariant v = engine->toVariant(val, -1);
+                    QVariant v = toSqlVariant(scope.engine, val);
                     if (key->isString()) {
                         query.bindValue(key->stringValue()->toQString(), v);
                     } else {
-                        assert(key->isInteger());
+                        Q_ASSERT(key->isInteger());
                         query.bindValue(key->integerValue(), v);
                     }
                 }
             } else {
-                query.bindValue(0, engine->toVariant(values, -1));
+                query.bindValue(0, toSqlVariant(scope.engine, values));
             }
         }
         if (query.exec()) {
-            QV4::Scoped<QQmlSqlDatabaseWrapper> rows(scope, new (ctx->engine->memoryManager) QQmlSqlDatabaseWrapper(engine));
-            QV4::ScopedObject p(scope, databaseData(engine)->rowsProto.value());
+            QV4::Scoped<QQmlSqlDatabaseWrapper> rows(scope, QQmlSqlDatabaseWrapper::create(scope.engine));
+            QV4::ScopedObject p(scope, databaseData(scope.engine)->rowsProto.value());
             rows->setPrototype(p.getPointer());
-            rows->type = QQmlSqlDatabaseWrapper::Rows;
-            rows->database = db;
-            rows->sqlQuery = query;
+            rows->d()->type = Heap::QQmlSqlDatabaseWrapper::Rows;
+            rows->d()->database = db;
+            rows->d()->sqlQuery = query;
 
-            Scoped<Object> resultObject(scope, ctx->engine->newObject());
+            ScopedObject resultObject(scope, scope.engine->newObject());
             result = resultObject.asReturnedValue();
             // XXX optimize
             ScopedString s(scope);
             ScopedValue v(scope);
-            resultObject->put((s = ctx->engine->newIdentifier("rowsAffected")), (v = Primitive::fromInt32(query.numRowsAffected())));
-            resultObject->put((s = ctx->engine->newIdentifier("insertId")), (v = engine->toString(query.lastInsertId().toString())));
-            resultObject->put((s = ctx->engine->newIdentifier("rows")), rows);
+            resultObject->put((s = scope.engine->newIdentifier("rowsAffected")).getPointer(), (v = Primitive::fromInt32(query.numRowsAffected())));
+            resultObject->put((s = scope.engine->newIdentifier("insertId")).getPointer(), (v = scope.engine->newString(query.lastInsertId().toString())));
+            resultObject->put((s = scope.engine->newIdentifier("rows")).getPointer(), rows);
         } else {
             err = true;
         }
@@ -332,7 +358,7 @@ static ReturnedValue qmlsqldatabase_executeSql(CallContext *ctx)
     if (err)
         V4THROW_SQL(SQLEXCEPTION_DATABASE_ERR,query.lastError().text());
 
-    return result.asReturnedValue();
+    return result->asReturnedValue();
 }
 
 struct TransactionRollback {
@@ -366,31 +392,29 @@ struct TransactionRollback {
 
 static ReturnedValue qmlsqldatabase_changeVersion(CallContext *ctx)
 {
-    if (ctx->callData->argc < 2)
+    if (ctx->argc() < 2)
         return Encode::undefined();
 
     Scope scope(ctx);
 
-    Scoped<QQmlSqlDatabaseWrapper> r(scope, ctx->callData->thisObject);
-    if (!r || r->type != QQmlSqlDatabaseWrapper::Database)
+    Scoped<QQmlSqlDatabaseWrapper> r(scope, ctx->thisObject());
+    if (!r || r->d()->type != Heap::QQmlSqlDatabaseWrapper::Database)
         V4THROW_REFERENCE("Not a SQLDatabase object");
 
-    QV8Engine *engine = ctx->engine->v8Engine;
+    QSqlDatabase db = r->d()->database;
+    QString from_version = ctx->args()[0].toQString();
+    QString to_version = ctx->args()[1].toQString();
+    ScopedFunctionObject callback(scope, ctx->argument(2));
 
-    QSqlDatabase db = r->database;
-    QString from_version = ctx->callData->args[0].toQString();
-    QString to_version = ctx->callData->args[1].toQString();
-    Scoped<FunctionObject> callback(scope, ctx->argument(2));
+    if (from_version != r->d()->version)
+        V4THROW_SQL(SQLEXCEPTION_VERSION_ERR, QQmlEngine::tr("Version mismatch: expected %1, found %2").arg(from_version).arg(r->d()->version));
 
-    if (from_version != r->version)
-        V4THROW_SQL(SQLEXCEPTION_VERSION_ERR, QQmlEngine::tr("Version mismatch: expected %1, found %2").arg(from_version).arg(r->version));
-
-    Scoped<QQmlSqlDatabaseWrapper> w(scope, new (ctx->engine->memoryManager) QQmlSqlDatabaseWrapper(engine));
-    ScopedObject p(scope, databaseData(engine)->queryProto.value());
+    Scoped<QQmlSqlDatabaseWrapper> w(scope, QQmlSqlDatabaseWrapper::create(scope.engine));
+    ScopedObject p(scope, databaseData(scope.engine)->queryProto.value());
     w->setPrototype(p.getPointer());
-    w->type = QQmlSqlDatabaseWrapper::Query;
-    w->database = db;
-    w->version = r->version;
+    w->d()->type = Heap::QQmlSqlDatabaseWrapper::Query;
+    w->d()->database = db;
+    w->d()->version = r->d()->version;
 
     bool ok = true;
     if (!!callback) {
@@ -398,10 +422,10 @@ static ReturnedValue qmlsqldatabase_changeVersion(CallContext *ctx)
         db.transaction();
 
         ScopedCallData callData(scope, 1);
-        callData->thisObject = engine->global();
+        callData->thisObject = scope.engine->globalObject;
         callData->args[0] = w;
 
-        TransactionRollback rollbackOnException(&db, &w->inTransaction);
+        TransactionRollback rollbackOnException(&db, &w->d()->inTransaction);
         callback->call(callData);
         rollbackOnException.clear();
         if (!db.commit()) {
@@ -413,9 +437,9 @@ static ReturnedValue qmlsqldatabase_changeVersion(CallContext *ctx)
     }
 
     if (ok) {
-        w->version = to_version;
+        w->d()->version = to_version;
 #ifndef QT_NO_SETTINGS
-        QSettings ini(qmlsqldatabase_databaseFile(db.connectionName(),engine) + QLatin1String(".ini"), QSettings::IniFormat);
+        QSettings ini(qmlsqldatabase_databaseFile(db.connectionName(), scope.engine) + QLatin1String(".ini"), QSettings::IniFormat);
         ini.setValue(QLatin1String("Version"), to_version);
 #endif
     }
@@ -426,32 +450,30 @@ static ReturnedValue qmlsqldatabase_changeVersion(CallContext *ctx)
 static ReturnedValue qmlsqldatabase_transaction_shared(CallContext *ctx, bool readOnly)
 {
     QV4::Scope scope(ctx);
-    QV4::Scoped<QQmlSqlDatabaseWrapper> r(scope, ctx->callData->thisObject.as<QQmlSqlDatabaseWrapper>());
-    if (!r || r->type != QQmlSqlDatabaseWrapper::Database)
+    QV4::Scoped<QQmlSqlDatabaseWrapper> r(scope, ctx->thisObject().as<QQmlSqlDatabaseWrapper>());
+    if (!r || r->d()->type != Heap::QQmlSqlDatabaseWrapper::Database)
         V4THROW_REFERENCE("Not a SQLDatabase object");
 
-    QV8Engine *engine = ctx->engine->v8Engine;
-
-    FunctionObject *callback = ctx->callData->argc ? ctx->callData->args[0].asFunctionObject() : 0;
+    const FunctionObject *callback = ctx->argc() ? ctx->args()[0].as<FunctionObject>() : 0;
     if (!callback)
         V4THROW_SQL(SQLEXCEPTION_UNKNOWN_ERR, QQmlEngine::tr("transaction: missing callback"));
 
-    QSqlDatabase db = r->database;
+    QSqlDatabase db = r->d()->database;
 
-    Scoped<QQmlSqlDatabaseWrapper> w(scope, new (ctx->engine->memoryManager) QQmlSqlDatabaseWrapper(engine));
-    QV4::ScopedObject p(scope, databaseData(engine)->queryProto.value());
+    Scoped<QQmlSqlDatabaseWrapper> w(scope, QQmlSqlDatabaseWrapper::create(scope.engine));
+    QV4::ScopedObject p(scope, databaseData(scope.engine)->queryProto.value());
     w->setPrototype(p.getPointer());
-    w->type = QQmlSqlDatabaseWrapper::Query;
-    w->database = db;
-    w->version = r->version;
-    w->readonly = readOnly;
+    w->d()->type = Heap::QQmlSqlDatabaseWrapper::Query;
+    w->d()->database = db;
+    w->d()->version = r->d()->version;
+    w->d()->readonly = readOnly;
 
     db.transaction();
     if (callback) {
         ScopedCallData callData(scope, 1);
-        callData->thisObject = engine->global();
+        callData->thisObject = scope.engine->globalObject;
         callData->args[0] = w;
-        TransactionRollback rollbackOnException(&db, &w->inTransaction);
+        TransactionRollback rollbackOnException(&db, &w->d()->inTransaction);
         callback->call(callData);
         rollbackOnException.clear();
 
@@ -472,12 +494,11 @@ static ReturnedValue qmlsqldatabase_read_transaction(CallContext *ctx)
     return qmlsqldatabase_transaction_shared(ctx, true);
 }
 
-QQmlSqlDatabaseData::QQmlSqlDatabaseData(QV8Engine *engine)
+QQmlSqlDatabaseData::QQmlSqlDatabaseData(ExecutionEngine *v4)
 {
-    ExecutionEngine *v4 = QV8Engine::getV4(engine);
     Scope scope(v4);
     {
-        Scoped<Object> proto(scope, v4->newObject());
+        ScopedObject proto(scope, v4->newObject());
         proto->defineDefaultProperty(QStringLiteral("transaction"), qmlsqldatabase_transaction);
         proto->defineDefaultProperty(QStringLiteral("readTransaction"), qmlsqldatabase_read_transaction);
         proto->defineAccessorProperty(QStringLiteral("version"), qmlsqldatabase_version, 0);
@@ -486,12 +507,12 @@ QQmlSqlDatabaseData::QQmlSqlDatabaseData(QV8Engine *engine)
     }
 
     {
-        Scoped<Object> proto(scope, v4->newObject());
+        ScopedObject proto(scope, v4->newObject());
         proto->defineDefaultProperty(QStringLiteral("executeSql"), qmlsqldatabase_executeSql);
         queryProto = proto;
     }
     {
-        Scoped<Object> proto(scope, v4->newObject());
+        ScopedObject proto(scope, v4->newObject());
         proto->defineDefaultProperty(QStringLiteral("item"), qmlsqldatabase_rows_item);
         proto->defineAccessorProperty(QStringLiteral("length"), qmlsqldatabase_rows_length, 0);
         proto->defineAccessorProperty(QStringLiteral("forwardOnly"),
@@ -564,9 +585,9 @@ import QtQuick.LocalStorage 2.0 as Sql
 db = Sql.openDatabaseSync(identifier, version, description, estimated_size, callback(db))
 \endcode
 The above code returns the database identified by \e identifier. If the database does not already exist, it
-is created, and the function \e callback is called with the database as a parameter. \e description
-and \e estimated_size are written to the INI file (described below), but are otherwise currently
-unused.
+is created, and the function \e callback is called with the database as a parameter. \e identifier is the
+name of the physical file (with or without full path) containing the database.  \e description and
+\e estimated_size are written to the INI file (described below), but are currently unused.
 
 May throw exception with code property SQLException.DATABASE_ERR, or SQLException.VERSION_ERR.
 
@@ -574,7 +595,7 @@ When a database is first created, an INI file is also created specifying its cha
 
 \table
 \header \li \b {Key} \li \b {Value}
-\row \li Name \li The name of the database passed to \c openDatabase()
+\row \li Identifier \li The name of the database passed to \c openDatabase()
 \row \li Version \li The version of the database passed to \c openDatabase()
 \row \li Description \li The description of the database passed to \c openDatabase()
 \row \li EstimatedSize \li The estimated size (in bytes) of the database passed to \c openDatabase()
@@ -594,12 +615,19 @@ you can call \e executeSql on \e tx to upgrade the database.
 
 May throw exception with code property SQLException.DATABASE_ERR or SQLException.UNKNOWN_ERR.
 
+See example below.
+
+\snippet qml/localstorage/dbtransaction.js 2
+
 \section3 db.transaction(callback(tx))
 
 This method creates a read/write transaction and passed to \e callback. In this function,
 you can call \e executeSql on \e tx to read and modify the database.
 
 If the callback throws exceptions, the transaction is rolled back.
+Below you will find an example of a database transaction which catches exceptions.
+
+\snippet qml/localstorage/dbtransaction.js 0
 
 \section3 db.readTransaction(callback(tx))
 
@@ -622,6 +650,9 @@ It returns a results object, with the following properties:
 
 May throw exception with code property SQLException.DATABASE_ERR, SQLException.SYNTAX_ERR, or SQLException.UNKNOWN_ERR.
 
+See below for an example:
+
+\snippet qml/localstorage/dbtransaction.js 1
 
 \section1 Method Documentation
 
@@ -659,13 +690,11 @@ public:
 void QQuickLocalStorage::openDatabaseSync(QQmlV4Function *args)
 {
 #ifndef QT_NO_SETTINGS
-    QV8Engine *engine = args->engine();
-    QV4::ExecutionContext *ctx = args->v4engine()->currentContext();
-    QV4::Scope scope(ctx);
-    if (engine->engine()->offlineStoragePath().isEmpty())
+    QV4::Scope scope(args->v4engine());
+    if (scope.engine->qmlEngine()->offlineStoragePath().isEmpty())
         V4THROW_SQL2(SQLEXCEPTION_DATABASE_ERR, QQmlEngine::tr("SQL: can't create database, offline storage is disabled."));
 
-    qmlsqldatabase_initDatabasesPath(engine);
+    qmlsqldatabase_initDatabasesPath(scope.engine);
 
     QSqlDatabase database;
 
@@ -674,13 +703,13 @@ void QQuickLocalStorage::openDatabaseSync(QQmlV4Function *args)
     QString dbversion = (v = (*args)[1])->toQStringNoThrow();
     QString dbdescription = (v = (*args)[2])->toQStringNoThrow();
     int dbestimatedsize = (v = (*args)[3])->toInt32();
-    FunctionObject *dbcreationCallback = (v = (*args)[4])->asFunctionObject();
+    FunctionObject *dbcreationCallback = (v = (*args)[4])->as<FunctionObject>();
 
     QCryptographicHash md5(QCryptographicHash::Md5);
     md5.addData(dbname.toUtf8());
     QString dbid(QLatin1String(md5.result().toHex()));
 
-    QString basename = qmlsqldatabase_databaseFile(dbid, engine);
+    QString basename = qmlsqldatabase_databaseFile(dbid, scope.engine);
     bool created = false;
     QString version = dbversion;
 
@@ -716,16 +745,15 @@ void QQuickLocalStorage::openDatabaseSync(QQmlV4Function *args)
             database.open();
     }
 
-    QV4::Scoped<QQmlSqlDatabaseWrapper> db(scope, new (ctx->engine->memoryManager) QQmlSqlDatabaseWrapper(engine));
-    QV4::ScopedObject p(scope, databaseData(engine)->databaseProto.value());
+    QV4::Scoped<QQmlSqlDatabaseWrapper> db(scope, QQmlSqlDatabaseWrapper::create(scope.engine));
+    QV4::ScopedObject p(scope, databaseData(scope.engine)->databaseProto.value());
     db->setPrototype(p.getPointer());
-    db->database = database;
-    db->version = version;
+    db->d()->database = database;
+    db->d()->version = version;
 
     if (created && dbcreationCallback) {
-        Scope scope(ctx);
         ScopedCallData callData(scope, 1);
-        callData->thisObject = engine->global();
+        callData->thisObject = scope.engine->globalObject;
         callData->args[0] = db;
         dbcreationCallback->call(callData);
     }
@@ -746,13 +774,13 @@ static QObject *module_api_factory(QQmlEngine *engine, QJSEngine *scriptEngine)
 class QQmlLocalStoragePlugin : public QQmlExtensionPlugin
 {
     Q_OBJECT
-    Q_PLUGIN_METADATA(IID "org.qt-project.Qt.QQmlExtensionInterface")
+    Q_PLUGIN_METADATA(IID QQmlExtensionInterface_iid)
 
 public:
-    QQmlLocalStoragePlugin()
+    QQmlLocalStoragePlugin(QObject *parent = 0) : QQmlExtensionPlugin(parent)
     {
+        initResources();
     }
-
     void registerTypes(const char *uri)
     {
         Q_ASSERT(QLatin1String(uri) == "QtQuick.LocalStorage");
@@ -760,4 +788,6 @@ public:
     }
 };
 
+QT_END_NAMESPACE
+
 #include "plugin.moc"
