@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -231,6 +237,8 @@ QQuickParticleEmitter::QQuickParticleEmitter(QQuickItem *parent) :
   , m_reset_last(true)
   , m_last_timestamp(-1)
   , m_last_emission(0)
+  , m_groupIdNeedRecalculation(false)
+  , m_groupId(QQuickParticleGroupData::DefaultGroupID)
 
 {
     //TODO: Reset velocity/acc back to null vector? Or allow null pointer?
@@ -251,10 +259,22 @@ bool QQuickParticleEmitter::isEmitConnected()
     IS_SIGNAL_CONNECTED(this, QQuickParticleEmitter, emitParticles, (QQmlV4Handle));
 }
 
+void QQuickParticleEmitter::reclaculateGroupId() const
+{
+    if (!m_system) {
+        m_groupId = QQuickParticleGroupData::InvalidID;
+        return;
+    }
+    m_groupId = m_system->groupIds.value(group(), QQuickParticleGroupData::InvalidID);
+    m_groupIdNeedRecalculation = m_groupId == QQuickParticleGroupData::InvalidID;
+}
+
 void QQuickParticleEmitter::componentComplete()
 {
     if (!m_system && qobject_cast<QQuickParticleSystem*>(parentItem()))
         setSystem(qobject_cast<QQuickParticleSystem*>(parentItem()));
+    if (m_system)
+        m_system->finishRegisteringParticleEmitter(this);
     QQuickItem::componentComplete();
 }
 
@@ -313,13 +333,6 @@ void QQuickParticleEmitter::setMaxParticleCount(int arg)
     }
 }
 
-int QQuickParticleEmitter::particleCount() const
-{
-    if (m_maxParticleCount >= 0)
-        return m_maxParticleCount;
-    return m_particlesPerSecond*((m_particleDuration+m_particleDurationVariation)/1000.0);
-}
-
 void QQuickParticleEmitter::setVelocityFromMovement(qreal t)
 {
     if (t == m_velocity_from_movement)
@@ -396,7 +409,6 @@ void QQuickParticleEmitter::emitWindow(int timeStamp)
         //int pos = m_last_particle % m_particle_count;
         QQuickParticleData* datum = m_system->newDatum(m_system->groupIds[m_group], !m_overwrite);
         if (datum){//actually emit(otherwise we've been asked to skip this one)
-            datum->e = this;//###useful?
             qreal t = 1 - (pt - opt) / dt;
             qreal vx =
               - 2 * ax * (1 - t)
@@ -469,7 +481,7 @@ void QQuickParticleEmitter::emitWindow(int timeStamp)
     }
 
     foreach (QQuickParticleData* d, toEmit)
-            m_system->emitParticle(d);
+            m_system->emitParticle(d, this);
 
     if (isEmitConnected()) {
         QQmlEngine *qmlEngine = ::qmlEngine(this);
@@ -481,7 +493,7 @@ void QQuickParticleEmitter::emitWindow(int timeStamp)
         QV4::ScopedArrayObject array(scope, v4->newArrayObject(toEmit.size()));
         QV4::ScopedValue v(scope);
         for (int i=0; i<toEmit.size(); i++)
-            array->putIndexed(i, (v = toEmit[i]->v4Value()));
+            array->putIndexed(i, (v = toEmit[i]->v4Value(m_system)));
 
         emitParticles(QQmlV4Handle(array));//A chance for arbitrary JS changes
     }
