@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -58,9 +64,13 @@ QT_BEGIN_NAMESPACE
 
     If empty, it will paint the default particle group ("").
 */
-QQuickParticlePainter::QQuickParticlePainter(QQuickItem *parent) :
-    QQuickItem(parent),
-    m_system(0), m_count(0), m_pleaseReset(true), m_window(0)
+QQuickParticlePainter::QQuickParticlePainter(QQuickItem *parent)
+    : QQuickItem(parent)
+    , m_system(0)
+    , m_count(0)
+    , m_pleaseReset(true)
+    , m_window(0)
+    , m_groupIdsNeedRecalculation(false)
 {
 }
 
@@ -83,11 +93,32 @@ void QQuickParticlePainter::componentComplete()
     QQuickItem::componentComplete();
 }
 
+void QQuickParticlePainter::recalculateGroupIds() const
+{
+    if (!m_system) {
+        m_groupIds.clear();
+        return;
+    }
+
+    m_groupIdsNeedRecalculation = false;
+    m_groupIds.clear();
+
+    for (const QString &str : groups()) {
+        QQuickParticleGroupData::ID groupId = m_system->groupIds.value(str, QQuickParticleGroupData::InvalidID);
+        if (groupId == QQuickParticleGroupData::InvalidID) {
+            // invalid data, not finished setting up, or whatever. Fallback: do not cache.
+            m_groupIdsNeedRecalculation = true;
+        } else {
+            m_groupIds.append(groupId);
+        }
+    }
+}
 
 void QQuickParticlePainter::setSystem(QQuickParticleSystem *arg)
 {
     if (m_system != arg) {
         m_system = arg;
+        m_groupIdsNeedRecalculation = true;
         if (m_system){
             m_system->registerParticlePainter(this);
             reset();
@@ -96,19 +127,29 @@ void QQuickParticlePainter::setSystem(QQuickParticleSystem *arg)
     }
 }
 
+void QQuickParticlePainter::setGroups(const QStringList &arg)
+{
+    if (m_groups != arg) {
+        m_groups = arg;
+        m_groupIdsNeedRecalculation = true;
+        //Note: The system watches this as it has to recalc things when groups change. It will request a reset if necessary
+        Q_EMIT groupsChanged(arg);
+    }
+}
+
 void QQuickParticlePainter::load(QQuickParticleData* d)
 {
-    initialize(d->group, d->index);
+    initialize(d->groupId, d->index);
     if (m_pleaseReset)
         return;
-    m_pendingCommits << qMakePair<int, int>(d->group, d->index);
+    m_pendingCommits << qMakePair<int, int>(d->groupId, d->index);
 }
 
 void QQuickParticlePainter::reload(QQuickParticleData* d)
 {
     if (m_pleaseReset)
         return;
-    m_pendingCommits << qMakePair<int, int>(d->group, d->index);
+    m_pendingCommits << qMakePair<int, int>(d->groupId, d->index);
 }
 
 void QQuickParticlePainter::reset()
@@ -127,11 +168,6 @@ void QQuickParticlePainter::setCount(int c)//### TODO: some resizeing so that pa
     reset();
 }
 
-int QQuickParticlePainter::count()
-{
-    return m_count;
-}
-
 void QQuickParticlePainter::calcSystemOffset(bool resetPending)
 {
     if (!m_system || !parentItem())
