@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
@@ -10,30 +10,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -45,12 +43,13 @@
 #include <QDebug>
 #include <private/qv4engine_p.h>
 #include <private/qv4functionobject_p.h>
+#include <QtCore/private/qnumeric_p.h>
 
 QT_BEGIN_NAMESPACE
 
 /*!
     \qmltype Particle
-    \inqmlmodule QtQuick.Particles 2
+    \inqmlmodule QtQuick.Particles
     \brief Represents particles manipulated by emitters and affectors
     \ingroup qtquick-particles
 
@@ -273,27 +272,25 @@ QT_BEGIN_NAMESPACE
 //### Particle data handles are not locked to within certain scopes like QQuickContext2D, but there's no way to reload either...
 struct QV4ParticleData : public QV4::Object
 {
-    V4_OBJECT
-    QV4ParticleData(QV4::ExecutionEngine *engine, QQuickParticleData *datum)
-        : Object(engine)
-    {
-        setVTable(staticVTable());
-        this->datum = datum;
-    }
-
-    QQuickParticleData* datum;//TODO: Guard needed?
-
-    static void destroy(Managed *that)
-    { that->as<QV4ParticleData>()->~QV4ParticleData(); }
+    struct Data : QV4::Object::Data {
+        Data(QQuickParticleData *datum, QQuickParticleSystem* particleSystem)
+            : datum(datum)
+            , particleSystem(particleSystem)
+        {
+        }
+        QQuickParticleData* datum;//TODO: Guard needed?
+        QQuickParticleSystem* particleSystem;
+    };
+    V4_OBJECT(QV4::Object)
 };
 
 DEFINE_OBJECT_VTABLE(QV4ParticleData);
 
-class QV8ParticleDataDeletable : public QV8Engine::Deletable
+class QV4ParticleDataDeletable : public QV8Engine::Deletable
 {
 public:
-    QV8ParticleDataDeletable(QV8Engine *engine);
-    ~QV8ParticleDataDeletable();
+    QV4ParticleDataDeletable(QV4::ExecutionEngine *engine);
+    ~QV4ParticleDataDeletable();
 
     QV4::PersistentValue proto;
 };
@@ -301,55 +298,55 @@ public:
 static QV4::ReturnedValue particleData_discard(QV4::CallContext *ctx)
 {
     QV4::Scope scope(ctx);
-    QV4::Scoped<QV4ParticleData> r(scope, ctx->callData->thisObject);
+    QV4::Scoped<QV4ParticleData> r(scope, ctx->thisObject());
 
-    if (!r || !r->datum)
-        return ctx->throwError(QStringLiteral("Not a valid ParticleData object"));
+    if (!r || !r->d()->datum)
+        return ctx->engine()->throwError(QStringLiteral("Not a valid ParticleData object"));
 
-    r->datum->lifeSpan = 0; //Don't kill(), because it could still be in the middle of being created
+    r->d()->datum->lifeSpan = 0; //Don't kill(), because it could still be in the middle of being created
     return QV4::Encode::undefined();
 }
 
 static QV4::ReturnedValue particleData_lifeLeft(QV4::CallContext *ctx)
 {
     QV4::Scope scope(ctx);
-    QV4::Scoped<QV4ParticleData> r(scope, ctx->callData->thisObject);
+    QV4::Scoped<QV4ParticleData> r(scope, ctx->thisObject());
 
-    if (!r || !r->datum)
-        return ctx->throwError(QStringLiteral("Not a valid ParticleData object"));
+    if (!r || !r->d()->datum)
+        return ctx->engine()->throwError(QStringLiteral("Not a valid ParticleData object"));
 
-    return QV4::Encode(r->datum->lifeLeft());
+    return QV4::Encode(r->d()->datum->lifeLeft(r->d()->particleSystem));
 }
 
 static QV4::ReturnedValue particleData_curSize(QV4::CallContext *ctx)
 {
     QV4::Scope scope(ctx);
-    QV4::Scoped<QV4ParticleData> r(scope, ctx->callData->thisObject);
+    QV4::Scoped<QV4ParticleData> r(scope, ctx->thisObject());
 
-    if (!r || !r->datum)
-        return ctx->throwError(QStringLiteral("Not a valid ParticleData object"));
+    if (!r || !r->d()->datum)
+        return ctx->engine()->throwError(QStringLiteral("Not a valid ParticleData object"));
 
-    return QV4::Encode(r->datum->curSize());
+    return QV4::Encode(r->d()->datum->curSize(r->d()->particleSystem));
 }
 #define COLOR_GETTER_AND_SETTER(VAR, NAME) static QV4::ReturnedValue particleData_get_ ## NAME (QV4::CallContext *ctx) \
 { \
     QV4::Scope scope(ctx); \
-    QV4::Scoped<QV4ParticleData> r(scope, ctx->callData->thisObject); \
-    if (!r || !r->datum) \
-        ctx->throwError(QStringLiteral("Not a valid ParticleData object")); \
+    QV4::Scoped<QV4ParticleData> r(scope, ctx->thisObject()); \
+    if (!r || !r->d()->datum) \
+        ctx->engine()->throwError(QStringLiteral("Not a valid ParticleData object")); \
 \
-    return QV4::Encode((r->datum->color. VAR )/255.0);\
+    return QV4::Encode((r->d()->datum->color. VAR )/255.0);\
 }\
 \
 static QV4::ReturnedValue particleData_set_ ## NAME (QV4::CallContext *ctx)\
 {\
     QV4::Scope scope(ctx); \
-    QV4::Scoped<QV4ParticleData> r(scope, ctx->callData->thisObject); \
-    if (!r || !r->datum)\
-        ctx->throwError(QStringLiteral("Not a valid ParticleData object"));\
+    QV4::Scoped<QV4ParticleData> r(scope, ctx->thisObject()); \
+    if (!r || !r->d()->datum)\
+        ctx->engine()->throwError(QStringLiteral("Not a valid ParticleData object"));\
 \
-    double d = ctx->callData->argc ? ctx->callData->args[0].toNumber() : 0; \
-    r->datum->color. VAR = qMin(255, qMax(0, (int)floor(d * 255.0)));\
+    double d = ctx->argc() ? ctx->args()[0].toNumber() : 0; \
+    r->d()->datum->color. VAR = qMin(255, qMax(0, (int)::floor(d * 255.0)));\
     return QV4::Encode::undefined(); \
 }
 
@@ -357,63 +354,63 @@ static QV4::ReturnedValue particleData_set_ ## NAME (QV4::CallContext *ctx)\
 #define SEMIBOOL_GETTER_AND_SETTER(VARIABLE) static QV4::ReturnedValue particleData_get_ ## VARIABLE (QV4::CallContext *ctx) \
 { \
     QV4::Scope scope(ctx); \
-    QV4::Scoped<QV4ParticleData> r(scope, ctx->callData->thisObject); \
-    if (!r || !r->datum) \
-        ctx->throwError(QStringLiteral("Not a valid ParticleData object")); \
+    QV4::Scoped<QV4ParticleData> r(scope, ctx->thisObject()); \
+    if (!r || !r->d()->datum) \
+        ctx->engine()->throwError(QStringLiteral("Not a valid ParticleData object")); \
 \
-    return QV4::Encode(r->datum-> VARIABLE);\
+    return QV4::Encode(r->d()->datum-> VARIABLE);\
 }\
 \
 static QV4::ReturnedValue particleData_set_ ## VARIABLE (QV4::CallContext *ctx)\
 {\
     QV4::Scope scope(ctx); \
-    QV4::Scoped<QV4ParticleData> r(scope, ctx->callData->thisObject); \
-    if (!r || !r->datum)\
-        ctx->throwError(QStringLiteral("Not a valid ParticleData object"));\
+    QV4::Scoped<QV4ParticleData> r(scope, ctx->thisObject()); \
+    if (!r || !r->d()->datum)\
+        ctx->engine()->throwError(QStringLiteral("Not a valid ParticleData object"));\
 \
-    r->datum-> VARIABLE = (ctx->callData->argc && ctx->callData->args[0].toBoolean()) ? 1.0 : 0.0;\
+    r->d()->datum-> VARIABLE = (ctx->argc() && ctx->args()[0].toBoolean()) ? 1.0 : 0.0;\
     return QV4::Encode::undefined(); \
 }
 
 #define FLOAT_GETTER_AND_SETTER(VARIABLE) static QV4::ReturnedValue particleData_get_ ## VARIABLE (QV4::CallContext *ctx) \
 { \
     QV4::Scope scope(ctx); \
-    QV4::Scoped<QV4ParticleData> r(scope, ctx->callData->thisObject); \
-    if (!r || !r->datum) \
-        ctx->throwError(QStringLiteral("Not a valid ParticleData object")); \
+    QV4::Scoped<QV4ParticleData> r(scope, ctx->thisObject()); \
+    if (!r || !r->d()->datum) \
+        ctx->engine()->throwError(QStringLiteral("Not a valid ParticleData object")); \
 \
-    return QV4::Encode(r->datum-> VARIABLE);\
+    return QV4::Encode(r->d()->datum-> VARIABLE);\
 }\
 \
 static QV4::ReturnedValue particleData_set_ ## VARIABLE (QV4::CallContext *ctx)\
 {\
     QV4::Scope scope(ctx); \
-    QV4::Scoped<QV4ParticleData> r(scope, ctx->callData->thisObject); \
-    if (!r || !r->datum)\
-        ctx->throwError(QStringLiteral("Not a valid ParticleData object"));\
+    QV4::Scoped<QV4ParticleData> r(scope, ctx->thisObject()); \
+    if (!r || !r->d()->datum)\
+        ctx->engine()->throwError(QStringLiteral("Not a valid ParticleData object"));\
 \
-    r->datum-> VARIABLE = ctx->callData->argc ? ctx->callData->args[0].toNumber() : qSNaN();\
+    r->d()->datum-> VARIABLE = ctx->argc() ? ctx->args()[0].toNumber() : qt_qnan();\
     return QV4::Encode::undefined(); \
 }
 
 #define FAKE_FLOAT_GETTER_AND_SETTER(VARIABLE, GETTER, SETTER) static QV4::ReturnedValue particleData_get_ ## VARIABLE (QV4::CallContext *ctx) \
 { \
     QV4::Scope scope(ctx); \
-    QV4::Scoped<QV4ParticleData> r(scope, ctx->callData->thisObject); \
-    if (!r || !r->datum) \
-        ctx->throwError(QStringLiteral("Not a valid ParticleData object")); \
+    QV4::Scoped<QV4ParticleData> r(scope, ctx->thisObject()); \
+    if (!r || !r->d()->datum) \
+        ctx->engine()->throwError(QStringLiteral("Not a valid ParticleData object")); \
 \
-    return QV4::Encode(r->datum-> GETTER ());\
+    return QV4::Encode(r->d()->datum-> GETTER (r->d()->particleSystem));\
 }\
 \
 static QV4::ReturnedValue particleData_set_ ## VARIABLE (QV4::CallContext *ctx)\
 {\
     QV4::Scope scope(ctx); \
-    QV4::Scoped<QV4ParticleData> r(scope, ctx->callData->thisObject); \
-    if (!r || !r->datum)\
-        ctx->throwError(QStringLiteral("Not a valid ParticleData object"));\
+    QV4::Scoped<QV4ParticleData> r(scope, ctx->thisObject()); \
+    if (!r || !r->d()->datum)\
+        ctx->engine()->throwError(QStringLiteral("Not a valid ParticleData object"));\
 \
-    r->datum-> SETTER (ctx->callData->argc ? ctx->callData->args[0].toNumber() : qSNaN());\
+    r->d()->datum-> SETTER (ctx->argc() ? ctx->args()[0].toNumber() : qt_qnan(), r->d()->particleSystem);\
     return QV4::Encode::undefined(); \
 }
 
@@ -455,11 +452,10 @@ FAKE_FLOAT_GETTER_AND_SETTER(curY, curY, setInstantaneousY)
 FAKE_FLOAT_GETTER_AND_SETTER(curVY, curVY, setInstantaneousVY)
 FAKE_FLOAT_GETTER_AND_SETTER(curAY, curAY, setInstantaneousAY)
 
-QV8ParticleDataDeletable::QV8ParticleDataDeletable(QV8Engine *engine)
+QV4ParticleDataDeletable::QV4ParticleDataDeletable(QV4::ExecutionEngine *v4)
 {
-    QV4::ExecutionEngine *v4 = QV8Engine::getV4(engine);
     QV4::Scope scope(v4);
-    QV4::Scoped<QV4::Object> p(scope, v4->newObject());
+    QV4::ScopedObject p(scope, v4->newObject());
 
     p->defineDefaultProperty(QStringLiteral("discard"), particleData_discard);
     p->defineDefaultProperty(QStringLiteral("lifeLeft"), particleData_lifeLeft);
@@ -503,24 +499,24 @@ QV8ParticleDataDeletable::QV8ParticleDataDeletable(QV8Engine *engine)
     proto = p;
 }
 
-QV8ParticleDataDeletable::~QV8ParticleDataDeletable()
+QV4ParticleDataDeletable::~QV4ParticleDataDeletable()
 {
 }
 
-V8_DEFINE_EXTENSION(QV8ParticleDataDeletable, particleV8Data);
+V4_DEFINE_EXTENSION(QV4ParticleDataDeletable, particleV4Data);
 
 
-QQuickV4ParticleData::QQuickV4ParticleData(QV8Engine* engine, QQuickParticleData* datum)
+QQuickV4ParticleData::QQuickV4ParticleData(QV8Engine* engine, QQuickParticleData* datum, QQuickParticleSystem *system)
 {
     if (!engine || !datum)
         return;
 
-    QV8ParticleDataDeletable *d = particleV8Data(engine);
     QV4::ExecutionEngine *v4 = QV8Engine::getV4(engine);
     QV4::Scope scope(v4);
-    QV4::ScopedObject o(scope, new (v4->memoryManager) QV4ParticleData(v4, datum));
+    QV4ParticleDataDeletable *d = particleV4Data(scope.engine);
+    QV4::ScopedObject o(scope, v4->memoryManager->allocObject<QV4ParticleData>(datum, system));
     QV4::ScopedObject p(scope, d->proto.value());
-    o->setPrototype(p.getPointer());
+    o->setPrototype(p);
     m_v4Value = o;
 }
 
