@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQml module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -33,6 +39,17 @@
 #ifndef QV4ASSEMBLER_P_H
 #define QV4ASSEMBLER_P_H
 
+//
+//  W A R N I N G
+//  -------------
+//
+// This file is not part of the Qt API.  It exists purely as an
+// implementation detail.  This header file may change from version to
+// version without notice, or even be removed.
+//
+// We mean it.
+//
+
 #include "private/qv4global_p.h"
 #include "private/qv4jsir_p.h"
 #include "private/qv4isel_p.h"
@@ -208,7 +225,7 @@ public:
             // sp was aligned before executing the call instruction. So, calculate all contents
             // that were saved after that aligned stack...:
             const int stackSpaceAllocatedOtherwise = StackSpaceAllocatedUponFunctionEntry
-                                                     + RegisterSize; // saved StackFrameRegister
+                                                     + RegisterSize; // saved FramePointerRegister
 
             // ... then calculate the stuff we want to store ...:
             int frameSize = RegisterSize * normalRegistersToSave + sizeof(double) * fpRegistersToSave;
@@ -261,7 +278,7 @@ public:
             Q_ASSERT(offset < savedRegCount);
 
             // Get the address of the bottom-most element of our frame:
-            Address ptr(Assembler::StackFrameRegister, -calculateStackFrameSize());
+            Address ptr(Assembler::FramePointerRegister, -calculateStackFrameSize());
             // This now is the element with offset 0. So:
             ptr.offset += offset * sizeof(QV4::Value);
             // and we're done!
@@ -363,6 +380,11 @@ public:
     void addPatch(DataLabelPtr patch, IR::BasicBlock *target);
     void generateCJumpOnNonZero(RegisterID reg, IR::BasicBlock *currentBlock,
                              IR::BasicBlock *trueBlock, IR::BasicBlock *falseBlock);
+#ifdef QV4_USE_64_BIT_VALUE_ENCODING
+    void generateCJumpOnCompare(RelationalCondition cond, RegisterID left, TrustedImm64 right,
+                                IR::BasicBlock *currentBlock, IR::BasicBlock *trueBlock,
+                                IR::BasicBlock *falseBlock);
+#endif
     void generateCJumpOnCompare(RelationalCondition cond, RegisterID left, TrustedImm32 right,
                                 IR::BasicBlock *currentBlock, IR::BasicBlock *trueBlock,
                                 IR::BasicBlock *falseBlock);
@@ -478,7 +500,7 @@ public:
             load64(addr, dest);
         } else {
             QV4::Value undefined = QV4::Primitive::undefinedValue();
-            move(TrustedImm64(undefined.val), dest);
+            move(TrustedImm64(undefined.rawValue()), dest);
         }
     }
 
@@ -491,7 +513,7 @@ public:
             load64(addr, dest);
         } else {
             QV4::Value undefined = QV4::Primitive::undefinedValue();
-            move(TrustedImm64(undefined.val), dest);
+            move(TrustedImm64(undefined.rawValue()), dest);
         }
     }
 
@@ -500,7 +522,7 @@ public:
         Q_UNUSED(argumentNumber);
 
         QV4::Value v = convertToValue(c);
-        move(TrustedImm64(v.val), dest);
+        move(TrustedImm64(v.rawValue()), dest);
     }
 
     void loadArgumentInRegister(IR::Expr* expr, RegisterID dest, int argumentNumber)
@@ -509,7 +531,7 @@ public:
 
         if (!expr) {
             QV4::Value undefined = QV4::Primitive::undefinedValue();
-            move(TrustedImm64(undefined.val), dest);
+            move(TrustedImm64(undefined.rawValue()), dest);
         } else if (IR::Temp *t = expr->asTemp()){
             loadArgumentInRegister(t, dest, argumentNumber);
         } else if (IR::ArgLocal *al = expr->asArgLocal()) {
@@ -565,6 +587,8 @@ public:
         moveIntsToDouble(JSC::ARMRegisters::r0, JSC::ARMRegisters::r1, dest, FPGpr0);
 #elif defined(Q_PROCESSOR_X86)
         moveIntsToDouble(JSC::X86Registers::eax, JSC::X86Registers::edx, dest, FPGpr0);
+#elif defined(Q_PROCESSOR_MIPS)
+        moveIntsToDouble(JSC::MIPSRegisters::v0, JSC::MIPSRegisters::v1, dest, FPGpr0);
 #else
         subPtr(TrustedImm32(sizeof(QV4::Value)), StackPointerRegister);
         Pointer tmp(StackPointerRegister, 0);
@@ -595,6 +619,14 @@ public:
         destination.offset += 4;
         store32(JSC::ARMRegisters::r1, destination);
     }
+#elif defined(Q_PROCESSOR_MIPS)
+    void storeReturnValue(const Pointer &dest)
+    {
+        Pointer destination = dest;
+        store32(JSC::MIPSRegisters::v0, destination);
+        destination.offset += 4;
+        store32(JSC::MIPSRegisters::v1, destination);
+    }
 #endif
 
     void storeReturnValue(IR::Expr *target)
@@ -724,7 +756,7 @@ public:
             moveDouble(source, (FPRegisterID) targetTemp->index);
             return;
         }
-#if QT_POINTER_SIZE == 8
+#ifdef QV4_USE_64_BIT_VALUE_ENCODING
         moveDoubleTo64(source, ReturnValueRegister);
         move(TrustedImm64(QV4::Value::NaNEncodeMask), ScratchRegister);
         xor64(ScratchRegister, ReturnValueRegister);
@@ -735,7 +767,7 @@ public:
         storeDouble(source, ptr);
 #endif
     }
-#if QT_POINTER_SIZE == 8
+#ifdef QV4_USE_64_BIT_VALUE_ENCODING
     // We need to (de)mangle the double
     void loadDouble(Address addr, FPRegisterID dest)
     {
@@ -781,11 +813,11 @@ public:
     void storeValue(QV4::Primitive value, Address destination)
     {
 #ifdef VALUE_FITS_IN_REGISTER
-        store64(TrustedImm64(value.val), destination);
+        store64(TrustedImm64(value.rawValue()), destination);
 #else
-        store32(TrustedImm32(value.int_32), destination);
+        store32(TrustedImm32(value.int_32()), destination);
         destination.offset += 4;
-        store32(TrustedImm32(value.tag), destination);
+        store32(TrustedImm32(value.tag()), destination);
 #endif
     }
 
@@ -820,6 +852,8 @@ public:
         else
 #if OS(WINDOWS) && CPU(X86_64)
             loadArgumentOnStack<argumentNumber>(value, argumentNumber);
+#elif CPU(MIPS) // Stack space for 4 arguments needs to be allocated for MIPS platforms.
+            loadArgumentOnStack<argumentNumber>(value, argumentNumber + 4);
 #else // Sanity:
             loadArgumentOnStack<argumentNumber - RegisterArgumentCount>(value, argumentNumber);
 #endif
@@ -846,7 +880,7 @@ public:
     template <int ArgumentIndex, typename Parameter>
     struct SizeOnStack
     {
-        enum { Size = Select<ArgumentIndex >= RegisterArgumentCount, QT_POINTER_SIZE, 0>::Chosen };
+        enum { Size = Select<ArgumentIndex >= RegisterArgumentCount, sizeof(void*), 0>::Chosen };
     };
 
     template <int ArgumentIndex>
@@ -945,8 +979,8 @@ public:
             tagAddr.offset += 4;
 
             QV4::Primitive v = convertToValue(c);
-            store32(TrustedImm32(v.int_32), addr);
-            store32(TrustedImm32(v.tag), tagAddr);
+            store32(TrustedImm32(v.int_32()), addr);
+            store32(TrustedImm32(v.tag()), tagAddr);
             return Pointer(addr);
         }
 
@@ -961,7 +995,7 @@ public:
     {
         store32(reg, addr);
         addr.offset += 4;
-        store32(TrustedImm32(QV4::Primitive::fromBoolean(0).tag), addr);
+        store32(TrustedImm32(QV4::Primitive::fromBoolean(0).tag()), addr);
     }
 
     void storeBool(RegisterID src, RegisterID dest)
@@ -1005,7 +1039,7 @@ public:
     {
         store32(reg, addr);
         addr.offset += 4;
-        store32(TrustedImm32(QV4::Primitive::fromInt32(0).tag), addr);
+        store32(TrustedImm32(QV4::Primitive::fromInt32(0).tag()), addr);
     }
 
     void storeInt32(RegisterID reg, IR::Expr *target)
@@ -1054,13 +1088,11 @@ public:
     FPRegisterID toDoubleRegister(IR::Expr *e, FPRegisterID target = FPGpr0)
     {
         if (IR::Const *c = e->asConst()) {
-#if QT_POINTER_SIZE == 8
-            union {
-                double d;
-                int64_t i;
-            } u;
-            u.d = c->value;
-            move(TrustedImm64(u.i), ReturnValueRegister);
+#ifdef QV4_USE_64_BIT_VALUE_ENCODING
+            Q_STATIC_ASSERT(sizeof(int64_t) == sizeof(double));
+            int64_t i;
+            memcpy(&i, &c->value, sizeof(double));
+            move(TrustedImm64(i), ReturnValueRegister);
             move64ToDouble(ReturnValueRegister, target);
 #else
             JSC::MacroAssembler::loadDouble(constantTable().loadValueAddress(c, ScratchRegister), target);
@@ -1084,7 +1116,7 @@ public:
     RegisterID toInt32Register(IR::Expr *e, RegisterID scratchReg)
     {
         if (IR::Const *c = e->asConst()) {
-            move(TrustedImm32(convertToValue(c).int_32), scratchReg);
+            move(TrustedImm32(convertToValue(c).int_32()), scratchReg);
             return scratchReg;
         }
 
@@ -1123,7 +1155,7 @@ public:
         Pointer tagAddr = addr;
         tagAddr.offset += 4;
         load32(tagAddr, scratchReg);
-        Jump inIntRange = branch32(Equal, scratchReg, TrustedImm32(QV4::Value::_Integer_Type));
+        Jump inIntRange = branch32(Equal, scratchReg, TrustedImm32(QV4::Value::Integer_Type_Internal));
 
         // it's not in signed int range, so load it as a double, and truncate it down
         loadDouble(addr, FPGpr0);
