@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQml module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -33,6 +39,17 @@
 #ifndef QV4SCOPEDVALUE_P_H
 #define QV4SCOPEDVALUE_P_H
 
+//
+//  W A R N I N G
+//  -------------
+//
+// This file is not part of the Qt API.  It exists purely as an
+// implementation detail.  This header file may change from version to
+// version without notice, or even be removed.
+//
+// We mean it.
+//
+
 #include "qv4engine_p.h"
 #include "qv4value_p.h"
 #include "qv4persistent_p.h"
@@ -54,18 +71,12 @@ struct ScopedValue;
 struct Scope {
     inline Scope(ExecutionContext *ctx)
         : engine(ctx->d()->engine)
-#ifndef QT_NO_DEBUG
-        , size(0)
-#endif
     {
         mark = engine->jsStackTop;
     }
 
     explicit Scope(ExecutionEngine *e)
         : engine(e)
-#ifndef QT_NO_DEBUG
-        , size(0)
-#endif
     {
         mark = engine->jsStackTop;
     }
@@ -73,22 +84,17 @@ struct Scope {
     ~Scope() {
 #ifndef QT_NO_DEBUG
         Q_ASSERT(engine->jsStackTop >= mark);
+        Q_ASSERT(engine->currentContext < mark);
         memset(mark, 0, (engine->jsStackTop - mark)*sizeof(Value));
 #endif
 #ifdef V4_USE_VALGRIND
-        VALGRIND_MAKE_MEM_UNDEFINED(mark, engine->jsStackLimit - mark);
+        VALGRIND_MAKE_MEM_UNDEFINED(mark, (engine->jsStackLimit - mark) * sizeof(Value));
 #endif
         engine->jsStackTop = mark;
     }
 
     Value *alloc(int nValues) {
-#ifndef QT_NO_DEBUG
-        size += nValues;
-#endif
-        Value *ptr = engine->jsStackTop;
-        engine->jsStackTop = ptr + nValues;
-        memset(ptr, 0, nValues*sizeof(Value));
-        return ptr;
+        return engine->jsAlloca(nValues);
     }
 
     bool hasException() const {
@@ -97,9 +103,6 @@ struct Scope {
 
     ExecutionEngine *engine;
     Value *mark;
-#ifndef QT_NO_DEBUG
-    mutable int size;
-#endif
 
 private:
     Q_DISABLE_COPY(Scope)
@@ -110,49 +113,31 @@ struct ScopedValue
     ScopedValue(const Scope &scope)
     {
         ptr = scope.engine->jsStackTop++;
-        ptr->val = 0;
-#ifndef QT_NO_DEBUG
-        ++scope.size;
-#endif
+        ptr->setRawValue(0);
     }
 
     ScopedValue(const Scope &scope, const Value &v)
     {
         ptr = scope.engine->jsStackTop++;
         *ptr = v;
-#ifndef QT_NO_DEBUG
-        ++scope.size;
-#endif
     }
 
     ScopedValue(const Scope &scope, Heap::Base *o)
     {
         ptr = scope.engine->jsStackTop++;
-        ptr->m = o;
-#if QT_POINTER_SIZE == 4
-        ptr->tag = QV4::Value::Managed_Type;
-#endif
-#ifndef QT_NO_DEBUG
-        ++scope.size;
-#endif
+        ptr->setM(o);
     }
 
     ScopedValue(const Scope &scope, Managed *m)
     {
         ptr = scope.engine->jsStackTop++;
-        ptr->val = m->asReturnedValue();
-#ifndef QT_NO_DEBUG
-        ++scope.size;
-#endif
+        ptr->setRawValue(m->asReturnedValue());
     }
 
     ScopedValue(const Scope &scope, const ReturnedValue &v)
     {
         ptr = scope.engine->jsStackTop++;
-        ptr->val = v;
-#ifndef QT_NO_DEBUG
-        ++scope.size;
-#endif
+        ptr->setRawValue(v);
     }
 
     ScopedValue &operator=(const Value &v) {
@@ -161,10 +146,7 @@ struct ScopedValue
     }
 
     ScopedValue &operator=(Heap::Base *o) {
-        ptr->m = o;
-#if QT_POINTER_SIZE == 4
-        ptr->tag = QV4::Value::Managed_Type;
-#endif
+        ptr->setM(o);
         return *this;
     }
 
@@ -174,7 +156,7 @@ struct ScopedValue
     }
 
     ScopedValue &operator=(const ReturnedValue &v) {
-        ptr->val = v;
+        ptr->setRawValue(v);
         return *this;
     }
 
@@ -200,111 +182,72 @@ struct ScopedValue
 template<typename T>
 struct Scoped
 {
-    enum _Convert { Convert };
+    enum ConvertType { Convert };
 
-    inline void setPointer(Managed *p) {
-        ptr->m = p ? p->m : 0;
-#if QT_POINTER_SIZE == 4
-        ptr->tag = QV4::Value::Managed_Type;
-#endif
+    inline void setPointer(const Managed *p) {
+        ptr->setM(p ? p->m() : 0);
     }
 
     Scoped(const Scope &scope)
     {
         ptr = scope.engine->jsStackTop++;
-        ptr->m = 0;
-#if QT_POINTER_SIZE == 4
-        ptr->tag = QV4::Value::Managed_Type;
-#endif
-#ifndef QT_NO_DEBUG
-        ++scope.size;
-#endif
+        ptr->setM(0);
     }
 
-    // ### GC FIX casting below to be safe
     Scoped(const Scope &scope, const Value &v)
     {
         ptr = scope.engine->jsStackTop++;
-        setPointer(value_cast<T>(v));
-#ifndef QT_NO_DEBUG
-        ++scope.size;
-#endif
+        setPointer(v.as<T>());
     }
     Scoped(const Scope &scope, Heap::Base *o)
     {
         Value v;
         v = o;
         ptr = scope.engine->jsStackTop++;
-        setPointer(value_cast<T>(v));
-#ifndef QT_NO_DEBUG
-        ++scope.size;
-#endif
+        setPointer(v.as<T>());
     }
     Scoped(const Scope &scope, const ScopedValue &v)
     {
         ptr = scope.engine->jsStackTop++;
-        setPointer(value_cast<T>(*v.ptr));
-#ifndef QT_NO_DEBUG
-        ++scope.size;
-#endif
+        setPointer(v.ptr->as<T>());
     }
 
-    Scoped(const Scope &scope, const Value &v, _Convert)
+    Scoped(const Scope &scope, const Value &v, ConvertType)
     {
         ptr = scope.engine->jsStackTop++;
-        ptr->val = value_convert<T>(scope.engine, v);
-#ifndef QT_NO_DEBUG
-        ++scope.size;
-#endif
+        ptr->setRawValue(value_convert<T>(scope.engine, v));
     }
 
     Scoped(const Scope &scope, const Value *v)
     {
         ptr = scope.engine->jsStackTop++;
-        setPointer(v ? value_cast<T>(*v) : 0);
-#ifndef QT_NO_DEBUG
-        ++scope.size;
-#endif
+        setPointer(v ? v->as<T>() : 0);
     }
 
     Scoped(const Scope &scope, T *t)
     {
         ptr = scope.engine->jsStackTop++;
         setPointer(t);
-#ifndef QT_NO_DEBUG
-        ++scope.size;
-#endif
     }
     Scoped(const Scope &scope, typename T::Data *t)
     {
         ptr = scope.engine->jsStackTop++;
         *ptr = t;
-#ifndef QT_NO_DEBUG
-        ++scope.size;
-#endif
     }
 
     Scoped(const Scope &scope, const ReturnedValue &v)
     {
         ptr = scope.engine->jsStackTop++;
-        setPointer(value_cast<T>(QV4::Value::fromReturnedValue(v)));
-#ifndef QT_NO_DEBUG
-        ++scope.size;
-#endif
+        setPointer(QV4::Value::fromReturnedValue(v).as<T>());
     }
-    Scoped(const Scope &scope, const ReturnedValue &v, _Convert)
+    Scoped(const Scope &scope, const ReturnedValue &v, ConvertType)
     {
         ptr = scope.engine->jsStackTop++;
-        ptr->val = value_convert<T>(scope.engine, QV4::Value::fromReturnedValue(v));
-#ifndef QT_NO_DEBUG
-        ++scope.size;
-#endif
+        ptr->setRawValue(value_convert<T>(scope.engine, QV4::Value::fromReturnedValue(v)));
     }
 
     Scoped<T> &operator=(Heap::Base *o) {
-        Value v;
-        v = o;
-        setPointer(value_cast<T>(v));
+        setPointer(Value::fromHeapObject(o).as<T>());
         return *this;
     }
     Scoped<T> &operator=(typename T::Data *t) {
@@ -312,16 +255,16 @@ struct Scoped
         return *this;
     }
     Scoped<T> &operator=(const Value &v) {
-        setPointer(value_cast<T>(v));
+        setPointer(v.as<T>());
         return *this;
     }
     Scoped<T> &operator=(Value *v) {
-        setPointer(v ? value_cast<T>(*v) : 0);
+        setPointer(v ? v->as<T>() : 0);
         return *this;
     }
 
     Scoped<T> &operator=(const ReturnedValue &v) {
-        setPointer(value_cast<T>(QV4::Value::fromReturnedValue(v)));
+        setPointer(QV4::Value::fromReturnedValue(v).as<T>());
         return *this;
     }
 
@@ -347,21 +290,21 @@ struct Scoped
     }
 
     bool operator!() const {
-        return !ptr->m;
+        return !ptr->m();
     }
     operator void *() const {
-        return ptr->m;
+        return ptr->m();
     }
 
     T *getPointer() {
         return ptr->cast<T>();
     }
-    typename T::Data **getRef() {
-        return reinterpret_cast<typename T::Data **>(&ptr->m);
+    Value *getRef() {
+        return ptr;
     }
 
     ReturnedValue asReturnedValue() const {
-        return ptr->m ? ptr->val : Encode::undefined();
+        return ptr->m() ? ptr->rawValue() : Encode::undefined();
     }
 
     Value *ptr;
@@ -372,7 +315,7 @@ struct ScopedCallData {
     {
         int size = qMax(argc, (int)QV4::Global::ReservedArgumentCount) + qOffsetOf(QV4::CallData, args)/sizeof(QV4::Value);
         ptr = reinterpret_cast<CallData *>(scope.alloc(size));
-        ptr->tag = QV4::Value::Integer_Type;
+        ptr->tag = QV4::Value::Integer_Type_Internal;
         ptr->argc = argc;
     }
 
@@ -390,41 +333,14 @@ struct ScopedCallData {
 
 inline Value &Value::operator =(const ScopedValue &v)
 {
-    val = v.ptr->val;
+    _val = v.ptr->rawValue();
     return *this;
 }
 
 template<typename T>
 inline Value &Value::operator=(const Scoped<T> &t)
 {
-    val = t.ptr->val;
-    return *this;
-}
-
-template<typename T>
-inline TypedValue<T> &TypedValue<T>::operator =(T *t)
-{
-    m = t ? t->m : 0;
-#if QT_POINTER_SIZE == 4
-    tag = Managed_Type;
-#endif
-    return *this;
-}
-
-template<typename T>
-inline TypedValue<T> &TypedValue<T>::operator =(const Scoped<T> &v)
-{
-    m = v.ptr->m;
-#if QT_POINTER_SIZE == 4
-    tag = Managed_Type;
-#endif
-    return *this;
-}
-
-template<typename T>
-inline TypedValue<T> &TypedValue<T>::operator=(const TypedValue<T> &t)
-{
-    val = t.val;
+    _val = t.ptr->rawValue();
     return *this;
 }
 
@@ -447,29 +363,18 @@ struct ScopedProperty
 struct ExecutionContextSaver
 {
     ExecutionEngine *engine;
-    Value *savedContext;
+    ExecutionContext *savedContext;
 
-    ExecutionContextSaver(Scope &scope, ExecutionContext *context)
-        : engine(context->d()->engine)
-        , savedContext(scope.alloc(1))
+    ExecutionContextSaver(Scope &scope)
+        : engine(scope.engine)
     {
-        savedContext->m = context->d();
-#if QT_POINTER_SIZE == 4
-        savedContext->tag = QV4::Value::Managed_Type;
-#endif
-    }
-    ExecutionContextSaver(Scope &scope, Heap::ExecutionContext *context)
-        : engine(context->engine)
-        , savedContext(scope.alloc(1))
-    {
-        savedContext->m = context;
-#if QT_POINTER_SIZE == 4
-        savedContext->tag = QV4::Value::Managed_Type;
-#endif
+        savedContext = engine->currentContext;
     }
     ~ExecutionContextSaver()
     {
-        engine->current = static_cast<Heap::ExecutionContext *>(savedContext->heapObject());
+        Q_ASSERT(engine->jsStackTop > engine->currentContext);
+        engine->currentContext = savedContext;
+        engine->current = savedContext->d();
     }
 };
 
