@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQml module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -162,7 +168,7 @@ void QQmlBinding::update(QQmlPropertyPrivate::WriteFlags flags)
         return;
     }
 
-    QQmlBindingProfiler prof(ep->profiler, f);
+    QQmlBindingProfiler prof(ep->profiler, this, f);
     setUpdatingFlag(true);
 
     QQmlJavaScriptExpression::DeleteWatcher watcher(this);
@@ -220,9 +226,6 @@ bool QQmlBinding::write(const QQmlPropertyData &core,
     Q_ASSERT(m_target.data());
     Q_ASSERT(core.coreIndex != -1);
 
-    QQmlEngine *engine = context()->engine;
-    QV8Engine *v8engine = QQmlEnginePrivate::getV8Engine(engine);
-
 #define QUICK_STORE(cpptype, conversion) \
         { \
             cpptype o = (conversion); \
@@ -233,7 +236,7 @@ bool QQmlBinding::write(const QQmlPropertyData &core,
         } \
 
 
-    if (!isUndefined && !core.isValueTypeVirtual()) {
+    if (Q_LIKELY(!isUndefined && !core.isValueTypeVirtual())) {
         switch (core.propType) {
         case QMetaType::Int:
             if (result.isInteger())
@@ -264,6 +267,9 @@ bool QQmlBinding::write(const QQmlPropertyData &core,
     }
 #undef QUICK_STORE
 
+    QQmlEngine *engine = context()->engine;
+    QV8Engine *v8engine = QQmlEnginePrivate::getV8Engine(engine);
+
     int type = core.isValueTypeVirtual() ? core.valueTypePropType : core.propType;
 
     QQmlJavaScriptExpression::DeleteWatcher watcher(this);
@@ -491,7 +497,7 @@ QQmlPropertyData QQmlBinding::getPropertyData() const
     Q_ASSERT(propertyData);
 
     QQmlPropertyData d = *propertyData;
-    if (valueTypeIndex != -1) {
+    if (Q_UNLIKELY(valueTypeIndex != -1)) {
         const QMetaObject *valueTypeMetaObject = QQmlValueTypeFactory::metaObjectForMetaType(d.propType);
         Q_ASSERT(valueTypeMetaObject);
         QMetaProperty vtProp = valueTypeMetaObject->property(valueTypeIndex);
