@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQml module of the Qt Toolkit.
 **
@@ -10,30 +10,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -68,6 +66,7 @@
 #include <private/qflagpointer_p.h>
 #include <private/qqmlguard_p.h>
 
+#include <private/qv4compileddata_p.h>
 #include <private/qv4identifier_p.h>
 
 QT_BEGIN_NAMESPACE
@@ -77,7 +76,7 @@ class QQmlExpression;
 class QQmlEngine;
 class QQmlExpression;
 class QQmlExpressionPrivate;
-class QQmlAbstractExpression;
+class QQmlJavaScriptExpression;
 class QQmlContextData;
 
 class QQmlContextPrivate : public QObjectPrivate
@@ -149,8 +148,12 @@ public:
     // VME data that is constructing this context if any
     void *activeVMEData;
 
-    // Property name cache
-    QV4::IdentifierHash<int> propertyNames;
+    // Compilation unit for contexts that belong to a compiled type.
+    QQmlRefPointer<QV4::CompiledData::CompilationUnit> typeCompilationUnit;
+
+    mutable QHash<int, int> objectIndexToId;
+    mutable QV4::IdentifierHash<int> propertyNameCache;
+    QV4::IdentifierHash<int> &propertyNames() const;
 
     // Context object
     QObject *contextObject;
@@ -158,9 +161,11 @@ public:
     // Any script blocks that exist on this context
     QV4::PersistentValue importedScripts; // This is a JS Array
 
-    // Context base url
-    QUrl url;
-    QString urlString;
+    QUrl baseUrl;
+    QString baseUrlString;
+
+    QUrl url() const;
+    QString urlString() const;
 
     // List of imports that apply to this context
     QQmlTypeNameCache *imports;
@@ -173,7 +178,7 @@ public:
     QQmlContextData **prevChild;
 
     // Expressions that use this context
-    QQmlAbstractExpression *expressions;
+    QQmlJavaScriptExpression *expressions;
 
     // Doubly-linked list of objects that are owned by this context
     QQmlData *contextObjects;
@@ -193,17 +198,10 @@ public:
         QFlagPointer<QQmlContextData> context;
         QQmlNotifier bindings;
     };
-    struct ObjectIdMapping {
-        ObjectIdMapping() : id(-1) {}
-        ObjectIdMapping(const QString &name, int id)
-            : name(name), id(id) {}
-        QString name;
-        int id;
-    };
     ContextGuard *idValues;
     int idValueCount;
     void setIdProperty(int, QObject *);
-    void setIdPropertyData(const QVector<ObjectIdMapping> &);
+    void setIdPropertyData(const QHash<int, int> &);
 
     // Linked contexts. this owns linkedContext.
     QQmlContextData *linkedContext;
@@ -221,7 +219,7 @@ public:
 
 private:
     void refreshExpressionsRecursive(bool isGlobal);
-    void refreshExpressionsRecursive(QQmlAbstractExpression *);
+    void refreshExpressionsRecursive(QQmlJavaScriptExpression *);
     ~QQmlContextData() {}
 };
 
