@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQml module of the Qt Toolkit.
 **
@@ -10,30 +10,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -44,6 +42,7 @@
 #include <QtCore/qurl.h>
 #include <QtCore/qobject.h>
 #include <QtCore/qmetaobject.h>
+#include <QtCore/qfile.h>
 #include <private/qqmlengine_p.h>
 #include <private/qqmlglobal_p.h>
 
@@ -53,7 +52,7 @@
 
 \internal
 
-Supports file://, qrc:/, bundle:// uris and whatever QNetworkAccessManager supports.
+Supports file:// and qrc:/ uris and whatever QNetworkAccessManager supports.
 */
 
 #define QQMLFILE_MAX_REDIRECT_RECURSION 16
@@ -62,7 +61,6 @@ QT_BEGIN_NAMESPACE
 
 static char qrc_string[] = "qrc";
 static char file_string[] = "file";
-static char bundle_string[] = "bundle";
 
 #if defined(Q_OS_ANDROID)
 static char assets_string[] = "assets";
@@ -108,9 +106,6 @@ public:
     mutable QUrl url;
     mutable QString urlString;
 
-    QQmlBundleData *bundle;
-    const QQmlBundle::FileEntry *file;
-
     QByteArray data;
 
     enum Error {
@@ -207,7 +202,7 @@ void QQmlFileNetworkReply::networkDownloadProgress(qint64 a, qint64 b)
 }
 
 QQmlFilePrivate::QQmlFilePrivate()
-: bundle(0), file(0), error(None), reply(0)
+: error(None), reply(0)
 {
 }
 
@@ -230,11 +225,7 @@ QQmlFile::QQmlFile(QQmlEngine *e, const QString &url)
 
 QQmlFile::~QQmlFile()
 {
-    if (d->reply)
-        delete d->reply;
-    if (d->bundle)
-        d->bundle->release();
-
+    delete d->reply;
     delete d;
     d = 0;
 }
@@ -295,31 +286,17 @@ QString QQmlFile::error() const
 
 qint64 QQmlFile::size() const
 {
-    if (d->file) return d->file->fileSize();
-    else return d->data.size();
+    return d->data.size();
 }
 
 const char *QQmlFile::data() const
 {
-    if (d->file) return d->file->contents();
-    else return d->data.constData();
+    return d->data.constData();
 }
 
 QByteArray QQmlFile::dataByteArray() const
 {
-    if (d->file) return QByteArray(d->file->contents(), d->file->fileSize());
-    else return d->data;
-}
-
-QByteArray QQmlFile::metaData(const QString &name) const
-{
-    if (d->file) {
-        Q_ASSERT(d->bundle);
-        const QQmlBundle::FileEntry *meta = d->bundle->link(d->file, name);
-        if (meta)
-            return QByteArray::fromRawData(meta->contents(), meta->fileSize());
-    }
-    return QByteArray();
+    return d->data;
 }
 
 void QQmlFile::load(QQmlEngine *engine, const QUrl &url)
@@ -329,25 +306,7 @@ void QQmlFile::load(QQmlEngine *engine, const QUrl &url)
     clear();
     d->url = url;
 
-    if (isBundle(url)) {
-        // Bundle
-        QQmlEnginePrivate *p = QQmlEnginePrivate::get(engine);
-        QQmlBundleData *bundle = p->typeLoader.getBundle(url.host());
-
-        d->error = QQmlFilePrivate::NotFound;
-
-        if (bundle) {
-            QString filename = url.path().mid(1);
-            const QQmlBundle::FileEntry *entry = bundle->find(filename);
-            if (entry) {
-                d->file = entry;
-                d->bundle = bundle;
-                d->bundle->addref();
-                d->error = QQmlFilePrivate::None;
-            }
-            bundle->release();
-        }
-    } else if (isLocalFile(url)) {
+    if (isLocalFile(url)) {
         QString lf = urlToLocalFileOrQrc(url);
 
         if (!QQml_isFileCaseCorrect(lf)) {
@@ -374,33 +333,7 @@ void QQmlFile::load(QQmlEngine *engine, const QString &url)
 
     d->urlString = url;
 
-    if (isBundle(url)) {
-        // Bundle
-        QQmlEnginePrivate *p = QQmlEnginePrivate::get(engine);
-
-        d->error = QQmlFilePrivate::NotFound;
-
-        int index = url.indexOf(QLatin1Char('/'), 9);
-        if (index == -1)
-            return;
-
-        QStringRef identifier(&url, 9, index - 9);
-
-        QQmlBundleData *bundle = p->typeLoader.getBundle(identifier);
-
-        d->error = QQmlFilePrivate::NotFound;
-
-        if (bundle) {
-            QString filename = url.mid(index);
-            const QQmlBundle::FileEntry *entry = bundle->find(filename);
-            if (entry) {
-                d->data = QByteArray(entry->contents(), entry->fileSize());
-                d->error = QQmlFilePrivate::None;
-            }
-            bundle->release();
-        }
-
-    } else if (isLocalFile(url)) {
+    if (isLocalFile(url)) {
         QString lf = urlToLocalFileOrQrc(url);
 
         if (!QQml_isFileCaseCorrect(lf)) {
@@ -427,9 +360,6 @@ void QQmlFile::clear()
     d->url = QUrl();
     d->urlString = QString();
     d->data = QByteArray();
-    if (d->bundle) d->bundle->release();
-    d->bundle = 0;
-    d->file = 0;
     d->error = QQmlFilePrivate::None;
 }
 
@@ -485,7 +415,7 @@ bool QQmlFile::connectDownloadProgress(QObject *object, int method)
 /*!
 Returns true if QQmlFile will open \a url synchronously.
 
-Synchronous urls have a qrc:/, file://, or bundle:// scheme.
+Synchronous urls have a qrc:/ or file:// scheme.
 
 \note On Android, urls with assets:/ scheme are also considered synchronous.
 */
@@ -494,7 +424,6 @@ bool QQmlFile::isSynchronous(const QUrl &url)
     QString scheme = url.scheme();
 
     if ((scheme.length() == 4 && 0 == scheme.compare(QLatin1String(file_string), Qt::CaseInsensitive)) ||
-        (scheme.length() == 6 && 0 == scheme.compare(QLatin1String(bundle_string), Qt::CaseInsensitive)) ||
         (scheme.length() == 3 && 0 == scheme.compare(QLatin1String(qrc_string), Qt::CaseInsensitive))) {
         return true;
 
@@ -511,7 +440,7 @@ bool QQmlFile::isSynchronous(const QUrl &url)
 /*!
 Returns true if QQmlFile will open \a url synchronously.
 
-Synchronous urls have a qrc:/, file://, or bundle:// scheme.
+Synchronous urls have a qrc:/ or file:// scheme.
 
 \note On Android, urls with assets:/ scheme are also considered synchronous.
 */
@@ -528,12 +457,6 @@ bool QQmlFile::isSynchronous(const QString &url)
                url.startsWith(QLatin1String(file_string), Qt::CaseInsensitive) &&
                url[4] == QLatin1Char(':') && url[5] == QLatin1Char('/') && url[6] == QLatin1Char('/');
 
-    } else if (f == QLatin1Char('b') || f == QLatin1Char('B')) {
-
-        return url.length() >= 9 /* bundle:// */ &&
-               url.startsWith(QLatin1String(bundle_string), Qt::CaseInsensitive) &&
-               url[6] == QLatin1Char(':') && url[7] == QLatin1Char('/') && url[8] == QLatin1Char('/');
-
     } else if (f == QLatin1Char('q') || f == QLatin1Char('Q')) {
 
         return url.length() >= 5 /* qrc:/ */ &&
@@ -554,29 +477,6 @@ bool QQmlFile::isSynchronous(const QString &url)
     return false;
 }
 
-/*!
-Returns true if \a url is a bundle.
-
-Bundle urls have a bundle:// scheme.
-*/
-bool QQmlFile::isBundle(const QString &url)
-{
-    return url.length() >= 9 && url.startsWith(QLatin1String(bundle_string), Qt::CaseInsensitive) &&
-           url[6] == QLatin1Char(':') && url[7] == QLatin1Char('/') && url[8] == QLatin1Char('/');
-}
-
-/*!
-Returns true if \a url is a bundle.
-
-Bundle urls have a bundle:// scheme.
-*/
-bool QQmlFile::isBundle(const QUrl &url)
-{
-    QString scheme = url.scheme();
-
-    return scheme.length() == 6 && 0 == scheme.compare(QLatin1String(bundle_string), Qt::CaseInsensitive);
-}
-
 /*!
 Returns true if \a url is a local file that can be opened with QFile.
 
@@ -666,18 +566,13 @@ QString QQmlFile::urlToLocalFileOrQrc(const QUrl& url)
 
 static QString toLocalFile(const QString &url)
 {
-    if (!url.startsWith(QLatin1String("file://"), Qt::CaseInsensitive))
+    const QUrl file(url);
+    if (!file.isLocalFile())
         return QString();
 
-    QString file = url.mid(7);
-
     //XXX TODO: handle windows hostnames: "//servername/path/to/file.txt"
 
-    // magic for drives on windows
-    if (file.length() > 2 && file.at(0) == QLatin1Char('/') && file.at(2) == QLatin1Char(':'))
-        file.remove(0, 1);
-
-    return file;
+    return file.toLocalFile();
 }
 
 /*!
@@ -688,7 +583,7 @@ QString QQmlFile::urlToLocalFileOrQrc(const QString& url)
 {
     if (url.startsWith(QLatin1String("qrc:"), Qt::CaseInsensitive)) {
         if (url.length() > 4)
-            return QLatin1Char(':') + url.mid(4);
+            return QLatin1Char(':') + url.midRef(4);
         return QString();
     }
 
@@ -701,162 +596,6 @@ QString QQmlFile::urlToLocalFileOrQrc(const QString& url)
     return toLocalFile(url);
 }
 
-bool QQmlFile::bundleDirectoryExists(const QString &dir, QQmlEngine *e)
-{
-    if (!isBundle(dir))
-        return false;
-
-    int index = dir.indexOf(QLatin1Char('/'), 9);
-
-    if (index == -1 && dir.length() > 9) // We accept "bundle://<blah>" with no extra path
-        index = dir.length();
-
-    if (index == -1)
-        return false;
-
-    QStringRef identifier(&dir, 9, index - 9);
-
-    QQmlBundleData *bundle = QQmlEnginePrivate::get(e)->typeLoader.getBundle(identifier);
-
-    if (bundle) {
-        int lastIndex = dir.lastIndexOf(QLatin1Char('/'));
-
-        if (lastIndex <= index) {
-            bundle->release();
-            return true;
-        }
-
-        QStringRef d(&dir, index + 1, lastIndex - index);
-
-        QList<const QQmlBundle::FileEntry *> entries = bundle->files();
-
-        for (int ii = 0; ii < entries.count(); ++ii) {
-            QString name = entries.at(ii)->fileName();
-            if (name.startsWith(d)) {
-                bundle->release();
-                return true;
-            }
-        }
-
-        bundle->release();
-    }
-
-    return false;
-}
-
-bool QQmlFile::bundleDirectoryExists(const QUrl &url, QQmlEngine *e)
-{
-    if (!isBundle(url))
-        return false;
-
-    QQmlBundleData *bundle = QQmlEnginePrivate::get(e)->typeLoader.getBundle(url.host());
-
-    if (bundle) {
-        QString path = url.path();
-
-        int lastIndex = path.lastIndexOf(QLatin1Char('/'));
-
-        if (lastIndex == -1) {
-            bundle->release();
-            return true;
-        }
-
-        QStringRef d(&path, 0, lastIndex);
-
-        QList<const QQmlBundle::FileEntry *> entries = bundle->files();
-
-        for (int ii = 0; ii < entries.count(); ++ii) {
-            QString name = entries.at(ii)->fileName();
-            if (name.startsWith(d)) {
-                bundle->release();
-                return true;
-            }
-        }
-
-        bundle->release();
-    }
-
-    return false;
-}
-
-bool QQmlFile::bundleFileExists(const QString &file, QQmlEngine *e)
-{
-    if (!isBundle(file))
-        return false;
-
-    int index = file.indexOf(QLatin1Char('/'), 9);
-
-    if (index == -1)
-        return false;
-
-    QStringRef identifier(&file, 9, index - 9);
-    QStringRef path(&file, index + 1, file.length() - index - 1);
-
-    QQmlBundleData *bundle = QQmlEnginePrivate::get(e)->typeLoader.getBundle(identifier);
-
-    if (bundle) {
-        const QQmlBundle::FileEntry *entry = bundle->find(path.constData(), path.length());
-        bundle->release();
-
-        return entry != 0;
-    }
-
-    return false;
-}
-
-bool QQmlFile::bundleFileExists(const QUrl &, QQmlEngine *)
-{
-    qFatal("Not implemented");
-    return true;
-}
-
-/*!
-Returns the file name for the bundle file referenced by \a url or an
-empty string if \a url isn't a bundle url.
-*/
-QString QQmlFile::bundleFileName(const QString &url, QQmlEngine *e)
-{
-    if (!isBundle(url))
-        return QString();
-
-    int index = url.indexOf(QLatin1Char('/'), 9);
-
-    if (index == -1)
-        index = url.length();
-
-    QStringRef identifier(&url, 9, index - 9);
-
-    QQmlBundleData *bundle = QQmlEnginePrivate::get(e)->typeLoader.getBundle(identifier);
-
-    if (bundle) {
-        QString rv = bundle->fileName;
-        bundle->release();
-        return rv;
-    }
-
-    return QString();
-}
-
-/*!
-Returns the file name for the bundle file referenced by \a url or an
-empty string if \a url isn't a bundle url.
-*/
-QString QQmlFile::bundleFileName(const QUrl &url, QQmlEngine *e)
-{
-    if (!isBundle(url))
-        return QString();
-
-    QQmlBundleData *bundle = QQmlEnginePrivate::get(e)->typeLoader.getBundle(url.host());
-
-    if (bundle) {
-        QString rv = bundle->fileName;
-        bundle->release();
-        return rv;
-    }
-
-    return QString();
-}
-
 QT_END_NAMESPACE
 
 #include "qqmlfile.moc"
