@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQml module of the Qt Toolkit.
 **
@@ -10,30 +10,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -44,8 +42,6 @@
 
 #include <QtQml/qtqmlglobal.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QUrl;
@@ -78,8 +74,6 @@ public:
     const char *data() const;
     QByteArray dataByteArray() const;
 
-    QByteArray metaData(const QString &) const;
-
     void load(QQmlEngine *, const QUrl &);
     void load(QQmlEngine *, const QString &);
 
@@ -94,24 +88,11 @@ public:
     static bool isSynchronous(const QString &url);
     static bool isSynchronous(const QUrl &url);
 
-    static bool isBundle(const QString &url);
-    static bool isBundle(const QUrl &url);
-
     static bool isLocalFile(const QString &url);
     static bool isLocalFile(const QUrl &url);
 
     static QString urlToLocalFileOrQrc(const QString &);
     static QString urlToLocalFileOrQrc(const QUrl &);
-
-    static bool bundleDirectoryExists(const QString &, QQmlEngine *);
-    static bool bundleDirectoryExists(const QUrl &, QQmlEngine *);
-
-    static bool bundleFileExists(const QString &, QQmlEngine *);
-    static bool bundleFileExists(const QUrl &, QQmlEngine *);
-
-    static QString bundleFileName(const QString &, QQmlEngine *);
-    static QString bundleFileName(const QUrl &, QQmlEngine *);
-
 private:
     Q_DISABLE_COPY(QQmlFile)
     QQmlFilePrivate *d;
@@ -119,6 +100,4 @@ private:
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QQMLFILE_H
