@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQml module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -34,11 +40,23 @@
 #ifndef QQMLLOCALE_H
 #define QQMLLOCALE_H
 
+//
+//  W A R N I N G
+//  -------------
+//
+// This file is not part of the Qt API.  It exists purely as an
+// implementation detail.  This header file may change from version to
+// version without notice, or even be removed.
+//
+// We mean it.
+//
+
 #include <qqml.h>
 
 #include <QtCore/qlocale.h>
 #include <QtCore/qobject.h>
 #include <private/qqmlglobal_p.h>
+#include <private/qv4object_p.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -71,13 +89,9 @@ private:
 };
 
 
-class Q_AUTOTEST_EXPORT QQmlLocale
+class Q_QML_PRIVATE_EXPORT QQmlLocale
 {
     Q_GADGET
-    Q_ENUMS(MeasurementSystem)
-    Q_ENUMS(FormatType)
-    Q_ENUMS(CurrencySymbolFormat)
-    Q_ENUMS(DayOfWeek)
 
 public:
     ~QQmlLocale();
@@ -88,16 +102,19 @@ public:
         ImperialUSSystem = QLocale::ImperialUSSystem,
         ImperialUKSystem = QLocale::ImperialUKSystem
     };
+    Q_ENUM(MeasurementSystem)
     enum FormatType {
         LongFormat = QLocale::LongFormat,
         ShortFormat = QLocale::ShortFormat,
         NarrowFormat = QLocale::NarrowFormat
     };
+    Q_ENUM(FormatType)
     enum CurrencySymbolFormat {
         CurrencyIsoCode = QLocale::CurrencyIsoCode,
         CurrencySymbol = QLocale::CurrencySymbol,
         CurrencyDisplayName = QLocale::CurrencyDisplayName
     };
+    Q_ENUM(CurrencySymbolFormat)
     // Qt defines Sunday as 7, but JS Date assigns Sunday 0
     enum DayOfWeek {
         Sunday = 0,
@@ -108,6 +125,7 @@ public:
         Friday = Qt::Friday,
         Saturday = Qt::Saturday
     };
+    Q_ENUM(DayOfWeek)
 
     static QV4::ReturnedValue locale(QV4::ExecutionEngine *engine, const QString &localeName);
     static QV4::ReturnedValue wrap(QV4::ExecutionEngine *engine, const QLocale &locale);
@@ -125,7 +143,7 @@ namespace QV4 {
 namespace Heap {
 
 struct QQmlLocaleData : Object {
-    inline QQmlLocaleData(ExecutionEngine *engine);
+    inline QQmlLocaleData() {}
     QLocale locale;
 };
 
@@ -137,7 +155,7 @@ struct QQmlLocaleData : public QV4::Object
     V4_NEEDS_DESTROY
 
     static QLocale *getThisLocale(QV4::CallContext *ctx) {
-        QV4::Object *o = ctx->thisObject().asObject();
+        QV4::Object *o = ctx->thisObject().as<Object>();
         QQmlLocaleData *thisObject = o ? o->as<QQmlLocaleData>() : 0;
         if (!thisObject) {
             ctx->engine()->throwTypeError();
@@ -175,11 +193,6 @@ struct QQmlLocaleData : public QV4::Object
     static QV4::ReturnedValue method_get_pmText(QV4::CallContext *ctx);
 };
 
-Heap::QQmlLocaleData::QQmlLocaleData(ExecutionEngine *engine)
-    : Heap::Object(engine)
-{
-}
-
 }
 
 QT_END_NAMESPACE
