@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQml module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia. For licensing terms and
-** conditions see http://qt.digia.com/licensing. For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights. These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -57,12 +63,13 @@ QT_BEGIN_NAMESPACE
 
 class QQmlType;
 class QQmlEngine;
+class QQmlEnginePrivate;
 class QQmlCustomParser;
 class QQmlTypePrivate;
 class QQmlTypeModule;
 class QHashedString;
 class QHashedStringRef;
-class QReadWriteLock;
+class QMutex;
 
 namespace QV4 { struct String; }
 
@@ -91,8 +98,8 @@ public:
     static QObject *toQObject(const QVariant &, bool *ok = 0);
 
     static int listType(int);
-    static int attachedPropertiesFuncId(const QMetaObject *);
-    static QQmlAttachedPropertiesFunc attachedPropertiesFuncById(int);
+    static int attachedPropertiesFuncId(QQmlEnginePrivate *engine, const QMetaObject *);
+    static QQmlAttachedPropertiesFunc attachedPropertiesFuncById(QQmlEnginePrivate *, int);
 
     enum TypeCategory { Unknown, Object, List };
     static TypeCategory typeCategory(int);
@@ -114,14 +121,16 @@ public:
 
     static const QQmlPrivate::CachedQmlUnit *findCachedCompilationUnit(const QUrl &uri);
 
-    static bool namespaceContainsRegistrations(const QString &);
+    static bool namespaceContainsRegistrations(const QString &, int majorVersion);
 
     static void protectNamespace(const QString &);
 
     static void setTypeRegistrationNamespace(const QString &);
     static QStringList typeRegistrationFailures();
 
-    static QReadWriteLock *typeRegistrationLock();
+    static QMutex *typeRegistrationLock();
+
+    static QString prettyTypeName(const QObject *object);
 };
 
 struct QQmlMetaTypeData;
@@ -166,9 +175,9 @@ public:
     int metaObjectRevision() const;
     bool containsRevisionedAttributes() const;
 
-    QQmlAttachedPropertiesFunc attachedPropertiesFunction() const;
-    const QMetaObject *attachedPropertiesType() const;
-    int attachedPropertiesId() const;
+    QQmlAttachedPropertiesFunc attachedPropertiesFunction(QQmlEnginePrivate *engine) const;
+    const QMetaObject *attachedPropertiesType(QQmlEnginePrivate *engine) const;
+    int attachedPropertiesId(QQmlEnginePrivate *engine) const;
 
     int parserStatusCast() const;
     const char *interfaceIId() const;
@@ -191,7 +200,7 @@ public:
 
         void setQObjectApi(QQmlEngine *, QObject *);
         QObject *qobjectApi(QQmlEngine *) const;
-        void setScriptApi(QQmlEngine *, QJSValue);
+        void setScriptApi(QQmlEngine *, const QJSValue &);
         QJSValue scriptApi(QQmlEngine *) const;
 
         void init(QQmlEngine *);
@@ -204,11 +213,13 @@ public:
 
     QUrl sourceUrl() const;
 
-    int enumValue(const QHashedStringRef &, bool *ok) const;
-    int enumValue(const QHashedCStringRef &, bool *ok) const;
-    int enumValue(const QV4::String *, bool *ok) const;
+    int enumValue(QQmlEnginePrivate *engine, const QHashedStringRef &, bool *ok) const;
+    int enumValue(QQmlEnginePrivate *engine, const QHashedCStringRef &, bool *ok) const;
+    int enumValue(QQmlEnginePrivate *engine, const QV4::String *, bool *ok) const;
 private:
     QQmlType *superType() const;
+    QQmlType *resolveCompositeBaseType(QQmlEnginePrivate *engine) const;
+    int resolveCompositeEnumValue(QQmlEnginePrivate *engine, const QString &name, bool *ok) const;
     friend class QQmlTypePrivate;
     friend struct QQmlMetaTypeData;
 
