@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQml module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -35,6 +41,7 @@
 #include <private/qqmlpropertycache_p.h>
 #include <private/qqmldata_p.h>
 #include <private/qmetaobjectbuilder_p.h>
+#include <private/qv8engine_p.h>
 #include <qqmlengine.h>
 #include <qdebug.h>
 
@@ -106,6 +113,28 @@ QMetaObject *QQmlOpenMetaObjectType::metaObject() const
     return d->mem;
 }
 
+void QQmlOpenMetaObjectType::createProperties(const QVector<QByteArray> &names)
+{
+    for (int i = 0; i < names.count(); ++i) {
+        const QByteArray &name = names.at(i);
+        const int id = d->mob.propertyCount();
+        d->mob.addSignal("__" + QByteArray::number(id) + "()");
+        QMetaPropertyBuilder build = d->mob.addProperty(name, "QVariant", id);
+        propertyCreated(id, build);
+        d->names.insert(name, id);
+    }
+    free(d->mem);
+    d->mem = d->mob.toMetaObject();
+    QSet<QQmlOpenMetaObject*>::iterator it = d->referers.begin();
+    while (it != d->referers.end()) {
+        QQmlOpenMetaObject *omo = *it;
+        *static_cast<QMetaObject *>(omo) = *d->mem;
+        if (d->cache)
+            d->cache->update(omo);
+        ++it;
+    }
+}
+
 int QQmlOpenMetaObjectType::createProperty(const QByteArray &name)
 {
     int id = d->mob.propertyCount();
@@ -230,8 +259,18 @@ QQmlOpenMetaObjectType *QQmlOpenMetaObject::type() const
     return d->type;
 }
 
-int QQmlOpenMetaObject::metaCall(QMetaObject::Call c, int id, void **a)
+void QQmlOpenMetaObject::emitPropertyNotification(const QByteArray &propertyName)
 {
+    QHash<QByteArray, int>::ConstIterator iter = d->type->d->names.constFind(propertyName);
+    if (iter == d->type->d->names.constEnd())
+        return;
+    activate(d->object, *iter + d->type->d->signalOffset, 0);
+}
+
+int QQmlOpenMetaObject::metaCall(QObject *o, QMetaObject::Call c, int id, void **a)
+{
+    Q_ASSERT(d->object == o);
+
     if (( c == QMetaObject::ReadProperty || c == QMetaObject::WriteProperty)
             && id >= d->type->d->propertyOffset) {
         int propId = id - d->type->d->propertyOffset;
@@ -239,21 +278,21 @@ int QQmlOpenMetaObject::metaCall(QMetaObject::Call c, int id, void **a)
             propertyRead(propId);
             *reinterpret_cast<QVariant *>(a[0]) = d->getData(propId);
         } else if (c == QMetaObject::WriteProperty) {
-            if (propId <= d->data.count() || d->data[propId].first != *reinterpret_cast<QVariant *>(a[0]))  {
+            if (propId >= d->data.count() || d->data[propId].first != *reinterpret_cast<QVariant *>(a[0]))  {
                 propertyWrite(propId);
                 QPair<QVariant, bool> &prop = d->getDataRef(propId);
                 prop.first = propertyWriteValue(propId, *reinterpret_cast<QVariant *>(a[0]));
                 prop.second = true;
                 propertyWritten(propId);
-                activate(d->object, d->type->d->signalOffset + propId, 0);
+                activate(o, d->type->d->signalOffset + propId, 0);
             }
         }
         return -1;
     } else {
         if (d->parent)
-            return d->parent->metaCall(c, id, a);
+            return d->parent->metaCall(o, c, id, a);
         else
-            return d->object->qt_metacall(c, id, a);
+            return o->qt_metacall(c, id, a);
     }
 }
 
@@ -277,8 +316,8 @@ void QQmlOpenMetaObject::setValue(int id, const QVariant &value)
 
 QVariant QQmlOpenMetaObject::value(const QByteArray &name) const
 {
-    QHash<QByteArray, int>::ConstIterator iter = d->type->d->names.find(name);
-    if (iter == d->type->d->names.end())
+    QHash<QByteArray, int>::ConstIterator iter = d->type->d->names.constFind(name);
+    if (iter == d->type->d->names.cend())
         return QVariant();
 
     return d->getData(*iter);
@@ -286,8 +325,8 @@ QVariant QQmlOpenMetaObject::value(const QByteArray &name) const
 
 QVariant &QQmlOpenMetaObject::operator[](const QByteArray &name)
 {
-    QHash<QByteArray, int>::ConstIterator iter = d->type->d->names.find(name);
-    Q_ASSERT(iter != d->type->d->names.end());
+    QHash<QByteArray, int>::ConstIterator iter = d->type->d->names.constFind(name);
+    Q_ASSERT(iter != d->type->d->names.cend());
 
     return d->getData(*iter);
 }
@@ -299,10 +338,10 @@ QVariant &QQmlOpenMetaObject::operator[](int id)
 
 bool QQmlOpenMetaObject::setValue(const QByteArray &name, const QVariant &val)
 {
-    QHash<QByteArray, int>::ConstIterator iter = d->type->d->names.find(name);
+    QHash<QByteArray, int>::ConstIterator iter = d->type->d->names.constFind(name);
 
     int id = -1;
-    if (iter == d->type->d->names.end()) {
+    if (iter == d->type->d->names.cend()) {
         id = createProperty(name.constData(), "") - d->type->d->propertyOffset;
     } else {
         id = *iter;
@@ -337,7 +376,7 @@ void QQmlOpenMetaObject::setCached(bool c)
     QQmlData *qmldata = QQmlData::get(d->object, true);
     if (d->cacheProperties) {
         if (!d->type->d->cache)
-            d->type->d->cache = new QQmlPropertyCache(d->type->d->engine, this);
+            d->type->d->cache = new QQmlPropertyCache(QV8Engine::getV4(d->type->d->engine), this);
         qmldata->propertyCache = d->type->d->cache;
         d->type->d->cache->addref();
     } else {
