@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQml module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -50,9 +56,9 @@
 #include "qqmlvaluetypeproxybinding_p.h"
 #include <private/qjsvalue_p.h>
 #include <private/qv4functionobject_p.h>
-#include <private/qv4runtime_p.h>
 
 #include <QStringList>
+#include <QVector>
 #include <private/qmetaobject_p.h>
 #include <private/qqmlvaluetypewrapper_p.h>
 #include <QtCore/qdebug.h>
@@ -235,14 +241,14 @@ void QQmlPropertyPrivate::initProperty(QObject *obj, const QString &name)
 
     QQmlTypeNameCache *typeNameCache = context?context->imports:0;
 
-    QStringList path = name.split(QLatin1Char('.'));
+    const auto path = name.splitRef(QLatin1Char('.'));
     if (path.isEmpty()) return;
 
     QObject *currentObject = obj;
 
     // Everything up to the last property must be an "object type" property
     for (int ii = 0; ii < path.count() - 1; ++ii) {
-        const QString &pathName = path.at(ii);
+        const QStringRef &pathName = path.at(ii);
 
         if (typeNameCache) {
             QQmlTypeNameCache::Result r = typeNameCache->query(pathName);
@@ -279,7 +285,7 @@ void QQmlPropertyPrivate::initProperty(QObject *obj, const QString &name)
 
         QQmlPropertyData local;
         QQmlPropertyData *property =
-            QQmlPropertyCache::property(engine, currentObject, pathName, context, local);
+            QQmlPropertyCache::property(engine, currentObject, pathName.toString(), context, local);
 
         if (!property) return; // Not a property
         if (property->isFunction())
@@ -319,14 +325,14 @@ void QQmlPropertyPrivate::initProperty(QObject *obj, const QString &name)
 
     }
 
-    const QString &terminal = path.last();
+    const QStringRef &terminal = path.last();
 
     if (terminal.count() >= 3 &&
         terminal.at(0) == QLatin1Char('o') &&
         terminal.at(1) == QLatin1Char('n') &&
         terminal.at(2).isUpper()) {
 
-        QString signalName = terminal.mid(2);
+        QString signalName = terminal.mid(2).toString();
         signalName[0] = signalName.at(0).toLower();
 
         // XXX - this code treats methods as signals
@@ -371,13 +377,14 @@ void QQmlPropertyPrivate::initProperty(QObject *obj, const QString &name)
     }
 
     // Property
+    const QString terminalString = terminal.toString();
     QQmlPropertyData local;
     QQmlPropertyData *property =
-        QQmlPropertyCache::property(engine, currentObject, terminal, context, local);
+        QQmlPropertyCache::property(engine, currentObject, terminalString, context, local);
     if (property && !property->isFunction()) {
         object = currentObject;
         core = *property;
-        nameCache = terminal;
+        nameCache = terminalString;
         isNameCached = true;
     }
 }
@@ -791,12 +798,12 @@ void QQmlPropertyPrivate::removeBinding(const QQmlProperty &that)
 QQmlAbstractBinding *
 QQmlPropertyPrivate::binding(QObject *object, int index)
 {
-    findAliasTarget(object, index, &object, &index);
-
     QQmlData *data = QQmlData::get(object);
     if (!data)
         return 0;
 
+    findAliasTarget(object, index, &object, &index);
+
     int coreIndex;
     int valueTypeIndex = QQmlPropertyData::decodeValueTypePropertyIndex(index, &coreIndex);
 
@@ -1332,12 +1339,7 @@ bool QQmlPropertyPrivate::write(QObject *object,
 
         if (!ok) {
             v = value;
-            if (variantType == QVariant::Double && propertyType == QVariant::String) {
-                QString number;
-                QV4::RuntimeHelpers::numberToString(&number, v.toDouble());
-                v = number;
-                ok = true;
-            } else if (v.convert(propertyType)) {
+            if (v.convert(propertyType)) {
                 ok = true;
             } else if (v.isValid() && value.isNull()) {
                 // For historical reasons converting a null QVariant to another type will do the trick
