@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQml module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -449,12 +455,7 @@ QQmlPropertyCache *QQmlPropertyCache::parent() const
 
 void QQmlPropertyCache::setParent(QQmlPropertyCache *newParent)
 {
-    if (_parent == newParent)
-        return;
-    if (_parent)
-        _parent->release();
     _parent = newParent;
-    _parent->addref();
 }
 
 // Returns the first C++ type's QMetaObject - that is, the first QMetaObject not created by
@@ -703,21 +704,12 @@ void QQmlPropertyCache::append(const QMetaObject *metaObject,
         if (accessorProperty) {
             data->flags |= QQmlPropertyData::HasAccessors;
             data->accessors = accessorProperty->accessors;
-            data->accessorData = accessorProperty->data;
         } else if (old) {
             data->markAsOverrideOf(old);
         }
     }
 }
 
-QQmlPropertyData *QQmlPropertyCache::ensureResolved(QQmlPropertyData *p) const
-{
-    if (p && p->notFullyResolved())
-        resolve(p);
-
-    return p;
-}
-
 void QQmlPropertyCache::resolve(QQmlPropertyData *data) const
 {
     Q_ASSERT(data->notFullyResolved());
@@ -845,19 +837,6 @@ int QQmlPropertyCache::methodIndexToSignalIndex(int index) const
     return index - methodIndexCacheStart + signalHandlerIndexCacheStart;
 }
 
-QQmlPropertyData *
-QQmlPropertyCache::property(int index) const
-{
-    if (index < 0 || index >= (propertyIndexCacheStart + propertyIndexCache.count()))
-        return 0;
-
-    if (index < propertyIndexCacheStart)
-        return _parent->property(index);
-
-    QQmlPropertyData *rv = const_cast<QQmlPropertyData *>(&propertyIndexCache.at(index - propertyIndexCacheStart));
-    return ensureResolved(rv);
-}
-
 QQmlPropertyData *
 QQmlPropertyCache::method(int index) const
 {
@@ -1067,11 +1046,8 @@ int QQmlPropertyCache::originalClone(QObject *object, int index)
     QQmlData *data = QQmlData::get(object, false);
     if (data && data->propertyCache) {
         QQmlPropertyCache *cache = data->propertyCache;
-        QQmlPropertyData *sig = cache->signal(index);
-        while (sig && sig->isCloned()) {
+        while (cache->signal(index)->isCloned())
             --index;
-            sig = cache->signal(index);
-        }
     } else {
         while (QMetaObjectPrivate::signal(object->metaObject(), index).attributes() & QMetaMethod::Cloned)
             --index;
