@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQml module of the Qt Toolkit.
 **
@@ -10,30 +10,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -41,7 +39,6 @@
 
 #include "qqmlstringconverters_p.h"
 #include <private/qqmlglobal_p.h>
-#include <private/qqmlinstruction_p.h>
 
 #include <QtCore/qpoint.h>
 #include <QtCore/qrect.h>
@@ -146,8 +143,8 @@ QPointF QQmlStringConverters::pointFFromString(const QString &s, bool *ok)
 
     bool xGood, yGood;
     int index = s.indexOf(QLatin1Char(','));
-    qreal xCoord = s.left(index).toDouble(&xGood);
-    qreal yCoord = s.mid(index+1).toDouble(&yGood);
+    qreal xCoord = s.leftRef(index).toDouble(&xGood);
+    qreal yCoord = s.midRef(index+1).toDouble(&yGood);
     if (!xGood || !yGood) {
         if (ok)
             *ok = false;
@@ -170,8 +167,8 @@ QSizeF QQmlStringConverters::sizeFFromString(const QString &s, bool *ok)
 
     bool wGood, hGood;
     int index = s.indexOf(QLatin1Char('x'));
-    qreal width = s.left(index).toDouble(&wGood);
-    qreal height = s.mid(index+1).toDouble(&hGood);
+    qreal width = s.leftRef(index).toDouble(&wGood);
+    qreal height = s.midRef(index+1).toDouble(&hGood);
     if (!wGood || !hGood) {
         if (ok)
             *ok = false;
@@ -194,12 +191,12 @@ QRectF QQmlStringConverters::rectFFromString(const QString &s, bool *ok)
 
     bool xGood, yGood, wGood, hGood;
     int index = s.indexOf(QLatin1Char(','));
-    qreal x = s.left(index).toDouble(&xGood);
+    qreal x = s.leftRef(index).toDouble(&xGood);
     int index2 = s.indexOf(QLatin1Char(','), index+1);
-    qreal y = s.mid(index+1, index2-index-1).toDouble(&yGood);
+    qreal y = s.midRef(index+1, index2-index-1).toDouble(&yGood);
     index = s.indexOf(QLatin1Char('x'), index2+1);
-    qreal width = s.mid(index2+1, index-index2-1).toDouble(&wGood);
-    qreal height = s.mid(index+1).toDouble(&hGood);
+    qreal width = s.midRef(index2+1, index-index2-1).toDouble(&wGood);
+    qreal height = s.midRef(index+1).toDouble(&hGood);
     if (!xGood || !yGood || !wGood || !hGood) {
         if (ok)
             *ok = false;
