@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQml module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -41,6 +47,7 @@
 #include <private/qjsvalue_p.h>
 #include <private/qv4functionobject_p.h>
 #include <private/qv4objectproto_p.h>
+#include <private/qv4qobjectwrapper_p.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -48,12 +55,8 @@ using namespace QV4;
 
 DEFINE_OBJECT_VTABLE(QmlTypeWrapper);
 
-Heap::QmlTypeWrapper::QmlTypeWrapper(ExecutionEngine *engine)
-    : Heap::Object(engine)
-    , mode(IncludeEnums)
-    , type(Q_NULLPTR)
-    , typeNamespace(Q_NULLPTR)
-    , importNamespace(Q_NULLPTR)
+Heap::QmlTypeWrapper::QmlTypeWrapper()
+    : mode(IncludeEnums)
 {
 }
 
@@ -103,7 +106,7 @@ ReturnedValue QmlTypeWrapper::create(QV4::ExecutionEngine *engine, QObject *o, Q
     Q_ASSERT(t);
     Scope scope(engine);
 
-    Scoped<QmlTypeWrapper> w(scope, engine->memoryManager->alloc<QmlTypeWrapper>(engine));
+    Scoped<QmlTypeWrapper> w(scope, engine->memoryManager->allocObject<QmlTypeWrapper>());
     w->d()->mode = mode; w->d()->object = o; w->d()->type = t;
     return w.asReturnedValue();
 }
@@ -117,31 +120,53 @@ ReturnedValue QmlTypeWrapper::create(QV4::ExecutionEngine *engine, QObject *o, Q
     Q_ASSERT(importNamespace);
     Scope scope(engine);
 
-    Scoped<QmlTypeWrapper> w(scope, engine->memoryManager->alloc<QmlTypeWrapper>(engine));
+    Scoped<QmlTypeWrapper> w(scope, engine->memoryManager->allocObject<QmlTypeWrapper>());
     w->d()->mode = mode; w->d()->object = o; w->d()->typeNamespace = t; w->d()->importNamespace = importNamespace;
     t->addref();
     return w.asReturnedValue();
 }
 
+static int enumForSingleton(String *name, QObject *qobjectSingleton)
+{
+    // ### Optimize
+    QByteArray enumName = name->toQString().toUtf8();
+    const QMetaObject *metaObject = qobjectSingleton->metaObject();
+    for (int ii = metaObject->enumeratorCount() - 1; ii >= 0; --ii) {
+        QMetaEnum e = metaObject->enumerator(ii);
+        bool ok;
+        int value = e.keyToValue(enumName.constData(), &ok);
+        if (ok)
+            return value;
+    }
+    return -1;
+}
 
-ReturnedValue QmlTypeWrapper::get(Managed *m, String *name, bool *hasProperty)
+static ReturnedValue throwLowercaseEnumError(QV4::ExecutionEngine *v4, String *name, QQmlType *type)
+{
+    const QString message =
+            QStringLiteral("Cannot access enum value '%1' of '%2', enum values need to start with an uppercase letter.")
+                .arg(name->toQString()).arg(QLatin1String(type->typeName()));
+    return v4->throwTypeError(message);
+}
+
+ReturnedValue QmlTypeWrapper::get(const Managed *m, String *name, bool *hasProperty)
 {
     Q_ASSERT(m->as<QmlTypeWrapper>());
 
-    QV4::ExecutionEngine *v4 = static_cast<QmlTypeWrapper *>(m)->engine();
+    QV4::ExecutionEngine *v4 = static_cast<const QmlTypeWrapper *>(m)->engine();
     QV4::Scope scope(v4);
 
-    Scoped<QmlTypeWrapper> w(scope, static_cast<QmlTypeWrapper *>(m));
+    Scoped<QmlTypeWrapper> w(scope, static_cast<const QmlTypeWrapper *>(m));
 
     if (hasProperty)
         *hasProperty = true;
 
-    QQmlContextData *context = v4->v8Engine->callingContext();
+    QQmlContextData *context = v4->callingQmlContext();
 
     QObject *object = w->d()->object;
+    QQmlType *type = w->d()->type;
 
-    if (w->d()->type) {
-        QQmlType *type = w->d()->type;
+    if (type) {
 
         // singleton types are handled differently to other types.
         if (type->isSingleton()) {
@@ -151,24 +176,29 @@ ReturnedValue QmlTypeWrapper::get(Managed *m, String *name, bool *hasProperty)
 
             QObject *qobjectSingleton = siinfo->qobjectApi(e);
             if (qobjectSingleton) {
+
                 // check for enum value
-                if (name->startsWithUpper()) {
-                    if (w->d()->mode == Heap::QmlTypeWrapper::IncludeEnums) {
-                        // ### Optimize
-                        QByteArray enumName = name->toQString().toUtf8();
-                        const QMetaObject *metaObject = qobjectSingleton->metaObject();
-                        for (int ii = metaObject->enumeratorCount() - 1; ii >= 0; --ii) {
-                            QMetaEnum e = metaObject->enumerator(ii);
-                            bool ok;
-                            int value = e.keyToValue(enumName.constData(), &ok);
-                            if (ok)
-                                return QV4::Primitive::fromInt32(value).asReturnedValue();
-                        }
-                    }
+                const bool includeEnums = w->d()->mode == Heap::QmlTypeWrapper::IncludeEnums;
+                if (includeEnums && name->startsWithUpper()) {
+                    const int value = enumForSingleton(name, qobjectSingleton);
+                    if (value != -1)
+                        return QV4::Primitive::fromInt32(value).asReturnedValue();
                 }
 
                 // check for property.
-                return QV4::QObjectWrapper::getQmlProperty(v4, context, qobjectSingleton, name, QV4::QObjectWrapper::IgnoreRevision, hasProperty);
+                bool ok;
+                const ReturnedValue result = QV4::QObjectWrapper::getQmlProperty(v4, context, qobjectSingleton, name, QV4::QObjectWrapper::IgnoreRevision, &ok);
+                if (hasProperty)
+                    *hasProperty = ok;
+
+                // Warn when attempting to access a lowercased enum value, singleton case
+                if (!ok && includeEnums && !name->startsWithUpper()) {
+                    const int value = enumForSingleton(name, qobjectSingleton);
+                    if (value != -1)
+                        return throwLowercaseEnumError(v4, name, type);
+                }
+
+                return result;
             } else if (!siinfo->scriptApi(e).isUndefined()) {
                 // NOTE: if used in a binding, changes will not trigger re-evaluation since non-NOTIFYable.
                 QV4::ScopedObject o(scope, QJSValuePrivate::convertedToValue(v4, siinfo->scriptApi(e)));
@@ -182,14 +212,14 @@ ReturnedValue QmlTypeWrapper::get(Managed *m, String *name, bool *hasProperty)
 
             if (name->startsWithUpper()) {
                 bool ok = false;
-                int value = type->enumValue(name, &ok);
+                int value = type->enumValue(QQmlEnginePrivate::get(v4->qmlEngine()), name, &ok);
                 if (ok)
                     return QV4::Primitive::fromInt32(value).asReturnedValue();
 
                 // Fall through to base implementation
 
             } else if (w->d()->object) {
-                QObject *ao = qmlAttachedPropertiesObjectById(type->attachedPropertiesId(), object);
+                QObject *ao = qmlAttachedPropertiesObjectById(type->attachedPropertiesId(QQmlEnginePrivate::get(v4->qmlEngine())), object);
                 if (ao)
                     return QV4::QObjectWrapper::getQmlProperty(v4, context, ao, name, QV4::QObjectWrapper::IgnoreRevision, hasProperty);
 
@@ -225,9 +255,20 @@ ReturnedValue QmlTypeWrapper::get(Managed *m, String *name, bool *hasProperty)
         Q_ASSERT(!"Unreachable");
     }
 
+    bool ok = false;
+    const ReturnedValue result = Object::get(m, name, &ok);
     if (hasProperty)
-        *hasProperty = false;
-    return Object::get(m, name, hasProperty);
+        *hasProperty = ok;
+
+    // Warn when attempting to access a lowercased enum value, non-singleton case
+    if (!ok && type && !type->isSingleton() && !name->startsWithUpper()) {
+        bool enumOk = false;
+        type->enumValue(QQmlEnginePrivate::get(v4->qmlEngine()), name, &enumOk);
+        if (enumOk)
+            return throwLowercaseEnumError(v4, name, type);
+    }
+
+    return result;
 }
 
 
@@ -240,12 +281,13 @@ void QmlTypeWrapper::put(Managed *m, String *name, const Value &value)
         return;
 
     QV4::Scope scope(v4);
-    QQmlContextData *context = v4->v8Engine->callingContext();
+    QQmlContextData *context = v4->callingQmlContext();
 
     QQmlType *type = w->d()->type;
     if (type && !type->isSingleton() && w->d()->object) {
         QObject *object = w->d()->object;
-        QObject *ao = qmlAttachedPropertiesObjectById(type->attachedPropertiesId(), object);
+        QQmlEngine *e = scope.engine->qmlEngine();
+        QObject *ao = qmlAttachedPropertiesObjectById(type->attachedPropertiesId(QQmlEnginePrivate::get(e)), object);
         if (ao)
             QV4::QObjectWrapper::setQmlProperty(v4, context, ao, name, QV4::QObjectWrapper::IgnoreRevision, value);
     } else if (type && type->isSingleton()) {
