@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQml module of the Qt Toolkit.
 **
@@ -10,30 +10,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -55,22 +53,53 @@
 
 #include <private/qqmlglobal_p.h>
 #include <private/qv4functionobject_p.h>
+#include <private/qjsengine_p.h>
 
 QT_BEGIN_NAMESPACE
 
 class QQmlEngine;
-class QV8Engine;
 
 namespace QV4 {
 
+namespace Heap {
+
+struct QtObject : Object {
+    QtObject(QQmlEngine *qmlEngine);
+    QObject *platform;
+    QObject *application;
+
+    enum { Finished = -1 };
+    int enumeratorIterator;
+    int keyIterator;
+
+    bool isComplete() const
+    { return enumeratorIterator == Finished; }
+};
+
+struct ConsoleObject : Object {
+    ConsoleObject();
+};
+
+struct QQmlBindingFunction : FunctionObject {
+    QQmlBindingFunction(const QV4::FunctionObject *originalFunction);
+    Pointer<FunctionObject> originalFunction;
+    // Set when the binding is created later
+    QQmlSourceLocation bindingLocation;
+};
+
+}
+
 struct QtObject : Object
 {
-    V4_OBJECT
-    QtObject(ExecutionEngine *v4, QQmlEngine *qmlEngine);
+    V4_OBJECT2(QtObject, Object)
+
+    static ReturnedValue get(const Managed *m, String *name, bool *hasProperty);
+    static void advanceIterator(Managed *m, ObjectIterator *it, Value *name, uint *index, Property *p, PropertyAttributes *attributes);
 
     static ReturnedValue method_isQtObject(CallContext *ctx);
     static ReturnedValue method_rgba(CallContext *ctx);
     static ReturnedValue method_hsla(CallContext *ctx);
+    static ReturnedValue method_hsva(CallContext *ctx);
     static ReturnedValue method_colorEqual(CallContext *ctx);
     static ReturnedValue method_font(CallContext *ctx);
     static ReturnedValue method_rect(CallContext *ctx);
@@ -104,17 +133,22 @@ struct QtObject : Object
 #ifndef QT_NO_IM
     static ReturnedValue method_get_inputMethod(CallContext *ctx);
 #endif
+    static ReturnedValue method_get_styleHints(CallContext *ctx);
 
-    QObject *m_platform;
-    QObject *m_application;
+private:
+    void addAll();
+    ReturnedValue findAndAdd(const QString *name, bool &foundProperty) const;
 };
 
 struct ConsoleObject : Object
 {
-    ConsoleObject(ExecutionEngine *v4);
+    typedef Heap::ConsoleObject Data;
+    const Data *d() const { return static_cast<const Data *>(Object::d()); }
+    Data *d() { return static_cast<Data *>(Object::d()); }
 
     static ReturnedValue method_error(CallContext *ctx);
     static ReturnedValue method_log(CallContext *ctx);
+    static ReturnedValue method_info(CallContext *ctx);
     static ReturnedValue method_profile(CallContext *ctx);
     static ReturnedValue method_profileEnd(CallContext *ctx);
     static ReturnedValue method_time(CallContext *ctx);
@@ -128,7 +162,7 @@ struct ConsoleObject : Object
 };
 
 struct GlobalExtensions {
-    static void init(QQmlEngine *qmlEngine, Object *globalObject);
+    static void init(Object *globalObject, QJSEngine::Extensions extensions);
 
 #ifndef QT_NO_TRANSLATION
     static ReturnedValue method_qsTranslate(CallContext *ctx);
@@ -147,18 +181,14 @@ struct GlobalExtensions {
 
 struct QQmlBindingFunction : public QV4::FunctionObject
 {
-    V4_OBJECT
-    QQmlBindingFunction(FunctionObject *originalFunction);
+    V4_OBJECT2(QQmlBindingFunction, FunctionObject)
+    V4_NEEDS_DESTROY
 
     void initBindingLocation(); // from caller stack trace
 
-    static ReturnedValue call(Managed *that, CallData *callData);
+    static ReturnedValue call(const Managed *that, CallData *callData);
 
-    static void markObjects(Managed *that, ExecutionEngine *e);
-
-    QV4::FunctionObject *originalFunction;
-    // Set when the binding is created later
-    QQmlSourceLocation bindingLocation;
+    static void markObjects(Heap::Base *that, ExecutionEngine *e);
 };
 
 }
