@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -34,6 +40,7 @@
 #include "quicktestresult_p.h"
 #include <QtTest/qtestcase.h>
 #include <QtTest/qtestsystem.h>
+#include <QtTest/private/qtestblacklist_p.h>
 #include <QtTest/private/qtestresult_p.h>
 #include <QtTest/private/qtesttable_p.h>
 #include <QtTest/private/qtestlog_p.h>
@@ -50,6 +57,7 @@
 #include <QtQuick/qquickwindow.h>
 #include <QtGui/qvector3d.h>
 #include <QtQml/private/qqmlglobal_p.h>
+#include <private/qv4qobjectwrapper_p.h>
 
 #include <algorithm>
 
@@ -104,9 +112,15 @@ public Q_SLOTS:
          || x * y >= m_image.width() * m_image.height())
             return QVariant();
 
-        const QRgb* pixel = reinterpret_cast<const QRgb*>(m_image.constScanLine(y));
-        pixel += x;
-        return QColor::fromRgba(*pixel);
+        return QColor::fromRgba(m_image.pixel(QPoint(x, y)));
+    }
+
+    bool equals(QuickTestImageObject *other) const
+    {
+        if (!other)
+            return m_image.isNull();
+
+        return m_image == other->m_image;
     }
 private:
     QImage m_image;
@@ -205,6 +219,7 @@ void QuickTestResult::setFunctionName(const QString &name)
             QString fullName = d->testCaseName + QLatin1String("::") + name;
             QTestResult::setCurrentTestFunction
                 (d->intern(fullName).constData());
+            QTestPrivate::checkBlackLists(fullName.toUtf8().constData(), 0);
         }
     } else {
         QTestResult::setCurrentTestFunction(0);
@@ -233,6 +248,7 @@ void QuickTestResult::setDataTag(const QString &tag)
     if (!tag.isEmpty()) {
         QTestData *data = &(QTest::newRow(tag.toUtf8().constData()));
         QTestResult::setCurrentTestData(data);
+        QTestPrivate::checkBlackLists((testCaseName() + QStringLiteral("::") + functionName()).toUtf8().constData(), tag.toUtf8().constData());
         emit dataTagChanged();
     } else {
         QTestResult::setCurrentTestData(0);
@@ -270,6 +286,8 @@ bool QuickTestResult::isSkipped() const
 void QuickTestResult::setSkipped(bool skip)
 {
     QTestResult::setSkipCurrentTest(skip);
+    if (!skip)
+        QTestResult::setBlacklistCurrentTest(false);
     emit skippedChanged();
 }
 
@@ -478,8 +496,8 @@ void QuickTestResult::stringify(QQmlV4Function *args)
 
     //Check for Object Type
     if (value->isObject()
-        && !value->asFunctionObject()
-        && !value->asArrayObject()) {
+        && !value->as<QV4::FunctionObject>()
+        && !value->as<QV4::ArrayObject>()) {
         QVariant v = scope.engine->toVariant(value, QMetaType::UnknownType);
         if (v.isValid()) {
             switch (v.type()) {
@@ -500,7 +518,7 @@ void QuickTestResult::stringify(QQmlV4Function *args)
 
     if (result.isEmpty()) {
         QString tmp = value->toQStringNoThrow();
-        if (value->asArrayObject())
+        if (value->as<QV4::ArrayObject>())
             result.append(QString::fromLatin1("[%1]").arg(tmp));
         else
             result.append(tmp);
@@ -707,6 +725,8 @@ void QuickTestResult::parseArgs(int argc, char *argv[])
 void QuickTestResult::setProgramName(const char *name)
 {
     if (name) {
+        QTestPrivate::parseBlackList();
+        QTestPrivate::parseGpuBlackList();
         QTestResult::reset();
     } else if (!name && loggingStarted) {
         QTestResult::setCurrentTestObject(globalProgramName);
