@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
@@ -10,30 +10,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -42,6 +40,17 @@
 #ifndef QQUICKACCESSIBLEATTACHED_H
 #define QQUICKACCESSIBLEATTACHED_H
 
+//
+//  W A R N I N G
+//  -------------
+//
+// This file is not part of the Qt API.  It exists purely as an
+// implementation detail.  This header file may change from version to
+// version without notice, or even be removed.
+//
+// We mean it.
+//
+
 #include <QtQuick/qquickitem.h>
 
 #include <QtCore/qobject.h>
@@ -78,6 +87,7 @@ class Q_QUICK_PRIVATE_EXPORT QQuickAccessibleAttached : public QObject
     Q_PROPERTY(QAccessible::Role role READ role WRITE setRole NOTIFY roleChanged FINAL)
     Q_PROPERTY(QString name READ name WRITE setName NOTIFY nameChanged FINAL)
     Q_PROPERTY(QString description READ description WRITE setDescription NOTIFY descriptionChanged FINAL)
+    Q_PROPERTY(bool ignored READ ignored WRITE setIgnored NOTIFY ignoredChanged FINAL)
 
 public:
     Q_ENUMS(QAccessible::Role QAccessible::Event)
@@ -95,6 +105,7 @@ public:
     STATE_PROPERTY(defaultButton)
     STATE_PROPERTY(passwordEdit)
     STATE_PROPERTY(selectableText)
+    STATE_PROPERTY(searchEdit)
 
     QQuickAccessibleAttached(QObject *parent);
     ~QQuickAccessibleAttached();
@@ -193,6 +204,9 @@ public:
     }
 
     QAccessible::State state() { return m_state; }
+    bool ignored() const;
+    bool doAction(const QString &actionName);
+    void availableActions(QStringList *actions) const;
 
 public Q_SLOTS:
     void valueChanged() {
@@ -204,10 +218,23 @@ public Q_SLOTS:
         QAccessible::updateAccessibility(&ev);
     }
 
+    void setIgnored(bool ignored);
+
 Q_SIGNALS:
     void roleChanged();
     void nameChanged();
     void descriptionChanged();
+    void ignoredChanged();
+    void pressAction();
+    void toggleAction();
+    void increaseAction();
+    void decreaseAction();
+    void scrollUpAction();
+    void scrollDownAction();
+    void scrollLeftAction();
+    void scrollRightAction();
+    void previousPageAction();
+    void nextPageAction();
 
 private:
     QQuickItem *item() const { return static_cast<QQuickItem*>(parent()); }
@@ -217,6 +244,17 @@ private:
     QString m_name;
     QString m_description;
 
+    static QMetaMethod sigPress;
+    static QMetaMethod sigToggle;
+    static QMetaMethod sigIncrease;
+    static QMetaMethod sigDecrease;
+    static QMetaMethod sigScrollUp;
+    static QMetaMethod sigScrollDown;
+    static QMetaMethod sigScrollLeft;
+    static QMetaMethod sigScrollRight;
+    static QMetaMethod sigPreviousPage;
+    static QMetaMethod sigNextPage;
+
 public:
     using QObject::property;
 };
