@@ -379,7 +379,7 @@ Item {
     \qmlproperty point QtQuick::WheelEvent::pixelDelta
 
     This property holds the delta in screen pixels and is available in platforms that
-    have high-resolution trackpads, such as OS X.
+    have high-resolution trackpads, such as \macos.
     The x and y cordinate of this property holds the delta in horizontal and
     vertical orientation. The value should be used directly to scroll content on screen.
 
