@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -152,8 +158,10 @@ void QQuickFlipable::setFront(QQuickItem *front)
     }
     d->front = front;
     d->front->setParentItem(this);
-    if (Back == d->current)
+    if (Back == d->current) {
         d->front->setOpacity(0.);
+        d->front->setEnabled(false);
+    }
     emit frontChanged();
 }
 
@@ -178,8 +186,11 @@ void QQuickFlipable::setBack(QQuickItem *back)
     d->backTransform = new QQuickLocalTransform(d->back);
     d->backTransform->prependToItem(d->back);
 
-    if (Front == d->current)
+    if (Front == d->current) {
         d->back->setOpacity(0.);
+        d->back->setEnabled(false);
+    }
+
     connect(back, SIGNAL(widthChanged()),
             this, SLOT(retransformBack()));
     connect(back, SIGNAL(heightChanged()),
@@ -271,10 +282,14 @@ void QQuickFlipablePrivate::updateSide()
         current = newSide;
         if (current == QQuickFlipable::Back && back)
             setBackTransform();
-        if (front)
+        if (front) {
             front->setOpacity((current==QQuickFlipable::Front)?1.:0.);
-        if (back)
+            front->setEnabled((current==QQuickFlipable::Front)?true:false);
+        }
+        if (back) {
             back->setOpacity((current==QQuickFlipable::Back)?1.:0.);
+            back->setEnabled((current==QQuickFlipable::Back)?true:false);
+        }
         emit q->sideChanged();
     }
 }
