@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
@@ -10,30 +10,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -64,9 +62,8 @@
 #include <QtQuick/qsgnode.h>
 #include "qquickclipnode_p.h"
 
-#include <private/qpodvector_p.h>
 #include <QtQuick/private/qquickstate_p.h>
-#include <private/qqmlnullablevalue_p_p.h>
+#include <private/qqmlnullablevalue_p.h>
 #include <private/qqmlnotifier_p.h>
 #include <private/qqmlglobal_p.h>
 #include <private/qlazilyallocated_p.h>
@@ -86,6 +83,7 @@ QT_BEGIN_NAMESPACE
 class QNetworkReply;
 class QQuickItemKeyFilter;
 class QQuickLayoutMirroringAttached;
+class QQuickEnterKeyAttached;
 class QQuickScreenAttached;
 
 class QQuickContents : public QQuickItemChangeListener
@@ -100,10 +98,10 @@ public:
     void complete();
 
 protected:
-    void itemGeometryChanged(QQuickItem *item, const QRectF &newGeometry, const QRectF &oldGeometry);
-    void itemDestroyed(QQuickItem *item);
-    void itemChildAdded(QQuickItem *, QQuickItem *);
-    void itemChildRemoved(QQuickItem *, QQuickItem *);
+    void itemGeometryChanged(QQuickItem *item, const QRectF &newGeometry, const QRectF &oldGeometry) Q_DECL_OVERRIDE;
+    void itemDestroyed(QQuickItem *item) Q_DECL_OVERRIDE;
+    void itemChildAdded(QQuickItem *, QQuickItem *) Q_DECL_OVERRIDE;
+    void itemChildRemoved(QQuickItem *, QQuickItem *) Q_DECL_OVERRIDE;
     //void itemVisibilityChanged(QQuickItem *item)
 
 private:
@@ -150,6 +148,7 @@ class QQuickItemLayer : public QObject, public QQuickItemChangeListener
     Q_PROPERTY(QQuickShaderEffectSource::Format format READ format WRITE setFormat NOTIFY formatChanged)
     Q_PROPERTY(QByteArray samplerName READ name WRITE setName NOTIFY nameChanged)
     Q_PROPERTY(QQmlComponent *effect READ effect WRITE setEffect NOTIFY effectChanged)
+    Q_PROPERTY(QQuickShaderEffectSource::TextureMirroring textureMirroring READ textureMirroring WRITE setTextureMirroring NOTIFY textureMirroringChanged)
 public:
     QQuickItemLayer(QQuickItem *item);
     ~QQuickItemLayer();
@@ -184,13 +183,16 @@ public:
     QQmlComponent *effect() const { return m_effectComponent; }
     void setEffect(QQmlComponent *effect);
 
+    QQuickShaderEffectSource::TextureMirroring textureMirroring() const { return m_textureMirroring; }
+    void setTextureMirroring(QQuickShaderEffectSource::TextureMirroring mirroring);
+
     QQuickShaderEffectSource *effectSource() const { return m_effectSource; }
 
-    void itemGeometryChanged(QQuickItem *, const QRectF &, const QRectF &);
-    void itemOpacityChanged(QQuickItem *);
-    void itemParentChanged(QQuickItem *, QQuickItem *);
-    void itemSiblingOrderChanged(QQuickItem *);
-    void itemVisibilityChanged(QQuickItem *);
+    void itemGeometryChanged(QQuickItem *, const QRectF &, const QRectF &) Q_DECL_OVERRIDE;
+    void itemOpacityChanged(QQuickItem *) Q_DECL_OVERRIDE;
+    void itemParentChanged(QQuickItem *, QQuickItem *) Q_DECL_OVERRIDE;
+    void itemSiblingOrderChanged(QQuickItem *) Q_DECL_OVERRIDE;
+    void itemVisibilityChanged(QQuickItem *) Q_DECL_OVERRIDE;
 
     void updateMatrix();
     void updateGeometry();
@@ -207,6 +209,7 @@ Q_SIGNALS:
     void smoothChanged(bool smooth);
     void formatChanged(QQuickShaderEffectSource::Format format);
     void sourceRectChanged(const QRectF &sourceRect);
+    void textureMirroringChanged(QQuickShaderEffectSource::TextureMirroring mirroring);
 
 private:
     friend class QQuickTransformAnimatorJob;
@@ -230,6 +233,7 @@ private:
     QQmlComponent *m_effectComponent;
     QQuickItem *m_effect;
     QQuickShaderEffectSource *m_effectSource;
+    QQuickShaderEffectSource::TextureMirroring m_textureMirroring;
 };
 
 class Q_QUICK_PRIVATE_EXPORT QQuickItemPrivate : public QObjectPrivate
@@ -297,6 +301,8 @@ public:
     static void transform_clear(QQmlListProperty<QQuickTransform> *list);
 
     void _q_resourceObjectDeleted(QObject *);
+    void _q_windowChanged(QQuickWindow *w);
+    quint64 _q_createJSWrapper(QV4::ExecutionEngine *engine);
 
     enum ChangeType {
         Geometry = 0x01,
@@ -326,7 +332,7 @@ public:
     Q_DECLARE_FLAGS(GeometryChangeTypes, GeometryChangeType)
 
     struct ChangeListener {
-        ChangeListener(QQuickItemChangeListener *l, QQuickItemPrivate::ChangeTypes t) : listener(l), types(t), gTypes(GeometryChange) {}
+        ChangeListener(QQuickItemChangeListener *l = Q_NULLPTR, QQuickItemPrivate::ChangeTypes t = 0) : listener(l), types(t), gTypes(GeometryChange) {}
         ChangeListener(QQuickItemChangeListener *l, QQuickItemPrivate::GeometryChangeTypes gt) : listener(l), types(Geometry), gTypes(gt) {}
         QQuickItemChangeListener *listener;
         QQuickItemPrivate::ChangeTypes types;
@@ -345,11 +351,11 @@ public:
         QQuickContents *contents;
         QQuickScreenAttached *screenAttached;
         QQuickLayoutMirroringAttached* layoutDirectionAttached;
+        QQuickEnterKeyAttached *enterKeyAttached;
         QQuickItemKeyFilter *keyHandler;
         mutable QQuickItemLayer *layer;
 #ifndef QT_NO_CURSOR
         QCursor cursor;
-        int numItemsWithCursor;
 #endif
         QPointF userTransformOriginPoint;
 
@@ -359,7 +365,8 @@ public:
         QSGOpacityNode *opacityNode;
         QQuickDefaultClipNode *clipNode;
         QSGRootNode *rootNode;
-        QSGNode *beforePaintNode;
+
+        QObjectList resourcesList;
 
         // Although acceptedMouseButtons is inside ExtraData, we actually store
         // the LeftButton flag in the extra.flag() bit.  This is because it is
@@ -368,8 +375,9 @@ public:
         Qt::MouseButtons acceptedMouseButtons;
 
         QQuickItem::TransformOrigin origin:5;
+        uint transparentForPositioner : 1;
 
-        QObjectList resourcesList;
+        // 26 bits padding
     };
     QLazilyAllocated<ExtraData> extra;
 
@@ -378,7 +386,7 @@ public:
 
     inline Qt::MouseButtons acceptedMouseButtons() const;
 
-    QPODVector<QQuickItemPrivate::ChangeListener,4> changeListeners;
+    QVector<QQuickItemPrivate::ChangeListener> changeListeners;
 
     void addItemChangeListener(QQuickItemChangeListener *listener, ChangeTypes types);
     void removeItemChangeListener(QQuickItemChangeListener *, ChangeTypes types);
@@ -394,7 +402,6 @@ public:
     quint32 flags:5;
     bool widthValid:1;
     bool heightValid:1;
-    bool baselineOffsetValid:1;
     bool componentComplete:1;
     bool keepMouse:1;
     bool keepTouch:1;
@@ -403,8 +410,8 @@ public:
     bool antialiasing:1;
     bool focus:1;
     bool activeFocus:1;
-    // Bit 16
     bool notifiedFocus:1;
+    // Bit 16
     bool notifiedActiveFocus:1;
     bool filtersChildMouseEvents:1;
     bool explicitVisible:1;
@@ -420,8 +427,18 @@ public:
     bool isAccessible:1;
     bool culled:1;
     bool hasCursor:1;
+    bool hasCursorInChild:1;
     // Bit 32
     bool activeFocusOnTab:1;
+    bool implicitAntialiasing:1;
+    bool antialiasingValid:1;
+    // isTabFence: When true, the item acts as a fence within the tab focus chain.
+    // This means that the item and its children will be skipped from the tab focus
+    // chain when navigating from its parent or any of its siblings. Similarly,
+    // when any of the item's descendants gets focus, the item constrains the tab
+    // focus chain and prevents tabbing outside.
+    bool isTabFence:1;
+    bool replayingPressEvent:1;
 
     enum DirtyType {
         TransformOrigin         = 0x00000001,
@@ -491,11 +508,14 @@ public:
     QTransform windowToItemTransform() const;
     QTransform itemToWindowTransform() const;
     void itemToParentTransform(QTransform &) const;
+    QTransform globalToWindowTransform() const;
+    QTransform windowToGlobalTransform() const;
 
     static bool focusNextPrev(QQuickItem *item, bool forward);
+    static QQuickItem *nextTabChildItem(const QQuickItem *item, int start);
+    static QQuickItem *prevTabChildItem(const QQuickItem *item, int start);
     static QQuickItem *nextPrevItemInTabFocusChain(QQuickItem *item, bool forward);
 
-    static bool qt_tab_all_widgets(); //todo: move to QGuiApplication?
     static bool canAcceptTabFocus(QQuickItem *item);
 
     qreal x;
@@ -514,13 +534,19 @@ public:
     inline qreal rotation() const { return extra.isAllocated()?extra->rotation:0; }
     inline qreal opacity() const { return extra.isAllocated()?extra->opacity:1; }
 
-    void setAccessibleFlagAndListener();
+    void setAccessible();
 
     virtual qreal getImplicitWidth() const;
     virtual qreal getImplicitHeight() const;
     virtual void implicitWidthChanged();
     virtual void implicitHeightChanged();
 
+#ifndef QT_NO_ACCESSIBILITY
+    virtual QAccessible::Role accessibleRole() const;
+#endif
+
+    void setImplicitAntialiasing(bool antialiasing);
+
     void resolveLayoutMirror();
     void setImplicitLayoutMirror(bool mirror, bool inherit);
     void setLayoutMirror(bool mirror);
@@ -540,16 +566,9 @@ public:
 #ifndef QT_NO_IM
     void deliverInputMethodEvent(QInputMethodEvent *);
 #endif
-    void deliverFocusEvent(QFocusEvent *);
-    void deliverMouseEvent(QMouseEvent *);
-#ifndef QT_NO_WHEELEVENT
-    void deliverWheelEvent(QWheelEvent *);
-#endif
-    void deliverTouchEvent(QTouchEvent *);
-    void deliverHoverEvent(QHoverEvent *);
-#ifndef QT_NO_DRAGANDDROP
-    void deliverDragEvent(QEvent *);
-#endif
+
+    bool isTransparentForPositioner() const;
+    void setTransparentForPositioner(bool trans);
 
     bool calcEffectiveVisible() const;
     bool setEffectiveVisibleRecur(bool);
@@ -565,8 +584,7 @@ public:
          - itemNode
          - (opacityNode)
          - (clipNode)
-         - (effectNode)
-         - groupNode
+         - (rootNode) (shader effect source's root node)
      */
 
     QSGOpacityNode *opacityNode() const { return extra.isAllocated()?extra->opacityNode:0; }
@@ -574,7 +592,6 @@ public:
     QSGRootNode *rootNode() const { return extra.isAllocated()?extra->rootNode:0; }
 
     QSGTransformNode *itemNodeInstance;
-    QSGNode *groupNode;
     QSGNode *paintNode;
 
     virtual QSGTransformNode *createTransformNode();
@@ -588,7 +605,7 @@ public:
 
     virtual void mirrorChange() {}
 
-    void incrementCursorCount(int delta);
+    void setHasCursorInChild(bool hasCursor);
 };
 
 /*
@@ -640,7 +657,7 @@ public:
     bool backtabSet : 1;
 };
 
-class QQuickKeyNavigationAttached : public QObject, public QQuickItemKeyFilter
+class Q_QUICK_PRIVATE_EXPORT QQuickKeyNavigationAttached : public QObject, public QQuickItemKeyFilter
 {
     Q_OBJECT
     Q_DECLARE_PRIVATE(QQuickKeyNavigationAttached)
@@ -653,8 +670,6 @@ class QQuickKeyNavigationAttached : public QObject, public QQuickItemKeyFilter
     Q_PROPERTY(QQuickItem *backtab READ backtab WRITE setBacktab NOTIFY backtabChanged)
     Q_PROPERTY(Priority priority READ priority WRITE setPriority NOTIFY priorityChanged)
 
-    Q_ENUMS(Priority)
-
 public:
     QQuickKeyNavigationAttached(QObject * = 0);
 
@@ -672,6 +687,7 @@ public:
     void setBacktab(QQuickItem *);
 
     enum Priority { BeforeItem, AfterItem };
+    Q_ENUM(Priority)
     Priority priority() const;
     void setPriority(Priority);
 
@@ -687,9 +703,10 @@ Q_SIGNALS:
     void priorityChanged();
 
 private:
-    virtual void keyPressed(QKeyEvent *event, bool post);
-    virtual void keyReleased(QKeyEvent *event, bool post);
-    void setFocusNavigation(QQuickItem *currentItem, const char *dir);
+    void keyPressed(QKeyEvent *event, bool post) Q_DECL_OVERRIDE;
+    void keyReleased(QKeyEvent *event, bool post) Q_DECL_OVERRIDE;
+    void setFocusNavigation(QQuickItem *currentItem, const char *dir,
+                            Qt::FocusReason reason = Qt::OtherFocusReason);
 };
 
 class QQuickLayoutMirroringAttached : public QObject
@@ -717,6 +734,27 @@ private:
     QQuickItemPrivate *itemPrivate;
 };
 
+class QQuickEnterKeyAttached : public QObject
+{
+    Q_OBJECT
+    Q_PROPERTY(Qt::EnterKeyType type READ type WRITE setType NOTIFY typeChanged)
+
+public:
+    explicit QQuickEnterKeyAttached(QObject *parent = Q_NULLPTR);
+
+    Qt::EnterKeyType type() const;
+    void setType(Qt::EnterKeyType type);
+
+    static QQuickEnterKeyAttached *qmlAttachedProperties(QObject *);
+Q_SIGNALS:
+    void typeChanged();
+private:
+    friend class QQuickItemPrivate;
+    QQuickItemPrivate *itemPrivate;
+
+    Qt::EnterKeyType keyType;
+};
+
 class QQuickKeysAttachedPrivate : public QObjectPrivate
 {
 public:
@@ -746,8 +784,6 @@ class QQuickKeysAttached : public QObject, public QQuickItemKeyFilter
     Q_PROPERTY(QQmlListProperty<QQuickItem> forwardTo READ forwardTo)
     Q_PROPERTY(Priority priority READ priority WRITE setPriority NOTIFY priorityChanged)
 
-    Q_ENUMS(Priority)
-
 public:
     QQuickKeysAttached(QObject *parent=0);
     ~QQuickKeysAttached();
@@ -762,6 +798,7 @@ public:
     }
 
     enum Priority { BeforeItem, AfterItem};
+    Q_ENUM(Priority)
     Priority priority() const;
     void setPriority(Priority);
 
@@ -770,7 +807,7 @@ public:
         return QQmlListProperty<QQuickItem>(this, d->targets);
     }
 
-    virtual void componentComplete();
+    void componentComplete() Q_DECL_OVERRIDE;
 
     static QQuickKeysAttached *qmlAttachedProperties(QObject *);
 
@@ -821,11 +858,11 @@ Q_SIGNALS:
     void volumeDownPressed(QQuickKeyEvent *event);
 
 private:
-    virtual void keyPressed(QKeyEvent *event, bool post);
-    virtual void keyReleased(QKeyEvent *event, bool post);
+    void keyPressed(QKeyEvent *event, bool post) Q_DECL_OVERRIDE;
+    void keyReleased(QKeyEvent *event, bool post) Q_DECL_OVERRIDE;
 #ifndef QT_NO_IM
-    virtual void inputMethodEvent(QInputMethodEvent *, bool post);
-    virtual QVariant inputMethodQuery(Qt::InputMethodQuery query) const;
+    void inputMethodEvent(QInputMethodEvent *, bool post) Q_DECL_OVERRIDE;
+    QVariant inputMethodQuery(Qt::InputMethodQuery query) const Q_DECL_OVERRIDE;
 #endif
     const QByteArray keyToSignal(int key);
 
@@ -873,7 +910,7 @@ QSGTransformNode *QQuickItemPrivate::itemNode()
         itemNodeInstance->setFlag(QSGNode::OwnedByParent, false);
 #ifdef QSG_RUNTIME_DESCRIPTION
         Q_Q(QQuickItem);
-        qsgnode_set_description(itemNodeInstance, QString::fromLatin1("QQuickItem(%1)").arg(QString::fromLatin1(q->metaObject()->className())));
+        qsgnode_set_description(itemNodeInstance, QString::fromLatin1("QQuickItem(%1:%2)").arg(QString::fromLatin1(q->metaObject()->className())).arg(q->objectName()));
 #endif
     }
     return itemNodeInstance;
@@ -881,24 +918,18 @@ QSGTransformNode *QQuickItemPrivate::itemNode()
 
 QSGNode *QQuickItemPrivate::childContainerNode()
 {
-    if (!groupNode) {
-        groupNode = new QSGNode();
-        if (rootNode())
-            rootNode()->appendChildNode(groupNode);
-        else if (clipNode())
-            clipNode()->appendChildNode(groupNode);
-        else if (opacityNode())
-            opacityNode()->appendChildNode(groupNode);
-        else
-            itemNode()->appendChildNode(groupNode);
-#ifdef QSG_RUNTIME_DESCRIPTION
-        qsgnode_set_description(groupNode, QLatin1String("group"));
-#endif
-    }
-    return groupNode;
+    if (rootNode())
+        return rootNode();
+    else if (clipNode())
+        return clipNode();
+    else if (opacityNode())
+        return opacityNode();
+    else
+        return itemNode();
 }
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(QQuickItemPrivate::ChangeTypes)
+Q_DECLARE_TYPEINFO(QQuickItemPrivate::ChangeListener, Q_PRIMITIVE_TYPE);
 
 QT_END_NAMESPACE
 
@@ -909,5 +940,7 @@ QML_DECLARE_TYPE(QQuickKeyNavigationAttached)
 QML_DECLARE_TYPEINFO(QQuickKeyNavigationAttached, QML_HAS_ATTACHED_PROPERTIES)
 QML_DECLARE_TYPE(QQuickLayoutMirroringAttached)
 QML_DECLARE_TYPEINFO(QQuickLayoutMirroringAttached, QML_HAS_ATTACHED_PROPERTIES)
+QML_DECLARE_TYPE(QQuickEnterKeyAttached)
+QML_DECLARE_TYPEINFO(QQuickEnterKeyAttached, QML_HAS_ATTACHED_PROPERTIES)
 
 #endif // QQUICKITEM_P_H
