@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -158,11 +164,6 @@ static void qt_quickitems_defineModule(const char *uri, int major, int minor)
     qmlRegisterType<QQuickPathView>(uri,major,minor,"PathView");
     qmlRegisterUncreatableType<QQuickBasePositioner>(uri,major,minor,"Positioner",
                                                   QStringLiteral("Positioner is an abstract type that is only available as an attached property."));
-#ifndef QT_NO_VALIDATOR
-    qmlRegisterType<QQuickIntValidator>(uri,major,minor,"IntValidator");
-    qmlRegisterType<QQuickDoubleValidator>(uri,major,minor,"DoubleValidator");
-    qmlRegisterType<QRegExpValidator>(uri,major,minor,"RegExpValidator");
-#endif
     qmlRegisterType<QQuickRectangle>(uri,major,minor,"Rectangle");
     qmlRegisterType<QQuickRepeater>(uri,major,minor,"Repeater");
     qmlRegisterType<QQuickRow>(uri,major,minor,"Row");
@@ -190,9 +191,6 @@ static void qt_quickitems_defineModule(const char *uri, int major, int minor)
     qmlRegisterType<QQuickCurve>();
     qmlRegisterType<QQuickScaleGrid>();
     qmlRegisterType<QQuickTextLine>();
-#ifndef QT_NO_VALIDATOR
-    qmlRegisterType<QValidator>();
-#endif
     qmlRegisterType<QQuickPen>();
     qmlRegisterType<QQuickFlickableVisibleArea>();
     qRegisterMetaType<QQuickAnchorLine>("QQuickAnchorLine");
@@ -269,6 +267,27 @@ static void qt_quickitems_defineModule(const char *uri, int major, int minor)
     qmlRegisterType<QQuickPinchArea, 1>(uri, 2, 5,"PinchArea");
     qmlRegisterType<QQuickImage, 2>(uri, 2, 5,"Image");
     qmlRegisterType<QQuickMouseArea, 2>(uri, 2, 5, "MouseArea");
+
+    qmlRegisterType<QQuickText, 6>(uri, 2, 6, "Text");
+    qmlRegisterType<QQuickTextEdit, 6>(uri, 2, 6, "TextEdit");
+    qmlRegisterType<QQuickTextInput, 6>(uri, 2, 6, "TextInput");
+    qmlRegisterUncreatableType<QQuickBasePositioner, 6>(uri, 2, 6, "Positioner",
+                                                  QStringLiteral("Positioner is an abstract type that is only available as an attached property."));
+    qmlRegisterType<QQuickColumn, 6>(uri, 2, 6, "Column");
+    qmlRegisterType<QQuickRow, 6>(uri, 2, 6, "Row");
+    qmlRegisterType<QQuickGrid, 6>(uri, 2, 6, "Grid");
+    qmlRegisterType<QQuickFlow, 6>(uri, 2, 6, "Flow");
+    qmlRegisterUncreatableType<QQuickEnterKeyAttached, 6>(uri, 2, 6, "EnterKey",
+                                                           QQuickEnterKeyAttached::tr("EnterKey is only available via attached properties"));
+    qmlRegisterType<QQuickShaderEffectSource, 1>(uri, 2, 6, "ShaderEffectSource");
+
+    qmlRegisterType<QQuickListView, 7>(uri, 2, 7, "ListView");
+    qmlRegisterType<QQuickGridView, 7>(uri, 2, 7, "GridView");
+    qmlRegisterType<QQuickTextInput, 7>(uri, 2, 7, "TextInput");
+    qmlRegisterType<QQuickTextEdit, 7>(uri, 2, 7, "TextEdit");
+    qmlRegisterType<QQuickPathView, 7>(uri, 2, 7, "PathView");
+
+    qmlRegisterUncreatableType<QQuickMouseEvent, 7>(uri, 2, 7, nullptr, QQuickMouseEvent::tr("MouseEvent is only available within handlers in MouseArea"));
 }
 
 static void initResources()
