@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -448,6 +454,29 @@ void QQuickItemView::setWrapEnabled(bool wrap)
     emit keyNavigationWrapsChanged();
 }
 
+bool QQuickItemView::isKeyNavigationEnabled() const
+{
+    Q_D(const QQuickItemView);
+    return d->explicitKeyNavigationEnabled ? d->keyNavigationEnabled : d->interactive;
+}
+
+void QQuickItemView::setKeyNavigationEnabled(bool keyNavigationEnabled)
+{
+    // TODO: default binding to "interactive" can be removed in Qt 6; it only exists for compatibility reasons.
+    Q_D(QQuickItemView);
+    const bool wasImplicit = !d->explicitKeyNavigationEnabled;
+    if (wasImplicit)
+        QObject::disconnect(this, &QQuickFlickable::interactiveChanged, this, &QQuickItemView::keyNavigationEnabledChanged);
+
+    d->explicitKeyNavigationEnabled = true;
+
+    // Ensure that we emit the change signal in case there is no different in value.
+    if (d->keyNavigationEnabled != keyNavigationEnabled || wasImplicit) {
+        d->keyNavigationEnabled = keyNavigationEnabled;
+        emit keyNavigationEnabledChanged();
+    }
+}
+
 int QQuickItemView::cacheBuffer() const
 {
     Q_D(const QQuickItemView);
@@ -1023,7 +1052,8 @@ QQuickItem *QQuickItemView::itemAt(qreal x, qreal y) const
 void QQuickItemView::forceLayout()
 {
     Q_D(QQuickItemView);
-    d->applyPendingChanges();
+    if (isComponentComplete() && (d->currentChanges.hasPendingChanges() || d->forceLayout))
+        d->layout();
 }
 
 void QQuickItemViewPrivate::applyPendingChanges()
@@ -1526,6 +1556,8 @@ QQuickItemViewPrivate::QQuickItemViewPrivate()
     , transitioner(0)
     , minExtent(0), maxExtent(0)
     , ownModel(false), wrap(false)
+    , keyNavigationEnabled(true)
+    , explicitKeyNavigationEnabled(false)
     , inLayout(false), inViewportMoved(false), forceLayout(false), currentIndexCleared(false)
     , haveHighlightRange(false), autoHighlight(true), highlightRangeStartValid(false), highlightRangeEndValid(false)
     , fillCacheBuffer(false), inRequest(false)
@@ -1654,6 +1686,7 @@ void QQuickItemViewPrivate::init()
     Q_Q(QQuickItemView);
     q->setFlag(QQuickItem::ItemIsFocusScope);
     QObject::connect(q, SIGNAL(movementEnded()), q, SLOT(animStopped()));
+    QObject::connect(q, &QQuickFlickable::interactiveChanged, q, &QQuickItemView::keyNavigationEnabledChanged);
     q->setFlickableDirection(QQuickFlickable::VerticalFlick);
 }
 
