@@ -58,7 +58,7 @@ DEFINE_BOOL_CONFIG_OPTION(qmlVisualTouchDebugging, QML_VISUAL_TOUCH_DEBUGGING)
 QQuickMouseAreaPrivate::QQuickMouseAreaPrivate()
 : enabled(true), scrollGestureEnabled(true), hovered(false), longPress(false),
   moved(false), stealMouse(false), doubleClick(false), preventStealing(false),
-  propagateComposedEvents(false), pressed(0)
+  propagateComposedEvents(false), overThreshold(false), pressed(0)
 #ifndef QT_NO_DRAGANDDROP
   , drag(0)
 #endif
@@ -672,6 +672,7 @@ void QQuickMouseArea::mousePressEvent(QMouseEvent *event)
     Q_D(QQuickMouseArea);
     d->moved = false;
     d->stealMouse = d->preventStealing;
+    d->overThreshold = false;
     if (!d->enabled || !(event->button() & acceptedMouseButtons())) {
         QQuickItem::mousePressEvent(event);
     } else {
@@ -721,7 +722,7 @@ void QQuickMouseArea::mouseMoveEvent(QMouseEvent *event)
             curLocalPos = event->windowPos();
         }
 
-        if (keepMouseGrab() && d->stealMouse && !d->drag->active())
+        if (keepMouseGrab() && d->stealMouse && d->overThreshold && !d->drag->active())
             d->drag->setActive(true);
 
         QPointF startPos = d->drag->target()->parentItem()
@@ -747,16 +748,19 @@ void QQuickMouseArea::mouseMoveEvent(QMouseEvent *event)
         if (d->drag->active())
             d->drag->target()->setPosition(dragPos);
 
-        if (!keepMouseGrab()
-                && (QQuickWindowPrivate::dragOverThreshold(dragPos.x() - startPos.x(), Qt::XAxis, event, d->drag->threshold())
-                || QQuickWindowPrivate::dragOverThreshold(dragPos.y() - startPos.y(), Qt::YAxis, event, d->drag->threshold()))) {
-            setKeepMouseGrab(true);
-            d->stealMouse = true;
-
+        if (!d->overThreshold && (QQuickWindowPrivate::dragOverThreshold(dragPos.x() - startPos.x(), Qt::XAxis, event, d->drag->threshold())
+                                  || QQuickWindowPrivate::dragOverThreshold(dragPos.y() - startPos.y(), Qt::YAxis, event, d->drag->threshold())))
+        {
+            d->overThreshold = true;
             if (d->drag->smoothed())
                 d->startScene = event->windowPos();
         }
 
+        if (!keepMouseGrab() && d->overThreshold) {
+            setKeepMouseGrab(true);
+            d->stealMouse = true;
+        }
+
         d->moved = true;
     }
 #endif
@@ -774,6 +778,7 @@ void QQuickMouseArea::mouseReleaseEvent(QMouseEvent *event)
 {
     Q_D(QQuickMouseArea);
     d->stealMouse = false;
+    d->overThreshold = false;
     if (!d->enabled && !d->pressed) {
         QQuickItem::mouseReleaseEvent(event);
     } else {
@@ -883,6 +888,7 @@ void QQuickMouseArea::ungrabMouse()
         d->pressed = 0;
         d->stealMouse = false;
         d->doubleClick = false;
+        d->overThreshold = false;
         setKeepMouseGrab(false);
 
 #ifndef QT_NO_DRAGANDDROP
@@ -947,6 +953,7 @@ bool QQuickMouseArea::sendMouseEvent(QMouseEvent *event)
             if (!d->pressed) {
                 // no other buttons are pressed
                 d->stealMouse = false;
+                d->overThreshold = false;
                 if (c && c->mouseGrabberItem() == this)
                     ungrabMouse();
                 emit canceled();
