@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
@@ -10,30 +10,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -42,7 +40,7 @@
 #include "qquickpainteditem.h"
 #include <private/qquickpainteditem_p.h>
 
-#include <QtQuick/private/qsgpainternode_p.h>
+#include <QtQuick/private/qsgdefaultpainternode_p.h>
 #include <QtQuick/private/qsgcontext_p.h>
 #include <private/qsgadaptationlayer_p.h>
 
@@ -50,6 +48,14 @@
 
 QT_BEGIN_NAMESPACE
 
+class QQuickPaintedItemTextureProvider : public QSGTextureProvider
+{
+public:
+    QSGPainterNode *node;
+    QSGTexture *texture() const { return node ? node->texture() : 0; }
+    void fireTextureChanged() { emit textureChanged(); }
+};
+
 /*!
     \class QQuickPaintedItem
     \brief The QQuickPaintedItem class provides a way to use the QPainter API in the
@@ -67,10 +73,11 @@ QT_BEGIN_NAMESPACE
 
     To enable QPainter to do anti-aliased rendering, use setAntialiasing().
 
-    To write your own painted item, you first create a subclass of QQuickPaintedItem, and then
-    start by implementing its only pure virtual public function: paint(), which implements
-    the actual painting. To get the size of the area painted by the item, use
-    contentsBoundingRect().
+    To write your own painted item, you first create a subclass of
+    QQuickPaintedItem, and then start by implementing its only pure virtual
+    public function: paint(), which implements the actual painting. The
+    painting will be inside the rectangle spanning from 0,0 to
+    width(),height().
 */
 
 /*!
@@ -104,11 +111,12 @@ QT_BEGIN_NAMESPACE
     This enum describes flags that you can enable to improve rendering
     performance in QQuickPaintedItem. By default, none of these flags are set.
 
-    \value FastFBOResizing If your item gets resized often and you are using the
-    QQuickPaintedItem::FramebufferObject render target, set this flag to true to reduce the
-    item resizing time at the cost of using more graphics memory. Resizing a Framebuffer object
-    is a costly operation, by enabling this property the Framebuffer Object will use a texture
-    larger than the actual size of the item to avoid as much as possible resizing it.
+    \value FastFBOResizing Resizing an FBO can be a costly operation on a few
+    OpenGL driver implementations. To work around this, one can set this flag
+    to let the QQuickPaintedItem allocate one large framebuffer object and
+    instead draw into a subregion of it. This saves the resize at the cost of
+    using more memory. Please note that this is not a common problem.
+
 */
 
 /*!
@@ -123,6 +131,8 @@ QQuickPaintedItemPrivate::QQuickPaintedItemPrivate()
     , opaquePainting(false)
     , antialiasing(false)
     , mipmap(false)
+    , textureProvider(0)
+    , node(0)
 {
 }
 
@@ -149,6 +159,9 @@ QQuickPaintedItem::QQuickPaintedItem(QQuickPaintedItemPrivate &dd, QQuickItem *p
 */
 QQuickPaintedItem::~QQuickPaintedItem()
 {
+    Q_D(QQuickPaintedItem);
+    if (d->textureProvider)
+        QQuickWindowQObjectCleanupJob::schedule(window(), d->textureProvider);
 }
 
 /*!
@@ -322,16 +335,47 @@ void QQuickPaintedItem::setPerformanceHints(QQuickPaintedItem::PerformanceHints
     update();
 }
 
+QSize QQuickPaintedItem::textureSize() const
+{
+    Q_D(const QQuickPaintedItem);
+    return d->textureSize;
+}
+
 /*!
-    This function returns the outer bounds of the item as a rectangle; all painting must be
-    restricted to inside an item's bounding rect.
+    \property QQuickPaintedItem::textureSize
 
-    If the contents size has not been set it reflects the size of the item; otherwise
-    it reflects the contents size scaled by the contents scale.
+    \brief Defines the size of the texture.
 
-    Use this function to know the area painted by the item.
+    Changing the texture's size does not affect the coordinate system used in
+    paint(). A scale factor is instead applied so painting should still happen
+    inside 0,0 to width(),height().
 
-    \sa QQuickItem::width(), QQuickItem::height(), contentsSize(), contentsScale()
+    By default, the texture size will have the same size as this item.
+
+    \note If the item is on a window with a device pixel ratio different from
+    1, this scale factor will be implicitly applied to the texture size.
+
+ */
+void QQuickPaintedItem::setTextureSize(const QSize &size)
+{
+    Q_D(QQuickPaintedItem);
+    if (d->textureSize == size)
+        return;
+    d->textureSize = size;
+    emit textureSizeChanged();
+}
+
+#if QT_VERSION >= 0x060000
+#warning "Remove: QQuickPaintedItem::contentsBoundingRect, contentsScale, contentsSize. Also remove them from qsgadaptationlayer_p.h and qsgdefaultpainternode.h/cpp."
+#endif
+
+/*!
+    \obsolete
+
+    This function is provided for compatibility, use size in combination
+    with textureSize to decide the size of what you are drawing.
+
+    \sa width(), height(), textureSize()
 */
 QRectF QQuickPaintedItem::contentsBoundingRect() const
 {
@@ -350,11 +394,13 @@ QRectF QQuickPaintedItem::contentsBoundingRect() const
 
 /*!
     \property QQuickPaintedItem::contentsSize
-    \brief The size of the contents
+    \brief Obsolete method for setting the contents size.
+    \obsolete
 
-    The contents size is the size of the item in regards to how it is painted
-    using the paint() function.  This is distinct from the size of the
-    item in regards to height() and width().
+    This function is provided for compatibility, use size in combination
+    with textureSize to decide the size of what you are drawing.
+
+    \sa width(), height(), textureSize()
 */
 QSize QQuickPaintedItem::contentsSize() const
 {
@@ -376,6 +422,7 @@ void QQuickPaintedItem::setContentsSize(const QSize &size)
 }
 
 /*!
+    \obsolete
     This convenience function is equivalent to calling setContentsSize(QSize()).
 */
 void QQuickPaintedItem::resetContentsSize()
@@ -385,12 +432,13 @@ void QQuickPaintedItem::resetContentsSize()
 
 /*!
     \property QQuickPaintedItem::contentsScale
-    \brief The scale of the contents
+    \brief Obsolete method for scaling the contents.
+    \obsolete
 
-    All painting happening in paint() is scaled by the contents scale. This is distinct
-    from the scale of the item in regards to scale().
+    This function is provided for compatibility, use size() in combination
+    with textureSize() to decide the size of what you are drawing.
 
-    The default value is 1.
+    \sa width(), height(), textureSize()
 */
 qreal QQuickPaintedItem::contentsScale() const
 {
@@ -441,7 +489,7 @@ void QQuickPaintedItem::setFillColor(const QColor &c)
     \brief The item's render target.
 
     This property defines which render target the QPainter renders into, it can be either
-    QSGPaintedItem::Image, QSGPaintedItem::FramebufferObject or QSGPaintedItem::InvertedYFramebufferObject.
+    QQuickPaintedItem::Image, QQuickPaintedItem::FramebufferObject or QQuickPaintedItem::InvertedYFramebufferObject.
 
     Each has certain benefits, typically performance versus quality. Using a framebuffer
     object avoids a costly upload of the image contents to the texture in graphics memory,
@@ -477,6 +525,9 @@ void QQuickPaintedItem::setRenderTarget(RenderTarget target)
     This function, which is usually called by the QML Scene Graph, paints the
     contents of an item in local coordinates.
 
+    The underlying texture will have a size defined by textureSize when set,
+    or the item's size, multiplied by the window's device pixel ratio.
+
     The function is called after the item has been filled with the fillColor.
 
     Reimplement this function in a QQuickPaintedItem subclass to provide the
@@ -490,6 +541,8 @@ void QQuickPaintedItem::setRenderTarget(RenderTarget target)
 
     \warning Extreme caution must be used when creating QObjects, emitting signals, starting
     timers and similar inside this function as these will have affinity to the rendering thread.
+
+    \sa width(), height(), textureSize
 */
 
 /*!
@@ -502,30 +555,119 @@ QSGNode *QQuickPaintedItem::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeDat
 
     if (width() <= 0 || height() <= 0) {
         delete oldNode;
+        if (d->textureProvider) {
+            d->textureProvider->node = 0;
+            d->textureProvider->fireTextureChanged();
+        }
         return 0;
     }
 
     QSGPainterNode *node = static_cast<QSGPainterNode *>(oldNode);
-    if (!node)
-        node = new QSGPainterNode(this);
+    if (!node) {
+        node = d->sceneGraphContext()->createPainterNode(this);
+        d->node = node;
+    }
 
-    QRectF br = contentsBoundingRect();
+    bool hasTextureSize = d->textureSize.width() > 0 && d->textureSize.height() > 0;
+
+    // Use the compat mode if any of the compat things are set and
+    // textureSize is 0x0.
+    if (!hasTextureSize
+        && (d->contentsScale != 1
+            || (d->contentsSize.width() > 0 && d->contentsSize.height() > 0))) {
+        QRectF br = contentsBoundingRect();
+        node->setContentsScale(d->contentsScale);
+        QSize size = QSize(qRound(br.width()), qRound(br.height()));
+        node->setSize(size);
+        node->setTextureSize(size);
+    } else {
+        // The default, use textureSize or item's size * device pixel ratio
+        node->setContentsScale(1);
+        QSize itemSize(qRound(width()), qRound(height()));
+        node->setSize(itemSize);
+        QSize textureSize = (hasTextureSize ? d->textureSize : itemSize)
+                            * window()->effectiveDevicePixelRatio();
+        node->setTextureSize(textureSize);
+    }
 
     node->setPreferredRenderTarget(d->renderTarget);
     node->setFastFBOResizing(d->performanceHints & FastFBOResizing);
-    node->setSize(QSize(qRound(br.width()), qRound(br.height())));
     node->setSmoothPainting(d->antialiasing);
     node->setLinearFiltering(d->smooth);
     node->setMipmapping(d->mipmap);
     node->setOpaquePainting(d->opaquePainting);
     node->setFillColor(d->fillColor);
-    node->setContentsScale(d->contentsScale);
     node->setDirty(d->dirtyRect);
     node->update();
 
     d->dirtyRect = QRect();
 
+    if (d->textureProvider) {
+        d->textureProvider->node = node;
+        d->textureProvider->fireTextureChanged();
+    }
+
     return node;
 }
 
+/*!
+   \reimp
+*/
+void QQuickPaintedItem::releaseResources()
+{
+    Q_D(QQuickPaintedItem);
+    if (d->textureProvider) {
+        QQuickWindowQObjectCleanupJob::schedule(window(), d->textureProvider);
+        d->textureProvider = 0;
+    }
+    d->node = 0; // Managed by the scene graph, just clear the pointer.
+}
+
+void QQuickPaintedItem::invalidateSceneGraph()
+{
+    Q_D(QQuickPaintedItem);
+    delete d->textureProvider;
+    d->textureProvider = 0;
+    d->node = 0; // Managed by the scene graph, just clear the pointer
+}
+
+/*!
+   \reimp
+*/
+bool QQuickPaintedItem::isTextureProvider() const
+{
+    return true;
+}
+
+/*!
+   \reimp
+*/
+QSGTextureProvider *QQuickPaintedItem::textureProvider() const
+{
+    // When Item::layer::enabled == true, QQuickItem will be a texture
+    // provider. In this case we should prefer to return the layer rather
+    // than the image itself. The layer will include any children and any
+    // the image's wrap and fill mode.
+    if (QQuickItem::isTextureProvider())
+        return QQuickItem::textureProvider();
+
+    Q_D(const QQuickPaintedItem);
+    QQuickWindow *w = window();
+    if (!w || !w->openglContext() || QThread::currentThread() != w->openglContext()->thread()) {
+        qWarning("QQuickPaintedItem::textureProvider: can only be queried on the rendering thread of an exposed window");
+        return 0;
+    }
+    if (!d->textureProvider)
+        d->textureProvider = new QQuickPaintedItemTextureProvider();
+    d->textureProvider->node = d->node;
+    return d->textureProvider;
+}
+
+void QQuickPaintedItem::itemChange(ItemChange change, const ItemChangeData &value)
+{
+    if (change == ItemDevicePixelRatioHasChanged)
+        update();
+    QQuickItem::itemChange(change, value);
+}
+
 QT_END_NAMESPACE
