@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -99,7 +105,8 @@ QQuickPathViewPrivate::QQuickPathViewPrivate()
     , dragMargin(0), deceleration(100), maximumFlickVelocity(QML_FLICK_DEFAULTMAXVELOCITY)
     , moveOffset(this, &QQuickPathViewPrivate::setAdjustedOffset), flickDuration(0)
     , pathItems(-1), requestedIndex(-1), cacheSize(0), requestedZ(0)
-    , moveReason(Other), moveDirection(Shortest), attType(0), highlightComponent(0), highlightItem(0)
+    , moveReason(Other), movementDirection(QQuickPathView::Shortest), moveDirection(QQuickPathView::Shortest)
+    , attType(0), highlightComponent(0), highlightItem(0)
     , moveHighlight(this, &QQuickPathViewPrivate::setHighlightPosition)
     , highlightPosition(0)
     , highlightRangeStart(0), highlightRangeEnd(0)
@@ -786,7 +793,7 @@ QQuickItem *QQuickPathView::currentItem() const
 void QQuickPathView::incrementCurrentIndex()
 {
     Q_D(QQuickPathView);
-    d->moveDirection = QQuickPathViewPrivate::Positive;
+    d->moveDirection = QQuickPathView::Positive;
     setCurrentIndex(currentIndex()+1);
 }
 
@@ -800,7 +807,7 @@ void QQuickPathView::incrementCurrentIndex()
 void QQuickPathView::decrementCurrentIndex()
 {
     Q_D(QQuickPathView);
-    d->moveDirection = QQuickPathViewPrivate::Negative;
+    d->moveDirection = QQuickPathView::Negative;
     setCurrentIndex(currentIndex()-1);
 }
 
@@ -1370,6 +1377,48 @@ void QQuickPathView::setSnapMode(SnapMode mode)
     emit snapModeChanged();
 }
 
+/*!
+    \qmlproperty enumeration QtQuick::PathView::movementDirection
+    \since 5.7
+
+    This property determines the direction in which items move when setting the current index.
+    The possible values are:
+
+    \list
+    \li PathView.Shortest (default) - the items move in the direction that requires the least
+        movement, which could be either \c Negative or \c Positive.
+    \li PathView.Negative - the items move backwards towards their destination.
+    \li PathView.Positive - the items move forwards towards their destination.
+    \endlist
+
+    For example, suppose that there are 5 items in the model, and \l currentIndex is \c 0.
+    If currentIndex is set to \c 2,
+
+    \list
+    \li a \c Positive movement direction will result in the following order: 0, 1, 2
+    \li a \c Negative movement direction will result in the following order: 0, 5, 4, 3, 2
+    \li a \c Shortest movement direction will result in same order with \c Positive .
+    \endlist
+
+    \note this property doesn't affect the movement of \l incrementCurrentIndex() and \l decrementCurrentIndex().
+*/
+QQuickPathView::MovementDirection QQuickPathView::movementDirection() const
+{
+    Q_D(const QQuickPathView);
+    return d->movementDirection;
+}
+
+void QQuickPathView::setMovementDirection(QQuickPathView::MovementDirection dir)
+{
+    Q_D(QQuickPathView);
+    if (dir == d->movementDirection)
+        return;
+    d->movementDirection = dir;
+    if (!d->tl.isActive())
+        d->moveDirection = d->movementDirection;
+    emit movementDirectionChanged();
+}
+
 /*!
     \qmlmethod QtQuick::PathView::positionViewAtIndex(int index, PositionMode mode)
 
@@ -1916,7 +1965,7 @@ void QQuickPathView::refill()
         if (lcItemViewDelegateLifecycle().isDebugEnabled()) {
             QQuickText *text = qmlobject_cast<QQuickText*>(item);
             if (text)
-                qCDebug(lcItemViewDelegateLifecycle) << "idx" << idx << "@" << pos << ": QQuickText" << text->objectName() << text->text().left(40);
+                qCDebug(lcItemViewDelegateLifecycle) << "idx" << idx << "@" << pos << ": QQuickText" << text->objectName() << text->text().leftRef(40);
             else
                 qCDebug(lcItemViewDelegateLifecycle) << "idx" << idx << "@" << pos << ":" << item;
         }
@@ -2245,6 +2294,7 @@ void QQuickPathView::movementEnding()
         emit movingChanged();
         emit movementEnded();
     }
+    d->moveDirection = d->movementDirection;
 }
 
 // find the item closest to the snap position
@@ -2354,7 +2404,7 @@ void QQuickPathViewPrivate::snapToIndex(int index, MovementReason reason)
 
     if (!duration) {
         tl.set(moveOffset, targetOffset);
-    } else if (moveDirection == Positive || (moveDirection == Shortest && targetOffset - offset > modelCount/2.0)) {
+    } else if (moveDirection == QQuickPathView::Positive || (moveDirection == QQuickPathView::Shortest && targetOffset - offset > modelCount/2.0)) {
         qreal distance = modelCount - targetOffset + offset;
         if (targetOffset > moveOffset) {
             tl.move(moveOffset, 0.0, QEasingCurve(QEasingCurve::InQuad), int(duration * offset / distance));
@@ -2363,7 +2413,7 @@ void QQuickPathViewPrivate::snapToIndex(int index, MovementReason reason)
         } else {
             tl.move(moveOffset, targetOffset, QEasingCurve(QEasingCurve::InOutQuad), duration);
         }
-    } else if (moveDirection == Negative || targetOffset - offset <= -modelCount/2.0) {
+    } else if (moveDirection == QQuickPathView::Negative || targetOffset - offset <= -modelCount/2.0) {
         qreal distance = modelCount - offset + targetOffset;
         if (targetOffset < moveOffset) {
             tl.move(moveOffset, modelCount, QEasingCurve(targetOffset == 0 ? QEasingCurve::InOutQuad : QEasingCurve::InQuad), int(duration * (modelCount-offset) / distance));
@@ -2375,7 +2425,6 @@ void QQuickPathViewPrivate::snapToIndex(int index, MovementReason reason)
     } else {
         tl.move(moveOffset, targetOffset, QEasingCurve(QEasingCurve::InOutQuad), duration);
     }
-    moveDirection = Shortest;
 }
 
 QQuickPathViewAttached *QQuickPathView::qmlAttachedProperties(QObject *obj)
