@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
@@ -10,30 +10,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -52,8 +50,13 @@
 QT_BEGIN_NAMESPACE
 
 QQuickRepeaterPrivate::QQuickRepeaterPrivate()
-    : model(0), ownModel(false), inRequest(false), dataSourceIsObject(false), delegateValidated(false), itemCount(0), createFrom(-1)
+    : model(0)
+    , ownModel(false)
+    , dataSourceIsObject(false)
+    , delegateValidated(false)
+    , itemCount(0)
 {
+    setTransparentForPositioner(true);
 }
 
 QQuickRepeaterPrivate::~QQuickRepeaterPrivate()
@@ -191,19 +194,24 @@ QVariant QQuickRepeater::model() const
     return d->dataSource;
 }
 
-void QQuickRepeater::setModel(const QVariant &model)
+void QQuickRepeater::setModel(const QVariant &m)
 {
     Q_D(QQuickRepeater);
+    QVariant model = m;
+    if (model.userType() == qMetaTypeId<QJSValue>())
+        model = model.value<QJSValue>().toVariant();
+
     if (d->dataSource == model)
         return;
 
     clear();
     if (d->model) {
-        disconnect(d->model, SIGNAL(modelUpdated(QQmlChangeSet,bool)),
-                this, SLOT(modelUpdated(QQmlChangeSet,bool)));
-        disconnect(d->model, SIGNAL(createdItem(int,QObject*)), this, SLOT(createdItem(int,QObject*)));
-        disconnect(d->model, SIGNAL(initItem(int,QObject*)), this, SLOT(initItem(int,QObject*)));
-//        disconnect(d->model, SIGNAL(destroyingItem(QObject*)), this, SLOT(destroyingItem(QObject*)));
+        qmlobject_disconnect(d->model, QQmlInstanceModel, SIGNAL(modelUpdated(QQmlChangeSet,bool)),
+                this, QQuickRepeater, SLOT(modelUpdated(QQmlChangeSet,bool)));
+        qmlobject_disconnect(d->model, QQmlInstanceModel, SIGNAL(createdItem(int,QObject*)),
+                this, QQuickRepeater, SLOT(createdItem(int,QObject*)));
+        qmlobject_disconnect(d->model, QQmlInstanceModel, SIGNAL(initItem(int,QObject*)),
+                this, QQuickRepeater, SLOT(initItem(int,QObject*)));
     }
     d->dataSource = model;
     QObject *object = qvariant_cast<QObject*>(model);
@@ -227,11 +235,12 @@ void QQuickRepeater::setModel(const QVariant &model)
             dataModel->setModel(model);
     }
     if (d->model) {
-        connect(d->model, SIGNAL(modelUpdated(QQmlChangeSet,bool)),
-                this, SLOT(modelUpdated(QQmlChangeSet,bool)));
-        connect(d->model, SIGNAL(createdItem(int,QObject*)), this, SLOT(createdItem(int,QObject*)));
-        connect(d->model, SIGNAL(initItem(int,QObject*)), this, SLOT(initItem(int,QObject*)));
-//        connect(d->model, SIGNAL(destroyingItem(QObject*)), this, SLOT(destroyingItem(QObject*)));
+        qmlobject_connect(d->model, QQmlInstanceModel, SIGNAL(modelUpdated(QQmlChangeSet,bool)),
+                this, QQuickRepeater, SLOT(modelUpdated(QQmlChangeSet,bool)));
+        qmlobject_connect(d->model, QQmlInstanceModel, SIGNAL(createdItem(int,QObject*)),
+                this, QQuickRepeater, SLOT(createdItem(int,QObject*)));
+        qmlobject_connect(d->model, QQmlInstanceModel, SIGNAL(initItem(int,QObject*)),
+                this, QQuickRepeater, SLOT(initItem(int,QObject*)));
         regenerate();
     }
     emit modelChanged();
@@ -354,12 +363,14 @@ void QQuickRepeater::clear()
     bool complete = isComponentComplete();
 
     if (d->model) {
-        for (int i = 0; i < d->deletables.count(); ++i) {
+        // We remove in reverse order deliberately; so that signals are emitted
+        // with sensible indices.
+        for (int i = d->deletables.count() - 1; i >= 0; --i) {
             if (QQuickItem *item = d->deletables.at(i)) {
                 if (complete)
                     emit itemRemoved(i, item);
-                item->setParentItem(0);
                 d->model->release(item);
+                item->setParentItem(0);
             }
         }
     }
@@ -380,64 +391,58 @@ void QQuickRepeater::regenerate()
 
     d->itemCount = count();
     d->deletables.resize(d->itemCount);
-    d->createFrom = 0;
-    d->createItems();
+    d->requestItems();
 }
 
-void QQuickRepeaterPrivate::createItems()
+void QQuickRepeaterPrivate::requestItems()
 {
-    Q_Q(QQuickRepeater);
-    if (createFrom == -1)
-        return;
-    inRequest = true;
-    for (int ii = createFrom; ii < itemCount; ++ii) {
-        if (!deletables.at(ii)) {
-            QObject *object = model->object(ii, false);
-            QQuickItem *item = qmlobject_cast<QQuickItem*>(object);
-            if (!item) {
-                if (object) {
-                    model->release(object);
-                    if (!delegateValidated) {
-                        delegateValidated = true;
-                        QObject* delegate = q->delegate();
-                        qmlInfo(delegate ? delegate : q) << QQuickRepeater::tr("Delegate must be of Item type");
-                    }
-                }
-                createFrom = ii;
-                break;
-            }
-            deletables[ii] = item;
-            item->setParentItem(q->parentItem());
-            if (ii > 0 && deletables.at(ii-1)) {
-                item->stackAfter(deletables.at(ii-1));
-            } else {
-                QQuickItem *after = q;
-                for (int si = ii+1; si < itemCount; ++si) {
-                    if (deletables.at(si)) {
-                        after = deletables.at(si);
-                        break;
-                    }
-                }
-                item->stackBefore(after);
-            }
-            emit q->itemAdded(ii, item);
-        }
+    for (int i = 0; i < itemCount; i++) {
+        QObject *object = model->object(i, false);
+        if (object)
+            model->release(object);
     }
-    inRequest = false;
 }
 
-void QQuickRepeater::createdItem(int, QObject *)
+void QQuickRepeater::createdItem(int index, QObject *)
 {
     Q_D(QQuickRepeater);
-    if (!d->inRequest)
-        d->createItems();
+    QObject *object = d->model->object(index, false);
+    QQuickItem *item = qmlobject_cast<QQuickItem*>(object);
+    emit itemAdded(index, item);
 }
 
-void QQuickRepeater::initItem(int, QObject *object)
+void QQuickRepeater::initItem(int index, QObject *object)
 {
+    Q_D(QQuickRepeater);
     QQuickItem *item = qmlobject_cast<QQuickItem*>(object);
-    if (item)
+
+    if (!d->deletables.at(index)) {
+        if (!item) {
+            if (object) {
+                d->model->release(object);
+                if (!d->delegateValidated) {
+                    d->delegateValidated = true;
+                    QObject* delegate = this->delegate();
+                    qmlInfo(delegate ? delegate : this) << QQuickRepeater::tr("Delegate must be of Item type");
+                }
+            }
+            return;
+        }
+        d->deletables[index] = item;
         item->setParentItem(parentItem());
+        if (index > 0 && d->deletables.at(index-1)) {
+            item->stackAfter(d->deletables.at(index-1));
+        } else {
+            QQuickItem *after = this;
+            for (int si = index+1; si < d->itemCount; ++si) {
+                if (d->deletables.at(si)) {
+                    after = d->deletables.at(si);
+                    break;
+                }
+            }
+            item->stackBefore(after);
+        }
+    }
 }
 
 void QQuickRepeater::modelUpdated(const QQmlChangeSet &changeSet, bool reset)
@@ -456,7 +461,7 @@ void QQuickRepeater::modelUpdated(const QQmlChangeSet &changeSet, bool reset)
 
     int difference = 0;
     QHash<int, QVector<QPointer<QQuickItem> > > moved;
-    foreach (const QQmlChangeSet::Remove &remove, changeSet.removes()) {
+    foreach (const QQmlChangeSet::Change &remove, changeSet.removes()) {
         int index = qMin(remove.index, d->deletables.count());
         int count = qMin(remove.index + remove.count, d->deletables.count()) - index;
         if (remove.isMove()) {
@@ -469,8 +474,8 @@ void QQuickRepeater::modelUpdated(const QQmlChangeSet &changeSet, bool reset)
             d->deletables.remove(index);
             emit itemRemoved(index, item);
             if (item) {
-                item->setParentItem(0);
                 d->model->release(item);
+                item->setParentItem(0);
             }
             --d->itemCount;
         }
@@ -478,8 +483,7 @@ void QQuickRepeater::modelUpdated(const QQmlChangeSet &changeSet, bool reset)
         difference -= remove.count;
     }
 
-    d->createFrom = -1;
-    foreach (const QQmlChangeSet::Insert &insert, changeSet.inserts()) {
+    foreach (const QQmlChangeSet::Change &insert, changeSet.inserts()) {
         int index = qMin(insert.index, d->deletables.count());
         if (insert.isMove()) {
             QVector<QPointer<QQuickItem> > items = moved.value(insert.moveId);
@@ -493,14 +497,13 @@ void QQuickRepeater::modelUpdated(const QQmlChangeSet &changeSet, bool reset)
             int modelIndex = index + i;
             ++d->itemCount;
             d->deletables.insert(modelIndex, 0);
-            if (d->createFrom == -1)
-                d->createFrom = modelIndex;
+            QObject *object = d->model->object(modelIndex, false);
+            if (object)
+                d->model->release(object);
         }
         difference += insert.count;
     }
 
-    d->createItems();
-
     if (difference != 0)
         emit countChanged();
 }
