@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -635,11 +641,11 @@ int QQuickStochasticEngine::nextState(int curState, int curThing)
         qreal r =(qreal) qrand() / (qreal) RAND_MAX;
         qreal total = 0.0;
         for (QVariantMap::const_iterator iter=m_states[curState]->m_to.constBegin();
-            iter!=m_states[curState]->m_to.constEnd(); iter++)
+            iter!=m_states[curState]->m_to.constEnd(); ++iter)
             total += (*iter).toReal();
         r*=total;
         for (QVariantMap::const_iterator iter= m_states[curState]->m_to.constBegin();
-                iter!=m_states[curState]->m_to.constEnd(); iter++){
+                iter!=m_states[curState]->m_to.constEnd(); ++iter){
             if (r < (*iter).toReal()){
                 bool superBreak = false;
                 for (int i=0; i<m_states.count(); i++){
@@ -698,7 +704,7 @@ int QQuickStochasticEngine::goalSeek(int curIdx, int spriteIdx, int dist)
         dist = m_states.count();
     QQuickStochasticState* curState = m_states[curIdx];
     for (QVariantMap::const_iterator iter = curState->m_to.constBegin();
-        iter!=curState->m_to.constEnd(); iter++){
+        iter!=curState->m_to.constEnd(); ++iter){
         if (iter.key() == goalName)
             for (int i=0; i<m_states.count(); i++)
                 if (m_states[i]->name() == goalName)
@@ -707,7 +713,7 @@ int QQuickStochasticEngine::goalSeek(int curIdx, int spriteIdx, int dist)
     QSet<int> options;
     for (int i=1; i<dist; i++){
         for (QVariantMap::const_iterator iter = curState->m_to.constBegin();
-            iter!=curState->m_to.constEnd(); iter++){
+            iter!=curState->m_to.constEnd(); ++iter){
             int option = -1;
             for (int j=0; j<m_states.count(); j++)//One place that could be a lot more efficient...
                 if (m_states[j]->name() == iter.key())
@@ -723,11 +729,11 @@ int QQuickStochasticEngine::goalSeek(int curIdx, int spriteIdx, int dist)
             qreal r =(qreal) qrand() / (qreal) RAND_MAX;
             qreal total = 0;
             for (QSet<int>::const_iterator iter=options.constBegin();
-                iter!=options.constEnd(); iter++)
+                iter!=options.constEnd(); ++iter)
                 total += curState->m_to.value(m_states[(*iter)]->name()).toReal();
             r *= total;
             for (QVariantMap::const_iterator iter = curState->m_to.constBegin();
-                iter!=curState->m_to.constEnd(); iter++){
+                iter!=curState->m_to.constEnd(); ++iter){
                 bool superContinue = true;
                 for (int j=0; j<m_states.count(); j++)
                     if (m_states[j]->name() == iter.key())
