@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -102,6 +108,7 @@ QQuickTextControlPrivate::QQuickTextControlPrivate()
       overwriteMode(false),
       acceptRichText(true),
       cursorVisible(false),
+      cursorBlinkingEnabled(false),
       hasFocus(false),
       hadSelectionOnMousePress(false),
       wordSelectionEnabled(false),
@@ -456,15 +463,31 @@ void QQuickTextControlPrivate::_q_updateCursorPosChanged(const QTextCursor &some
 
 void QQuickTextControlPrivate::setBlinkingCursorEnabled(bool enable)
 {
-    Q_Q(QQuickTextControl);
+    if (cursorBlinkingEnabled == enable)
+        return;
 
-    if (enable && QGuiApplication::styleHints()->cursorFlashTime() > 0)
-        cursorBlinkTimer.start(QGuiApplication::styleHints()->cursorFlashTime() / 2, q);
+    cursorBlinkingEnabled = enable;
+    updateCursorFlashTime();
+
+    if (enable)
+        connect(qApp->styleHints(), &QStyleHints::cursorFlashTimeChanged, this, &QQuickTextControlPrivate::updateCursorFlashTime);
+    else
+        disconnect(qApp->styleHints(), &QStyleHints::cursorFlashTimeChanged, this, &QQuickTextControlPrivate::updateCursorFlashTime);
+}
+
+void QQuickTextControlPrivate::updateCursorFlashTime()
+{
+    // Note: cursorOn represents the current blinking state controlled by a timer, and
+    // should not be confused with cursorVisible or cursorBlinkingEnabled. However, we
+    // interpretate a cursorFlashTime of 0 to mean "always on, never blink".
+    cursorOn = true;
+    int flashTime = QGuiApplication::styleHints()->cursorFlashTime();
+
+    if (cursorBlinkingEnabled && flashTime >= 2)
+        cursorBlinkTimer.start(flashTime / 2, q_func());
     else
         cursorBlinkTimer.stop();
 
-    cursorOn = enable;
-
     repaintCursor();
 }
 
@@ -577,8 +600,15 @@ void QQuickTextControl::redo()
     updateCursorRectangle(true);
 }
 
+void QQuickTextControl::clear()
+{
+    Q_D(QQuickTextControl);
+    d->cursor.select(QTextCursor::Document);
+    d->cursor.removeSelectedText();
+}
+
 QQuickTextControl::QQuickTextControl(QTextDocument *doc, QObject *parent)
-    : QInputControl(TextEdit, *new QQuickTextControlPrivate, parent)
+    : QObject(*new QQuickTextControlPrivate, parent)
 {
     Q_D(QQuickTextControl);
     Q_ASSERT(doc);
@@ -950,8 +980,9 @@ void QQuickTextControlPrivate::keyPressEvent(QKeyEvent *e)
 
 process:
     {
-        if (q->isAcceptableInput(e)) {
-            cursor.insertText(e->text());
+        QString text = e->text();
+        if (!text.isEmpty() && (text.at(0).isPrint() || text.at(0) == QLatin1Char('\t'))) {
+            cursor.insertText(text);
             selectionChanged();
         } else {
             e->ignore();
@@ -1320,8 +1351,10 @@ void QQuickTextControlPrivate::inputMethodEvent(QInputMethodEvent *e)
 
     QTextBlock block = cursor.block();
     QTextLayout *layout = block.layout();
-    if (isGettingInput)
+    if (isGettingInput) {
         layout->setPreeditArea(cursor.position() - block.position(), e->preeditString());
+        emit q->preeditTextChanged();
+    }
     QVector<QTextLayout::FormatRange> overrides;
     const int oldPreeditCursor = preeditCursor;
     preeditCursor = e->preeditString().length();
@@ -1368,10 +1401,16 @@ QVariant QQuickTextControl::inputMethodQuery(Qt::InputMethodQuery property, QVar
     switch (property) {
     case Qt::ImCursorRectangle:
         return cursorRect();
+    case Qt::ImAnchorRectangle:
+        return anchorRect();
     case Qt::ImFont:
         return QVariant(d->cursor.charFormat().font());
-    case Qt::ImCursorPosition:
+    case Qt::ImCursorPosition: {
+        const QPointF pt = argument.toPointF();
+        if (!pt.isNull())
+            return QVariant(d->doc->documentLayout()->hitTest(pt, Qt::FuzzyHit) - block.position());
         return QVariant(d->cursor.position() - block.position());
+    }
     case Qt::ImSurroundingText:
         return QVariant(block.text());
     case Qt::ImCurrentSelection:
@@ -1418,7 +1457,7 @@ QVariant QQuickTextControl::inputMethodQuery(Qt::InputMethodQuery property, QVar
             tmpCursor.movePosition(QTextCursor::NextBlock);
             --numBlocks;
         }
-        result += block.text().mid(0,localPos);
+        result += block.text().midRef(0,localPos);
         return QVariant(result);
     }
     default:
@@ -1481,6 +1520,17 @@ void QQuickTextControl::setCursorVisible(bool visible)
             && (d->interactionFlags & (Qt::TextEditable | Qt::TextSelectableByKeyboard)));
 }
 
+QRectF QQuickTextControl::anchorRect() const
+{
+    Q_D(const QQuickTextControl);
+    QRectF rect;
+    QTextCursor cursor = d->cursor;
+    if (!cursor.isNull()) {
+        rect = d->rectForPosition(cursor.anchor());
+    }
+    return rect;
+}
+
 QRectF QQuickTextControl::cursorRect(const QTextCursor &cursor) const
 {
     Q_D(const QQuickTextControl);
@@ -1755,6 +1805,19 @@ QRectF QQuickTextControl::blockBoundingRect(const QTextBlock &block) const
     return d->doc->documentLayout()->blockBoundingRect(block);
 }
 
+QString QQuickTextControl::preeditText() const
+{
+#ifndef QT_NO_IM
+    Q_D(const QQuickTextControl);
+    QTextLayout *layout = d->cursor.block().layout();
+    if (!layout)
+        return QString();
+
+    return layout->preeditAreaText();
+#else
+    return QString();
+#endif
+}
 
 
 QStringList QQuickTextEditMimeData::formats() const
