@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
@@ -10,30 +10,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -186,14 +184,22 @@ void QQuickAbstractAnimationPrivate::commence()
     }
 }
 
-QQmlProperty QQuickAbstractAnimationPrivate::createProperty(QObject *obj, const QString &str, QObject *infoObj)
+QQmlProperty QQuickAbstractAnimationPrivate::createProperty(QObject *obj, const QString &str, QObject *infoObj, QString *errorMessage)
 {
     QQmlProperty prop(obj, str, qmlContext(infoObj));
     if (!prop.isValid()) {
-        qmlInfo(infoObj) << QQuickAbstractAnimation::tr("Cannot animate non-existent property \"%1\"").arg(str);
+        const QString message = QQuickAbstractAnimation::tr("Cannot animate non-existent property \"%1\"").arg(str);
+        if (errorMessage)
+            *errorMessage = message;
+        else
+            qmlInfo(infoObj) << message;
         return QQmlProperty();
     } else if (!prop.isWritable()) {
-        qmlInfo(infoObj) << QQuickAbstractAnimation::tr("Cannot animate read-only property \"%1\"").arg(str);
+        const QString message = QQuickAbstractAnimation::tr("Cannot animate read-only property \"%1\"").arg(str);
+        if (errorMessage)
+            *errorMessage = message;
+        else
+            qmlInfo(infoObj) << message;
         return QQmlProperty();
     }
     return prop;
@@ -875,7 +881,7 @@ void QActionAnimation::updateState(State newState, State oldState)
 
 void QActionAnimation::debugAnimation(QDebug d) const
 {
-    d << "ActionAnimation(" << hex << (void *) this << dec << ")";
+    d << "ActionAnimation(" << hex << (const void *) this << dec << ")";
 
     if (animAction) {
         int indentLevel = 1;
@@ -980,7 +986,7 @@ void QQuickScriptActionPrivate::debugAction(QDebug d, int indentLevel) const
         QByteArray ind(indentLevel, ' ');
         QString exprStr = expr.expression();
         int endOfFirstLine = exprStr.indexOf('\n');
-        d << "\n" << ind.constData() << exprStr.left(endOfFirstLine);
+        d << "\n" << ind.constData() << exprStr.leftRef(endOfFirstLine);
         if (endOfFirstLine != -1 && endOfFirstLine < exprStr.length())
             d << "...";
     }
@@ -1435,7 +1441,7 @@ QQuickVector3dAnimation::~QQuickVector3dAnimation()
 }
 
 /*!
-    \qmlproperty real QtQuick::Vector3dAnimation::from
+    \qmlproperty vector3d QtQuick::Vector3dAnimation::from
     This property holds the starting value for the animation.
 
     If the Vector3dAnimation is defined within a \l Transition or \l Behavior,
@@ -1457,7 +1463,7 @@ void QQuickVector3dAnimation::setFrom(QVector3D f)
 }
 
 /*!
-    \qmlproperty real QtQuick::Vector3dAnimation::to
+    \qmlproperty vector3d QtQuick::Vector3dAnimation::to
     This property holds the end value for the animation.
 
     If the Vector3dAnimation is defined within a \l Transition or \l Behavior,
@@ -1968,7 +1974,7 @@ void QQuickBulkValueAnimator::topLevelAnimationLoopChanged()
 
 void QQuickBulkValueAnimator::debugAnimation(QDebug d) const
 {
-    d << "BulkValueAnimation(" << hex << (void *) this << dec << ")" << "duration:" << duration();
+    d << "BulkValueAnimation(" << hex << (const void *) this << dec << ")" << "duration:" << duration();
 
     if (animValue) {
         int indentLevel = 1;
@@ -2152,7 +2158,10 @@ void QQuickPropertyAnimation::setTo(const QVariant &t)
     \c Easing.Linear.
 
     \qml
-    PropertyAnimation { properties: "y"; easing.type: Easing.InOutElastic; easing.amplitude: 2.0; easing.period: 1.5 }
+    PropertyAnimation { properties: "y";
+                        easing.type: Easing.InOutElastic;
+                        easing.amplitude: 2.0;
+                        easing.period: 1.5 }
     \endqml
 
     Available types are:
@@ -2184,7 +2193,7 @@ void QQuickPropertyAnimation::setTo(const QVariant &t)
         \li \inlineimage qeasingcurve-incubic.png
     \row
         \li \c Easing.OutCubic
-        \li Easing curve for a cubic (t^3) function: decelerating from zero velocity.
+        \li Easing curve for a cubic (t^3) function: decelerating to zero velocity.
         \li \inlineimage qeasingcurve-outcubic.png
     \row
         \li \c Easing.InOutCubic
@@ -2200,7 +2209,7 @@ void QQuickPropertyAnimation::setTo(const QVariant &t)
         \li \inlineimage qeasingcurve-inquart.png
     \row
         \li \c Easing.OutQuart
-        \li Easing curve for a quartic (t^4) function: decelerating from zero velocity.
+        \li Easing curve for a quartic (t^4) function: decelerating to zero velocity.
         \li \inlineimage qeasingcurve-outquart.png
     \row
         \li \c Easing.InOutQuart
@@ -2216,7 +2225,7 @@ void QQuickPropertyAnimation::setTo(const QVariant &t)
         \li \inlineimage qeasingcurve-inquint.png
     \row
         \li \c Easing.OutQuint
-        \li Easing curve for a quintic (t^5) function: decelerating from zero velocity.
+        \li Easing curve for a quintic (t^5) function: decelerating to zero velocity.
         \li \inlineimage qeasingcurve-outquint.png
     \row
         \li \c Easing.InOutQuint
@@ -2232,7 +2241,7 @@ void QQuickPropertyAnimation::setTo(const QVariant &t)
         \li \inlineimage qeasingcurve-insine.png
     \row
         \li \c Easing.OutSine
-        \li Easing curve for a sinusoidal (sin(t)) function: decelerating from zero velocity.
+        \li Easing curve for a sinusoidal (sin(t)) function: decelerating to zero velocity.
         \li \inlineimage qeasingcurve-outsine.png
     \row
         \li \c Easing.InOutSine
@@ -2248,7 +2257,7 @@ void QQuickPropertyAnimation::setTo(const QVariant &t)
         \li \inlineimage qeasingcurve-inexpo.png
     \row
         \li \c Easing.OutExpo
-        \li Easing curve for an exponential (2^t) function: decelerating from zero velocity.
+        \li Easing curve for an exponential (2^t) function: decelerating to zero velocity.
         \li \inlineimage qeasingcurve-outexpo.png
     \row
         \li \c Easing.InOutExpo
@@ -2264,7 +2273,7 @@ void QQuickPropertyAnimation::setTo(const QVariant &t)
         \li \inlineimage qeasingcurve-incirc.png
     \row
         \li \c Easing.OutCirc
-        \li Easing curve for a circular (sqrt(1-t^2)) function: decelerating from zero velocity.
+        \li Easing curve for a circular (sqrt(1-t^2)) function: decelerating to zero velocity.
         \li \inlineimage qeasingcurve-outcirc.png
     \row
         \li \c Easing.InOutCirc
@@ -2281,7 +2290,7 @@ void QQuickPropertyAnimation::setTo(const QVariant &t)
         \li \inlineimage qeasingcurve-inelastic.png
     \row
         \li \c Easing.OutElastic
-        \li Easing curve for an elastic (exponentially decaying sine wave) function: decelerating from zero velocity.
+        \li Easing curve for an elastic (exponentially decaying sine wave) function: decelerating to zero velocity.
         \br The peak amplitude can be set with the \e amplitude parameter, and the period of decay by the \e period parameter.
         \li \inlineimage qeasingcurve-outelastic.png
     \row
@@ -2314,7 +2323,7 @@ void QQuickPropertyAnimation::setTo(const QVariant &t)
         \li \inlineimage qeasingcurve-inbounce.png
     \row
         \li \c Easing.OutBounce
-        \li Easing curve for a bounce (exponentially decaying parabolic bounce) function: decelerating from zero velocity.
+        \li Easing curve for a bounce (exponentially decaying parabolic bounce) function: decelerating to zero velocity.
         \li \inlineimage qeasingcurve-outbounce.png
     \row
         \li \c Easing.InOutBounce
@@ -2589,18 +2598,28 @@ QQuickStateActions QQuickPropertyAnimation::createTransitionActions(QQuickStateA
     if (defaultTarget && targets.isEmpty())
         targets << defaultTarget;
 
+    bool usingDefaultProperties = false;
     if (props.isEmpty() && !d->defaultProperties.isEmpty()) {
         props << d->defaultProperties.split(QLatin1Char(','));
+        usingDefaultProperties = true;
     }
 
     bool hasExplicit = false;
     //an explicit animation has been specified
     if (d->toIsDefined) {
+        QVector<QString> errorMessages;
+        bool successfullyCreatedDefaultProperty = false;
+
         for (int i = 0; i < props.count(); ++i) {
             for (int j = 0; j < targets.count(); ++j) {
                 QQuickStateAction myAction;
-                myAction.property = d->createProperty(targets.at(j), props.at(i), this);
+                QString errorMessage;
+                const QString propertyName = props.at(i);
+                myAction.property = d->createProperty(targets.at(j), propertyName, this, &errorMessage);
                 if (myAction.property.isValid()) {
+                    if (usingDefaultProperties)
+                        successfullyCreatedDefaultProperty = true;
+
                     if (d->fromIsDefined) {
                         myAction.fromValue = d->from;
                         d->convertVariant(myAction.fromValue, d->interpolatorType ? d->interpolatorType : myAction.property.propertyType());
@@ -2617,9 +2636,16 @@ QQuickStateActions QQuickPropertyAnimation::createTransitionActions(QQuickStateA
                             break;  //### any chance there could be multiples?
                         }
                     }
+                } else {
+                    errorMessages.append(errorMessage);
                 }
             }
         }
+
+        if (!successfullyCreatedDefaultProperty) {
+            foreach (const QString &errorMessage, errorMessages)
+                qmlInfo(this) << errorMessage;
+        }
     }
 
     if (!hasExplicit)
