@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
@@ -10,30 +10,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -52,10 +50,19 @@
 
 QT_BEGIN_NAMESPACE
 
-QQuickAnimatorController::QQuickAnimatorController()
-    : m_window(0)
+QQuickAnimatorController::QQuickAnimatorController(QQuickWindow *window)
+    : m_window(window)
     , m_nodesAreInvalid(false)
 {
+    m_guiEntity = new QQuickAnimatorControllerGuiThreadEntity();
+    m_guiEntity->controller = this;
+    connect(window, SIGNAL(frameSwapped()), m_guiEntity, SLOT(frameSwapped()));
+}
+
+void QQuickAnimatorControllerGuiThreadEntity::frameSwapped()
+{
+    if (!controller.isNull())
+        controller->stopProxyJobs();
 }
 
 QQuickAnimatorController::~QQuickAnimatorController()
@@ -71,14 +78,18 @@ QQuickAnimatorController::~QQuickAnimatorController()
 
     foreach (QQuickAnimatorProxyJob *proxy, m_animatorRoots)
         proxy->controllerWasDeleted();
-    qDeleteAll(m_animatorRoots.keys());
+    for (auto it = m_animatorRoots.keyBegin(), end = m_animatorRoots.keyEnd(); it != end; ++it)
+        delete *it;
 
     // Delete those who have been started, stopped and are now still
     // pending for restart.
-    foreach (QAbstractAnimationJob *job, m_starting.keys()) {
+    for (auto it = m_starting.keyBegin(), end = m_starting.keyEnd(); it != end; ++it) {
+        QAbstractAnimationJob *job = *it;
         if (!m_animatorRoots.contains(job))
             delete job;
     }
+
+    delete m_guiEntity;
 }
 
 static void qquickanimator_invalidate_node(QAbstractAnimationJob *job)
@@ -137,6 +148,11 @@ static void qquick_initialize_helper(QAbstractAnimationJob *job, QQuickAnimatorC
 {
     if (job->isRenderThreadJob()) {
         QQuickAnimatorJob *j = static_cast<QQuickAnimatorJob *>(job);
+        // Note: since a QQuickAnimatorJob::m_target is a QPointer,
+        // if m_target is destroyed between the time it was set
+        // as the target of the animator job and before this step,
+        // (e.g a Loader being set inactive just after starting the animator)
+        // we are sure it will be NULL and won't be dangling around
         if (!j->target()) {
             return;
         } else if (c->m_deletedSinceLastFrame.contains(j->target())) {
@@ -192,8 +208,6 @@ void QQuickAnimatorController::beforeNodeSync()
         m_nodesAreInvalid = false;
     }
 
-
-
     foreach (QQuickAnimatorJob *job, m_activeLeafAnimations) {
         if (!job->target())
             continue;
@@ -205,9 +219,9 @@ void QQuickAnimatorController::beforeNodeSync()
         }
     }
     foreach (QQuickItem *wiped, m_deletedSinceLastFrame) {
-        QQuickTransformAnimatorJob::Helper *helper = m_transforms.value(wiped);
-        if (helper)
-            helper->item = 0;
+        QQuickTransformAnimatorJob::Helper *helper = m_transforms.take(wiped);
+        // Helper will now already have been reset in all animators referencing it.
+        delete helper;
     }
 
     m_deletedSinceLastFrame.clear();
@@ -216,11 +230,32 @@ void QQuickAnimatorController::beforeNodeSync()
 void QQuickAnimatorController::afterNodeSync()
 {
     foreach (QQuickAnimatorJob *job, m_activeLeafAnimations) {
-        if (job->isUniform() && job->target())
-            static_cast<QQuickUniformAnimatorJob *>(job)->afterNodeSync();
+        if (job->target())
+            job->afterNodeSync();
     }
 }
 
+void QQuickAnimatorController::proxyWasDestroyed(QQuickAnimatorProxyJob *proxy)
+{
+    lock();
+    m_proxiesToStop.remove(proxy);
+    unlock();
+}
+
+void QQuickAnimatorController::stopProxyJobs()
+{
+    // Need to make a copy under lock and then stop while unlocked.
+    // Stopping triggers writeBack which in turn may lock, so it needs
+    // to be outside the lock. It is also safe because deletion of
+    // proxies happens on the GUI thread, where this code is also executing.
+    lock();
+    QSet<QQuickAnimatorProxyJob *> jobs = m_proxiesToStop;
+    m_proxiesToStop.clear();
+    unlock();
+    foreach (QQuickAnimatorProxyJob *p, jobs)
+        p->stop();
+}
+
 void QQuickAnimatorController::animationFinished(QAbstractAnimationJob *job)
 {
     /* We are currently on the render thread and m_deleting is primarily
@@ -231,8 +266,10 @@ void QQuickAnimatorController::animationFinished(QAbstractAnimationJob *job)
      */
     if (!m_deleting.contains(job)) {
         QQuickAnimatorProxyJob *proxy = m_animatorRoots[job];
-        if (proxy)
-            QCoreApplication::postEvent(proxy, new QEvent(QEvent::User));
+        if (proxy) {
+            m_window->update();
+            m_proxiesToStop << proxy;
+        }
         // else already gone...
     }
 }
@@ -262,13 +299,16 @@ void QQuickAnimatorController::requestSync()
 // These functions are called on the GUI thread.
 void QQuickAnimatorController::startJob(QQuickAnimatorProxyJob *proxy, QAbstractAnimationJob *job)
 {
+    proxy->markJobManagedByController();
     m_starting[job] = proxy;
+    m_stopping.remove(job);
     requestSync();
 }
 
 void QQuickAnimatorController::stopJob(QQuickAnimatorProxyJob *proxy, QAbstractAnimationJob *job)
 {
     m_stopping[job] = proxy;
+    m_starting.remove(job);
     requestSync();
 }
 
