@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
@@ -10,30 +10,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -50,6 +48,7 @@
 #include <private/qanimationgroupjob_p.h>
 
 #include <qcoreapplication.h>
+#include <qdebug.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -57,6 +56,7 @@ QQuickAnimatorProxyJob::QQuickAnimatorProxyJob(QAbstractAnimationJob *job, QObje
     : m_controller(0)
     , m_job(job)
     , m_internalState(State_Stopped)
+    , m_jobManagedByController(false)
 {
     m_isRenderThreadProxy = true;
     m_animation = qobject_cast<QQuickAbstractAnimation *>(item);
@@ -85,13 +85,16 @@ QQuickAnimatorProxyJob::QQuickAnimatorProxyJob(QAbstractAnimationJob *job, QObje
         QQuickItem *item = qobject_cast<QQuickItem *>(ctx);
         if (item->window())
             setWindow(item->window());
-        connect(item, SIGNAL(windowChanged(QQuickWindow*)), this, SLOT(windowChanged(QQuickWindow*)));
+
+        qmlobject_connect(item, QQuickItem, SIGNAL(windowChanged(QQuickWindow*)), this, QQuickAnimatorProxyJob, SLOT(windowChanged(QQuickWindow*)));
     }
 }
 
 QQuickAnimatorProxyJob::~QQuickAnimatorProxyJob()
 {
     deleteJob();
+    if (m_controller)
+        m_controller->proxyWasDestroyed(this);
 }
 
 void QQuickAnimatorProxyJob::deleteJob()
@@ -101,7 +104,10 @@ void QQuickAnimatorProxyJob::deleteJob()
         // so delete it through the controller to clean up properly.
         if (m_controller)
             m_controller->deleteJob(m_job);
-        else
+
+        // We explicitly delete the job if the animator controller has never touched
+        // it. If it has, it will have ownership as well.
+        else if (!m_jobManagedByController)
             delete m_job;
         m_job = 0;
     }
@@ -127,14 +133,20 @@ void QQuickAnimatorProxyJob::updateState(QAbstractAnimationJob::State newState,
             m_controller->startJob(this, m_job);
     } else if (newState == Stopped) {
         syncBackCurrentValues();
-        if (m_internalState == State_Starting)
-            m_internalState = State_Stopped;
-        else if (m_controller) {
+        m_internalState = State_Stopped;
+        if (m_controller) {
             m_controller->stopJob(this, m_job);
         }
     }
 }
 
+void QQuickAnimatorProxyJob::debugAnimation(QDebug d) const
+{
+    d << "QuickAnimatorProxyJob("<< hex << (const void *) this << dec
+      << "state:" << state() << "duration:" << duration()
+      << "proxying: (" << job() << ')';
+}
+
 void QQuickAnimatorProxyJob::windowChanged(QQuickWindow *window)
 {
     setWindow(window);
@@ -149,11 +161,15 @@ void QQuickAnimatorProxyJob::controllerWasDeleted()
 void QQuickAnimatorProxyJob::setWindow(QQuickWindow *window)
 {
     if (!window) {
-        // Stop will trigger syncBackCurrentValues so best to do it before
-        // we delete m_job.
         stop();
         deleteJob();
-        return;
+
+        // Upon leaving a window, we reset the controller. This means that
+        // animators will only enter the Starting phase and won't be making
+        // calls to QQuickAnimatorController::startjob().
+        if (m_controller)
+            m_controller->proxyWasDestroyed(this);
+        m_controller = 0;
 
     } else if (!m_controller && m_job) {
         m_controller = QQuickWindowPrivate::get(window)->animationController;
@@ -181,15 +197,6 @@ void QQuickAnimatorProxyJob::startedByController()
     m_internalState = State_Running;
 }
 
-bool QQuickAnimatorProxyJob::event(QEvent *e)
-{
-    if (e->type() == QEvent::User) {
-        stop();
-        return true;
-    }
-    return QObject::event(e);
-}
-
 static void qquick_syncback_helper(QAbstractAnimationJob *job)
 {
     if (job->isRenderThreadJob()) {
@@ -224,6 +231,17 @@ QQuickAnimatorJob::QQuickAnimatorJob()
     m_isRenderThreadJob = true;
 }
 
+void QQuickAnimatorJob::debugAnimation(QDebug d) const
+{
+    d << "QuickAnimatorJob(" << hex << (const void *) this << dec
+      << ") state:" << state() << "duration:" << duration()
+      << "target:" << m_target << "value:" << m_value;
+}
+
+qreal QQuickAnimatorJob::progress(int time) const
+{
+    return m_easing.valueForProgress((m_duration == 0) ? qreal(1) : qreal(time) / qreal(m_duration));
+}
 qreal QQuickAnimatorJob::value() const
 {
     qreal v;
@@ -258,6 +276,10 @@ QQuickTransformAnimatorJob::QQuickTransformAnimatorJob()
 QQuickTransformAnimatorJob::~QQuickTransformAnimatorJob()
 {
     if (m_helper && --m_helper->ref == 0) {
+        // The only condition for not having a controller is when target was
+        // destroyed, in which case we have neither m_helper nor m_contorller.
+        Q_ASSERT(m_controller);
+        Q_ASSERT(m_helper->item);
         m_controller->m_transforms.remove(m_helper->item);
         delete m_helper;
     }
@@ -268,6 +290,7 @@ void QQuickTransformAnimatorJob::initialize(QQuickAnimatorController *controller
     QQuickAnimatorJob::initialize(controller);
 
     if (m_controller) {
+        bool newHelper = m_helper == 0;
         m_helper = m_controller->m_transforms.value(m_target);
         if (!m_helper) {
             m_helper = new Helper();
@@ -275,7 +298,8 @@ void QQuickTransformAnimatorJob::initialize(QQuickAnimatorController *controller
             m_controller->m_transforms.insert(m_target, m_helper);
             QObject::connect(m_target, SIGNAL(destroyed(QObject*)), m_controller, SLOT(itemDestroyed(QObject*)), Qt::DirectConnection);
         } else {
-            ++m_helper->ref;
+            if (newHelper) // only add reference the first time around..
+                ++m_helper->ref;
             // Make sure leftovers from previous runs are being used...
             m_helper->wasSynced = false;
         }
@@ -289,6 +313,12 @@ void QQuickTransformAnimatorJob::nodeWasDestroyed()
         m_helper->node = 0;
 }
 
+void QQuickTransformAnimatorJob::targetWasDeleted()
+{
+    m_helper = 0;
+    QQuickAnimatorJob::targetWasDeleted();
+}
+
 void QQuickTransformAnimatorJob::Helper::sync()
 {
     const quint32 mask = QQuickItemPrivate::Position
@@ -360,9 +390,9 @@ void QQuickXAnimatorJob::updateCurrentTime(int time)
 {
     if (!m_controller)
         return;
-    Q_ASSERT(m_controller->m_window->openglContext()->thread() == QThread::currentThread());
+    Q_ASSERT(!m_controller->m_window->openglContext() || m_controller->m_window->openglContext()->thread() == QThread::currentThread());
 
-    m_value = m_from + (m_to - m_from) * m_easing.valueForProgress(time / (qreal) m_duration);
+    m_value = m_from + (m_to - m_from) * progress(time);
     m_helper->dx = m_value;
     m_helper->wasChanged = true;
 }
@@ -377,9 +407,9 @@ void QQuickYAnimatorJob::updateCurrentTime(int time)
 {
     if (!m_controller)
         return;
-    Q_ASSERT(m_controller->m_window->openglContext()->thread() == QThread::currentThread());
+    Q_ASSERT(!m_controller->m_window->openglContext() || m_controller->m_window->openglContext()->thread() == QThread::currentThread());
 
-    m_value = m_from + (m_to - m_from) * m_easing.valueForProgress(time / (qreal) m_duration);
+    m_value = m_from + (m_to - m_from) * progress(time);
     m_helper->dy = m_value;
     m_helper->wasChanged = true;
 }
@@ -402,21 +432,33 @@ void QQuickOpacityAnimatorJob::initialize(QQuickAnimatorController *controller)
     m_opacityNode = d->opacityNode();
     if (!m_opacityNode) {
         m_opacityNode = new QSGOpacityNode();
-        d->extra.value().opacityNode = m_opacityNode;
 
-        QSGNode *child = d->clipNode();
-        if (!child)
-            child = d->rootNode();
-        if (!child)
-            child = d->groupNode;
-
-        if (child) {
+        /* The item node subtree is like this
+         *
+         * itemNode
+         * (opacityNode)            optional
+         * (clipNode)               optional
+         * (rootNode)               optional
+         * children / paintNode
+         *
+         * If the opacity node doesn't exist, we need to insert it into
+         * the hierarchy between itemNode and clipNode or rootNode. If
+         * neither clip or root exists, we need to reparent all children
+         * from itemNode to opacityNode.
+         */
+        QSGNode *iNode = d->itemNode();
+        QSGNode *child = d->childContainerNode();
+        if (child != iNode) {
             if (child->parent())
                 child->parent()->removeChildNode(child);
             m_opacityNode->appendChildNode(child);
+            iNode->appendChildNode(m_opacityNode);
+        } else {
+            iNode->reparentChildNodesTo(m_opacityNode);
+            iNode->appendChildNode(m_opacityNode);
         }
 
-        d->itemNode()->appendChildNode(m_opacityNode);
+        d->extra.value().opacityNode = m_opacityNode;
     }
 }
 
@@ -435,9 +477,9 @@ void QQuickOpacityAnimatorJob::updateCurrentTime(int time)
 {
     if (!m_controller || !m_opacityNode)
         return;
-    Q_ASSERT(m_controller->m_window->openglContext()->thread() == QThread::currentThread());
+    Q_ASSERT(!m_controller->m_window->openglContext() || m_controller->m_window->openglContext()->thread() == QThread::currentThread());
 
-    m_value = m_from + (m_to - m_from) * m_easing.valueForProgress(time / (qreal) m_duration);
+    m_value = m_from + (m_to - m_from) * progress(time);
     m_opacityNode->setOpacity(m_value);
 }
 
@@ -451,9 +493,9 @@ void QQuickScaleAnimatorJob::updateCurrentTime(int time)
 {
     if (!m_controller)
         return;
-    Q_ASSERT(m_controller->m_window->openglContext()->thread() == QThread::currentThread());
+    Q_ASSERT(!m_controller->m_window->openglContext() || m_controller->m_window->openglContext()->thread() == QThread::currentThread());
 
-    m_value = m_from + (m_to - m_from) * m_easing.valueForProgress(time / (qreal) m_duration);
+    m_value = m_from + (m_to - m_from) * progress(time);
     m_helper->scale = m_value;
     m_helper->wasChanged = true;
 }
@@ -471,9 +513,10 @@ void QQuickRotationAnimatorJob::updateCurrentTime(int time)
 {
     if (!m_controller)
         return;
-    Q_ASSERT(m_controller->m_window->openglContext()->thread() == QThread::currentThread());
+    Q_ASSERT(!m_controller->m_window->openglContext() || m_controller->m_window->openglContext()->thread() == QThread::currentThread());
+
+    float t = progress(time);
 
-    float t =  m_easing.valueForProgress(time / (qreal) m_duration);
     switch (m_direction) {
     case QQuickRotationAnimator::Clockwise:
         m_value = _q_interpolateClockwiseRotation(m_from, m_to, t).toFloat();
@@ -551,12 +594,12 @@ void QQuickUniformAnimatorJob::updateCurrentTime(int time)
 {
     if (!m_controller)
         return;
-    Q_ASSERT(m_controller->m_window->openglContext()->thread() == QThread::currentThread());
+    Q_ASSERT(!m_controller->m_window->openglContext() || m_controller->m_window->openglContext()->thread() == QThread::currentThread());
 
     if (!m_node || m_uniformIndex == -1 || m_uniformType == -1)
         return;
 
-    m_value = m_from + (m_to - m_from) * m_easing.valueForProgress(time / (qreal) m_duration);
+    m_value = m_from + (m_to - m_from) * progress(time);
 
     QQuickShaderEffectMaterial *material =
             static_cast<QQuickShaderEffectMaterial *>(m_node->material());
