@@ -1,7 +1,7 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
@@ -10,30 +10,28 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -55,6 +53,8 @@
 #include <qqmlinfo.h>
 #include <qqmlfile.h>
 
+#include <QtCore/QCoreApplication>
+
 QT_BEGIN_NAMESPACE
 
 #define FONTLOADER_MAXIMUM_REDIRECT_RECURSION 16
@@ -140,10 +140,50 @@ public:
     QUrl url;
     QString name;
     QQuickFontLoader::Status status;
-    static QHash<QUrl, QQuickFontObject*> fonts;
 };
 
-QHash<QUrl, QQuickFontObject*> QQuickFontLoaderPrivate::fonts;
+static void q_QFontLoaderFontsStaticReset();
+static void q_QFontLoaderFontsAddReset()
+{
+    qAddPostRoutine(q_QFontLoaderFontsStaticReset);
+}
+class QFontLoaderFonts
+{
+public:
+    QFontLoaderFonts()
+    {
+        qAddPostRoutine(q_QFontLoaderFontsStaticReset);
+        qAddPreRoutine(q_QFontLoaderFontsAddReset);
+    }
+
+    ~QFontLoaderFonts()
+    {
+        qRemovePostRoutine(q_QFontLoaderFontsStaticReset);
+        reset();
+    }
+
+
+    void reset()
+    {
+        QVector<QQuickFontObject *> deleted;
+        QHash<QUrl, QQuickFontObject*>::iterator it;
+        for (it = map.begin(); it != map.end(); ++it) {
+            if (!deleted.contains(it.value())) {
+                deleted.append(it.value());
+                delete it.value();
+            }
+        }
+        map.clear();
+    }
+
+    QHash<QUrl, QQuickFontObject *> map;
+};
+Q_GLOBAL_STATIC(QFontLoaderFonts, fontLoaderFonts);
+
+static void q_QFontLoaderFontsStaticReset()
+{
+    fontLoaderFonts()->reset();
+}
 
 /*!
     \qmltype FontLoader
@@ -170,7 +210,7 @@ QHash<QUrl, QQuickFontObject*> QQuickFontLoaderPrivate::fonts;
     }
     \endqml
 
-    \sa {declarative/text/fonts}{Fonts example}
+    \sa {Qt Quick Examples - Text#Fonts}{Qt Quick Examples - Text Fonts}
 */
 QQuickFontLoader::QQuickFontLoader(QObject *parent)
     : QObject(*(new QQuickFontLoaderPrivate), parent)
@@ -201,29 +241,29 @@ void QQuickFontLoader::setSource(const QUrl &url)
 
     QString localFile = QQmlFile::urlToLocalFileOrQrc(d->url);
     if (!localFile.isEmpty()) {
-        if (!d->fonts.contains(d->url)) {
+        if (!fontLoaderFonts()->map.contains(d->url)) {
             int id = QFontDatabase::addApplicationFont(localFile);
             if (id != -1) {
                 updateFontInfo(QFontDatabase::applicationFontFamilies(id).at(0), Ready);
                 QQuickFontObject *fo = new QQuickFontObject(id);
-                d->fonts[d->url] = fo;
+                fontLoaderFonts()->map[d->url] = fo;
             } else {
                 updateFontInfo(QString(), Error);
             }
         } else {
-            updateFontInfo(QFontDatabase::applicationFontFamilies(d->fonts[d->url]->id).at(0), Ready);
+            updateFontInfo(QFontDatabase::applicationFontFamilies(fontLoaderFonts()->map[d->url]->id).at(0), Ready);
         }
     } else {
-        if (!d->fonts.contains(d->url)) {
+        if (!fontLoaderFonts()->map.contains(d->url)) {
             QQuickFontObject *fo = new QQuickFontObject;
-            d->fonts[d->url] = fo;
+            fontLoaderFonts()->map[d->url] = fo;
             fo->download(d->url, qmlEngine(this)->networkAccessManager());
             d->status = Loading;
             emit statusChanged();
             QObject::connect(fo, SIGNAL(fontDownloaded(QString,QQuickFontLoader::Status)),
                 this, SLOT(updateFontInfo(QString,QQuickFontLoader::Status)));
         } else {
-            QQuickFontObject *fo = d->fonts[d->url];
+            QQuickFontObject *fo = fontLoaderFonts()->map[d->url];
             if (fo->id == -1) {
                 d->status = Loading;
                 emit statusChanged();
