@@ -1,31 +1,37 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:LGPL$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 3 as published by the Free Software
+** Foundation and appearing in the file LICENSE.LGPL3 included in the
+** packaging of this file. Please review the following information to
+** ensure the GNU Lesser General Public License version 3 requirements
+** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
 **
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 2.0 or (at your option) the GNU General
+** Public license version 3 or any later version approved by the KDE Free
+** Qt Foundation. The licenses are as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-2.0.html and
+** https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -39,8 +45,9 @@
 #include "private/qquickitemchangelistener_p.h"
 #include "private/qquickrendercontrol_p.h"
 
+#include <private/qqmldebugconnector_p.h>
 #include <private/qquickprofiler_p.h>
-#include <private/qqmlinspectorservice_p.h>
+#include <private/qqmldebugserviceinterfaces_p.h>
 #include <private/qqmlmemoryprofiler_p.h>
 
 #include <QtQml/qqmlengine.h>
@@ -89,15 +96,9 @@ void QQuickWidgetPrivate::init(QQmlEngine* e)
 
     engine = e;
 
-    if (engine.isNull())
-        engine = new QQmlEngine(q);
-
-    if (!engine.data()->incubationController())
+    if (!engine.isNull() && !engine.data()->incubationController())
         engine.data()->setIncubationController(offscreenWindow->incubationController());
 
-    if (QQmlDebugService::isDebuggingEnabled())
-        QQmlInspectorService::instance()->addView(q);
-
 #ifndef QT_NO_DRAGANDDROP
     q->setAcceptDrops(true);
 #endif
@@ -108,6 +109,16 @@ void QQuickWidgetPrivate::init(QQmlEngine* e)
     QObject::connect(renderControl, SIGNAL(sceneChanged()), q, SLOT(triggerUpdate()));
 }
 
+void QQuickWidgetPrivate::ensureEngine() const
+{
+    Q_Q(const QQuickWidget);
+    if (!engine.isNull())
+        return;
+
+    engine = new QQmlEngine(const_cast<QQuickWidget*>(q));
+    engine.data()->setIncubationController(offscreenWindow->incubationController());
+}
+
 void QQuickWidgetPrivate::invalidateRenderControl()
 {
     if (!context) // this is not an error, could be called before creating the context, or multiple times
@@ -142,14 +153,12 @@ QQuickWidgetPrivate::QQuickWidgetPrivate()
     , eventPending(false)
     , updatePending(false)
     , fakeHidden(false)
+    , requestedSamples(0)
 {
 }
 
 QQuickWidgetPrivate::~QQuickWidgetPrivate()
 {
-    if (QQmlDebugService::isDebuggingEnabled())
-        QQmlInspectorService::instance()->removeView(q_func());
-
     invalidateRenderControl();
 
     // context and offscreenSurface are current at this stage, if the context was created.
@@ -165,10 +174,7 @@ QQuickWidgetPrivate::~QQuickWidgetPrivate()
 void QQuickWidgetPrivate::execute()
 {
     Q_Q(QQuickWidget);
-    if (!engine) {
-        qWarning() << "QQuickWidget: invalid qml engine.";
-        return;
-    }
+    ensureEngine();
 
     if (root) {
         delete root;
@@ -202,6 +208,13 @@ void QQuickWidgetPrivate::itemGeometryChanged(QQuickItem *resizeItem, const QRec
 
 void QQuickWidgetPrivate::render(bool needsSync)
 {
+    // createFramebufferObject() bails out when the size is empty. In this case
+    // we cannot render either.
+    if (!fbo)
+        return;
+
+    Q_ASSERT(context);
+
     if (!context->makeCurrent(offscreenSurface)) {
         qWarning("QQuickWidget: Cannot render due to failing makeCurrent()");
         return;
@@ -241,8 +254,15 @@ void QQuickWidgetPrivate::renderSceneGraph()
     }
 
     Q_ASSERT(offscreenSurface);
+
     render(true);
-    q->update(); // schedule composition
+
+#ifndef QT_NO_GRAPHICSVIEW
+    if (q->window()->graphicsProxyWidget())
+        QWidgetPrivate::nearestGraphicsProxyWidget(q)->update();
+    else
+#endif
+        q->update(); // schedule composition
 }
 
 QImage QQuickWidgetPrivate::grabFramebuffer()
@@ -336,8 +356,8 @@ QObject *QQuickWidgetPrivate::focusObject()
     This limitation only applies when there are other widgets underneath the QQuickWidget
     inside the same window. Making the window semi-transparent, with other applications
     and the desktop visible in the background, is done in the traditional way: Set
-    Qt::WA_TranslucentBackground and change the Qt Quick Scenegraph's clear color to
-    Qt::transparent via setClearColor().
+    Qt::WA_TranslucentBackground on the top-level window, request an alpha channel, and
+    change the Qt Quick Scenegraph's clear color to Qt::transparent via setClearColor().
 
     \sa {Exposing Attributes of C++ Types to QML}, {Qt Quick Widgets Example}, QQuickView
 */
@@ -388,7 +408,6 @@ QQuickWidget::QQuickWidget(QQmlEngine* engine, QWidget *parent)
 {
     setMouseTracking(true);
     setFocusPolicy(Qt::StrongFocus);
-    Q_ASSERT(engine);
     d_func()->init(engine);
 }
 
@@ -474,7 +493,8 @@ QUrl QQuickWidget::source() const
 QQmlEngine* QQuickWidget::engine() const
 {
     Q_D(const QQuickWidget);
-    return d->engine ? const_cast<QQmlEngine *>(d->engine.data()) : 0;
+    d->ensureEngine();
+    return const_cast<QQmlEngine *>(d->engine.data());
 }
 
 /*!
@@ -487,7 +507,8 @@ QQmlEngine* QQuickWidget::engine() const
 QQmlContext* QQuickWidget::rootContext() const
 {
     Q_D(const QQuickWidget);
-    return d->engine ? d->engine.data()->rootContext() : 0;
+    d->ensureEngine();
+    return d->engine.data()->rootContext();
 }
 
 /*!
@@ -532,12 +553,15 @@ QQmlContext* QQuickWidget::rootContext() const
 QQuickWidget::Status QQuickWidget::status() const
 {
     Q_D(const QQuickWidget);
-    if (!d->engine)
+    if (!d->engine && !d->source.isEmpty())
         return QQuickWidget::Error;
 
     if (!d->component)
         return QQuickWidget::Null;
 
+    if (d->component->status() == QQmlComponent::Ready && !d->root)
+        return QQuickWidget::Error;
+
     return QQuickWidget::Status(d->component->status());
 }
 
@@ -555,11 +579,16 @@ QList<QQmlError> QQuickWidget::errors() const
     if (d->component)
         errs = d->component->errors();
 
-    if (!d->engine) {
+    if (!d->engine && !d->source.isEmpty()) {
         QQmlError error;
         error.setDescription(QLatin1String("QQuickWidget: invalid qml engine."));
         errs << error;
     }
+    if (d->component && d->component->status() == QQmlComponent::Ready && !d->root) {
+        QQmlError error;
+        error.setDescription(QLatin1String("QQuickWidget: invalid root object."));
+        errs << error;
+    }
 
     return errs;
 }
@@ -687,6 +716,7 @@ void QQuickWidgetPrivate::handleContextCreationFailure(const QSurfaceFormat &for
 
 void QQuickWidgetPrivate::createContext()
 {
+    Q_Q(QQuickWidget);
     // On hide-show we invalidate() but our context is kept.
     // We nonetheless need to initialize() again.
     const bool reinit = context && !offscreenWindow->openglContext();
@@ -698,11 +728,13 @@ void QQuickWidgetPrivate::createContext()
         context = new QOpenGLContext;
         context->setFormat(offscreenWindow->requestedFormat());
 
-        if (qt_gl_global_share_context()) {
-            context->setShareContext(qt_gl_global_share_context());
-            context->setScreen(context->shareContext()->screen());
+        QOpenGLContext *shareContext = qt_gl_global_share_context();
+        if (!shareContext)
+            shareContext = QWidgetPrivate::get(q->window())->shareContext();
+        if (shareContext) {
+            context->setShareContext(shareContext);
+            context->setScreen(shareContext->screen());
         }
-
         if (!context->create()) {
             const bool isEs = context->isOpenGLES();
             delete context;
@@ -750,9 +782,10 @@ void QQuickWidget::createFramebufferObject()
         return;
     }
 
-    if (context->shareContext() != QWidgetPrivate::get(window())->shareContext()) {
-        context->setShareContext(QWidgetPrivate::get(window())->shareContext());
-        context->setScreen(context->shareContext()->screen());
+    QOpenGLContext *shareWindowContext = QWidgetPrivate::get(window())->shareContext();
+    if (shareWindowContext && context->shareContext() != shareWindowContext) {
+        context->setShareContext(shareWindowContext);
+        context->setScreen(shareWindowContext->screen());
         if (!context->create())
             qWarning("QQuickWidget: Failed to recreate context");
         // The screen may be different so we must recreate the offscreen surface too.
@@ -764,7 +797,7 @@ void QQuickWidget::createFramebufferObject()
 
     context->makeCurrent(d->offscreenSurface);
 
-    int samples = d->offscreenWindow->requestedFormat().samples();
+    int samples = d->requestedSamples;
     if (!QOpenGLExtensions(context).hasOpenGLExtension(QOpenGLExtensions::FramebufferMultisample))
         samples = 0;
 
@@ -878,14 +911,15 @@ void QQuickWidgetPrivate::setRootObject(QObject *obj)
     if (QQuickItem *sgItem = qobject_cast<QQuickItem *>(obj)) {
         root = sgItem;
         sgItem->setParentItem(offscreenWindow->contentItem());
+    } else if (qobject_cast<QWindow *>(obj)) {
+        qWarning() << "QQuickWidget does not support using windows as a root item." << endl
+                   << endl
+                   << "If you wish to create your root window from QML, consider using QQmlApplicationEngine instead." << endl;
     } else {
         qWarning() << "QQuickWidget only supports loading of root objects that derive from QQuickItem." << endl
                    << endl
-                   << "If your example is using QML 2, (such as qmlscene) and the .qml file you" << endl
-                   << "loaded has 'import QtQuick 1.0' or 'import Qt 4.7', this error will occur." << endl
-                   << endl
-                   << "To load files with 'import QtQuick 1.0' or 'import Qt 4.7', use the" << endl
-                   << "QDeclarativeView class in the Qt Quick 1 module." << endl;
+                   << "Ensure your QML code is written for QtQuick 2, and uses a root that is or" << endl
+                   << "inherits from QtQuick's Item (not a Timer, QtObject, etc)." << endl;
         delete obj;
         root = 0;
     }
@@ -959,7 +993,9 @@ QSize QQuickWidget::initialSize() const
 }
 
 /*!
-  Returns the view's root \l {QQuickItem} {item}.
+  Returns the view's root \l {QQuickItem} {item}. Can be null
+  when setContents/setSource has not been called, if they were called with
+  broken QtQuick code or while the QtQuick contents are being created.
  */
 QQuickItem *QQuickWidget::rootObject() const
 {
@@ -1020,7 +1056,8 @@ void QQuickWidget::resizeEvent(QResizeEvent *e)
 void QQuickWidget::keyPressEvent(QKeyEvent *e)
 {
     Q_D(QQuickWidget);
-    Q_QUICK_INPUT_PROFILE(addEvent<QQuickProfiler::Key>());
+    Q_QUICK_INPUT_PROFILE(QQuickProfiler::Key, QQuickProfiler::InputKeyPress, e->key(),
+                          e->modifiers());
 
     QCoreApplication::sendEvent(d->offscreenWindow, e);
 }
@@ -1029,7 +1066,8 @@ void QQuickWidget::keyPressEvent(QKeyEvent *e)
 void QQuickWidget::keyReleaseEvent(QKeyEvent *e)
 {
     Q_D(QQuickWidget);
-    Q_QUICK_INPUT_PROFILE(addEvent<QQuickProfiler::Key>());
+    Q_QUICK_INPUT_PROFILE(QQuickProfiler::Key, QQuickProfiler::InputKeyRelease, e->key(),
+                          e->modifiers());
 
     QCoreApplication::sendEvent(d->offscreenWindow, e);
 }
@@ -1038,7 +1076,8 @@ void QQuickWidget::keyReleaseEvent(QKeyEvent *e)
 void QQuickWidget::mouseMoveEvent(QMouseEvent *e)
 {
     Q_D(QQuickWidget);
-    Q_QUICK_INPUT_PROFILE(addEvent<QQuickProfiler::Mouse>());
+    Q_QUICK_INPUT_PROFILE(QQuickProfiler::Mouse, QQuickProfiler::InputMouseMove, e->localPos().x(),
+                          e->localPos().y());
 
     // Use the constructor taking localPos and screenPos. That puts localPos into the
     // event's localPos and windowPos, and screenPos into the event's screenPos. This way
@@ -1053,7 +1092,8 @@ void QQuickWidget::mouseMoveEvent(QMouseEvent *e)
 void QQuickWidget::mouseDoubleClickEvent(QMouseEvent *e)
 {
     Q_D(QQuickWidget);
-    Q_QUICK_INPUT_PROFILE(addEvent<QQuickProfiler::Mouse>());
+    Q_QUICK_INPUT_PROFILE(QQuickProfiler::Mouse, QQuickProfiler::InputMouseDoubleClick,
+                          e->button(), e->buttons());
 
     // As the second mouse press is suppressed in widget windows we emulate it here for QML.
     // See QTBUG-25831
@@ -1070,9 +1110,31 @@ void QQuickWidget::mouseDoubleClickEvent(QMouseEvent *e)
 void QQuickWidget::showEvent(QShowEvent *)
 {
     Q_D(QQuickWidget);
-    d->updatePending = false;
     d->createContext();
-    d->render(true);
+    if (d->offscreenWindow->openglContext()) {
+        d->render(true);
+        // render() may have led to a QQuickWindow::update() call (for
+        // example, having a scene with a QQuickFramebufferObject::Renderer
+        // calling update() in its render()) which in turn results in
+        // renderRequested in the rendercontrol, ending up in
+        // triggerUpdate. In this case just calling update() is not
+        // acceptable, we need the full renderSceneGraph issued from
+        // timerEvent().
+        if (!d->eventPending && d->updatePending) {
+            d->updatePending = false;
+            update();
+        }
+    } else {
+        triggerUpdate();
+    }
+    QWindowPrivate *offscreenPrivate = QWindowPrivate::get(d->offscreenWindow);
+    if (!offscreenPrivate->visible) {
+        offscreenPrivate->visible = true;
+        emit d->offscreenWindow->visibleChanged(true);
+        offscreenPrivate->updateVisibility();
+    }
+    if (QQmlInspectorService *service = QQmlDebugConnector::service<QQmlInspectorService>())
+        service->setParentWindow(d->offscreenWindow, window()->windowHandle());
 }
 
 /*! \reimp */
@@ -1080,13 +1142,22 @@ void QQuickWidget::hideEvent(QHideEvent *)
 {
     Q_D(QQuickWidget);
     d->invalidateRenderControl();
+    QWindowPrivate *offscreenPrivate = QWindowPrivate::get(d->offscreenWindow);
+    if (offscreenPrivate->visible) {
+        offscreenPrivate->visible = false;
+        emit d->offscreenWindow->visibleChanged(false);
+        offscreenPrivate->updateVisibility();
+    }
+    if (QQmlInspectorService *service = QQmlDebugConnector::service<QQmlInspectorService>())
+        service->setParentWindow(d->offscreenWindow, d->offscreenWindow);
 }
 
 /*! \reimp */
 void QQuickWidget::mousePressEvent(QMouseEvent *e)
 {
     Q_D(QQuickWidget);
-    Q_QUICK_INPUT_PROFILE(addEvent<QQuickProfiler::Mouse>());
+    Q_QUICK_INPUT_PROFILE(QQuickProfiler::Mouse, QQuickProfiler::InputMousePress, e->button(),
+                          e->buttons());
 
     QMouseEvent mappedEvent(e->type(), e->localPos(), e->screenPos(), e->button(), e->buttons(), e->modifiers());
     QCoreApplication::sendEvent(d->offscreenWindow, &mappedEvent);
@@ -1097,7 +1168,8 @@ void QQuickWidget::mousePressEvent(QMouseEvent *e)
 void QQuickWidget::mouseReleaseEvent(QMouseEvent *e)
 {
     Q_D(QQuickWidget);
-    Q_QUICK_INPUT_PROFILE(addEvent<QQuickProfiler::Mouse>());
+    Q_QUICK_INPUT_PROFILE(QQuickProfiler::Mouse, QQuickProfiler::InputMouseRelease, e->button(),
+                          e->buttons());
 
     QMouseEvent mappedEvent(e->type(), e->localPos(), e->screenPos(), e->button(), e->buttons(), e->modifiers());
     QCoreApplication::sendEvent(d->offscreenWindow, &mappedEvent);
@@ -1109,26 +1181,46 @@ void QQuickWidget::mouseReleaseEvent(QMouseEvent *e)
 void QQuickWidget::wheelEvent(QWheelEvent *e)
 {
     Q_D(QQuickWidget);
-    Q_QUICK_INPUT_PROFILE(addEvent<QQuickProfiler::Mouse>());
+    Q_QUICK_INPUT_PROFILE(QQuickProfiler::Mouse, QQuickProfiler::InputMouseWheel,
+                          e->angleDelta().x(), e->angleDelta().y());
 
     // Wheel events only have local and global positions, no need to map.
     QCoreApplication::sendEvent(d->offscreenWindow, e);
 }
 #endif
 
-
+/*!
+   \reimp
+*/
 void QQuickWidget::focusInEvent(QFocusEvent * event)
 {
     Q_D(QQuickWidget);
     d->offscreenWindow->focusInEvent(event);
 }
 
+/*!
+   \reimp
+*/
 void QQuickWidget::focusOutEvent(QFocusEvent * event)
 {
     Q_D(QQuickWidget);
     d->offscreenWindow->focusOutEvent(event);
 }
 
+static Qt::WindowState resolveWindowState(Qt::WindowStates states)
+{
+    // No more than one of these 3 can be set
+    if (states & Qt::WindowMinimized)
+        return Qt::WindowMinimized;
+    if (states & Qt::WindowMaximized)
+        return Qt::WindowMaximized;
+    if (states & Qt::WindowFullScreen)
+        return Qt::WindowFullScreen;
+
+    // No state means "windowed" - we ignore Qt::WindowActive
+    return Qt::WindowNoState;
+}
+
 /*! \reimp */
 bool QQuickWidget::event(QEvent *e)
 {
@@ -1150,6 +1242,17 @@ bool QQuickWidget::event(QEvent *e)
         break;
 
     case QEvent::ScreenChangeInternal:
+        if (QWindow *window = this->window()->windowHandle()) {
+            QScreen *newScreen = window->screen();
+
+            if (d->offscreenWindow)
+                d->offscreenWindow->setScreen(newScreen);
+            if (d->offscreenSurface)
+                d->offscreenSurface->setScreen(newScreen);
+            if (d->context)
+                d->context->setScreen(newScreen);
+        }
+
         if (d->fbo) {
             // This will check the size taking the devicePixelRatio into account
             // and recreate if needed.
@@ -1158,10 +1261,15 @@ bool QQuickWidget::event(QEvent *e)
         }
         break;
 
+    case QEvent::Show:
     case QEvent::Move:
         d->updatePosition();
         break;
 
+    case QEvent::WindowStateChange:
+        d->offscreenWindow->setWindowState(resolveWindowState(windowState()));
+        break;
+
     default:
         break;
     }
@@ -1214,6 +1322,14 @@ void QQuickWidget::triggerUpdate()
     Q_D(QQuickWidget);
     d->updatePending = true;
      if (!d->eventPending) {
+        // There's no sense in immediately kicking a render off now, as
+        // there may be a number of triggerUpdate calls to come from a multitude
+        // of different sources (network, touch/mouse/keyboard, timers,
+        // animations, ...), and we want to batch them all into single frames as
+        // much as possible for the sake of interactivity and responsiveness.
+        //
+        // To achieve this, we set a timer and only perform the rendering when
+        // this is complete.
         const int exhaustDelay = 5;
         d->updateTimer.start(exhaustDelay, Qt::PreciseTimer, this);
         d->eventPending = true;
@@ -1239,6 +1355,14 @@ void QQuickWidget::setFormat(const QSurfaceFormat &format)
     newFormat.setDepthBufferSize(qMax(newFormat.depthBufferSize(), currentFormat.depthBufferSize()));
     newFormat.setStencilBufferSize(qMax(newFormat.stencilBufferSize(), currentFormat.stencilBufferSize()));
     newFormat.setAlphaBufferSize(qMax(newFormat.alphaBufferSize(), currentFormat.alphaBufferSize()));
+
+    // Do not include the sample count. Requesting a multisampled context is not necessary
+    // since we render into an FBO, never to an actual surface. What's more, attempting to
+    // create a pbuffer with a multisampled config crashes certain implementations. Just
+    // avoid the entire hassle, the result is the same.
+    d->requestedSamples = newFormat.samples();
+    newFormat.setSamples(0);
+
     d->offscreenWindow->setFormat(newFormat);
 }
 
@@ -1268,9 +1392,10 @@ QImage QQuickWidget::grabFramebuffer() const
 /*!
   Sets the clear \a color. By default this is an opaque color.
 
-  To get a semi- or fully transparent QQuickWidget, call this function with \a
-  color set to Qt::transparent and set the Qt::WA_TranslucentBackground widget
-  attribute.
+  To get a semi-transparent QQuickWidget, call this function with
+  \a color set to Qt::transparent, set the Qt::WA_TranslucentBackground
+  widget attribute on the top-level window, and request an alpha
+  channel via setFormat().
 
   \sa QQuickWindow::setColor()
  */
