@@ -588,11 +588,11 @@ void tst_QQmlProfilerService::scenegraphData()
     checkTraceReceived();
     checkJsHeap();
 
-    // check that at least one frame was rendered
-    // there should be a SGPolishAndSync + SGRendererFrame + SGRenderLoopFrame sequence
-    // (though we can't be sure to get the SGRenderLoopFrame in the threaded renderer)
+    // Check that at least one frame was rendered.
+    // There should be a SGContextFrame + SGRendererFrame + SGRenderLoopFrame sequence,
+    // but we can't be sure to get the SGRenderLoopFrame in the threaded renderer.
     //
-    // since the rendering happens in a different thread, there could be other unrelated events
+    // Since the rendering happens in a different thread, there could be other unrelated events
     // interleaved. Also, events could carry the same time stamps and be sorted in an unexpected way
     // if the clocks are acting up.
     qint64 contextFrameTime = -1;
@@ -621,8 +621,13 @@ void tst_QQmlProfilerService::scenegraphData()
 
     foreach (const QQmlProfilerData &msg, m_client->asynchronousMessages) {
         if (msg.detailType == QQmlProfilerDefinitions::SceneGraphRenderLoopFrame) {
-            QVERIFY(msg.time >= renderFrameTime);
-            break;
+            if (msg.time >= contextFrameTime) {
+                // Make sure SceneGraphRenderLoopFrame is not between SceneGraphContextFrame and
+                // SceneGraphRendererFrame. A SceneGraphRenderLoopFrame before everything else is
+                // OK as the scene graph might decide to do an initial rendering.
+                QVERIFY(msg.time >= renderFrameTime);
+                break;
+            }
         }
     }
 }
