@@ -1,39 +1,34 @@
 
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
 
-#ifndef DEBUGUTIL_H
-#define DEBUGUTIL_H
+#ifndef DEBUGUTIL_P_H
+#define DEBUGUTIL_P_H
 
 //
 //  W A R N I N G
@@ -46,21 +41,23 @@
 // We mean it.
 //
 
-#include <QEventLoop>
-#include <QTimer>
-#include <QThread>
-#include <QTest>
-#include <QProcess>
-#include <QMutex>
+#include <private/qqmldebugclient_p.h>
 
+#include <QtCore/qeventloop.h>
+#include <QtCore/qtimer.h>
+#include <QtCore/qthread.h>
+#include <QtCore/qprocess.h>
+#include <QtCore/qmutex.h>
+#include <QtTest/qtest.h>
 #include <QtQml/qqmlengine.h>
 
-#include "qqmldebugclient.h"
-
 class QQmlDebugTest
 {
 public:
     static bool waitForSignal(QObject *receiver, const char *member, int timeout = 5000);
+    static QList<QQmlDebugClient *> createOtherClients(QQmlDebugConnection *connection);
+    static QString clientStateString(const QQmlDebugClient *client);
+    static QString connectionStateString(const QQmlDebugConnection *connection);
 };
 
 class QQmlDebugTestClient : public QQmlDebugClient
@@ -92,7 +89,7 @@ public:
 
     QString state();
 
-    void setEnvironment(const QStringList &environment);
+    void addEnvironment(const QString &environment);
 
     void start(const QStringList &arguments);
     bool waitForSessionStart();
@@ -128,4 +125,23 @@ private:
     int m_receivedBindErrors;
 };
 
-#endif // DEBUGUTIL_H
+class QQmlInspectorResultRecipient : public QObject
+{
+    Q_OBJECT
+public:
+    QQmlInspectorResultRecipient(QObject *parent = 0) :
+        QObject(parent), lastResponseId(-1), lastResult(false) {}
+
+    int lastResponseId;
+    bool lastResult;
+
+public slots:
+
+    void recordResponse(int requestId, bool result)
+    {
+        lastResponseId = requestId;
+        lastResult = result;
+    }
+};
+
+#endif // DEBUGUTIL_P_H
