@@ -1,31 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -56,7 +51,7 @@ private slots:
     void errors();
     void rewriteErrors();
     void singletonTypeTarget();
-    void clearImplicitTarget();
+    void enableDisable_QTBUG_36350();
 
 private:
     QQmlEngine engine;
@@ -330,28 +325,29 @@ void tst_qqmlconnections::singletonTypeTarget()
     delete object;
 }
 
-//QTBUG-56499
-void tst_qqmlconnections::clearImplicitTarget()
+void tst_qqmlconnections::enableDisable_QTBUG_36350()
 {
     QQmlEngine engine;
-    QQmlComponent c(&engine, testFileUrl("test-connection-implicit.qml"));
+    QQmlComponent c(&engine, testFileUrl("test-connection.qml"));
     QQuickItem *item = qobject_cast<QQuickItem*>(c.create());
-
     QVERIFY(item != 0);
 
-    // normal case: fire Connections
-    item->setWidth(100.);
-    QCOMPARE(item->property("tested").toBool(), true);
-
-    item->setProperty("tested", false);
-    // clear the implicit target
-    QQmlConnections *connections = item->findChild<QQmlConnections*>();
+    QQmlConnections *connections = item->findChild<QQmlConnections*>("connections");
     QVERIFY(connections);
-    connections->setTarget(0);
 
-    // target cleared: no longer fire Connections
-    item->setWidth(150.);
+    connections->setEnabled(false);
     QCOMPARE(item->property("tested").toBool(), false);
+    QCOMPARE(item->width(), 50.);
+    emit item->setWidth(100.);
+    QCOMPARE(item->width(), 100.);
+    QCOMPARE(item->property("tested").toBool(), false); //Should not have received signal to change property
+
+    connections->setEnabled(true); //Re-enable the connectSignals()
+    QCOMPARE(item->property("tested").toBool(), false);
+    QCOMPARE(item->width(), 100.);
+    emit item->setWidth(50.);
+    QCOMPARE(item->width(), 50.);
+    QCOMPARE(item->property("tested").toBool(), true); //Should have received signal to change property
 
     delete item;
 }
