@@ -1,39 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -46,15 +33,17 @@
 #include <QtCore/qdatetime.h>
 #include <QtGui/qmatrix.h>
 #include <QtGui/qcolor.h>
+#include <QtGui/qvector2d.h>
 #include <QtGui/qvector3d.h>
 #include <QtGui/qvector4d.h>
+#include <QtGui/qquaternion.h>
 #include <QtQml/qqml.h>
 #include <QtQml/qqmlcomponent.h>
 #include <QtQml/qqmlparserstatus.h>
 #include <QtQml/qqmlpropertyvaluesource.h>
 #include <QtQml/qqmlscriptstring.h>
 #include <QtQml/qqmlproperty.h>
-
+#include <private/qqmlcompiler_p.h>
 #include <private/qqmlcustomparser_p.h>
 
 QVariant myCustomVariantTypeConverter(const QString &data);
@@ -255,8 +244,10 @@ class MyTypeObject : public QObject
     Q_PROPERTY(QRectF rectFProperty READ rectFProperty WRITE setRectFProperty NOTIFY rectFPropertyChanged)
     Q_PROPERTY(bool boolProperty READ boolProperty WRITE setBoolProperty NOTIFY boolPropertyChanged)
     Q_PROPERTY(QVariant variantProperty READ variantProperty WRITE setVariantProperty NOTIFY variantPropertyChanged)
+    Q_PROPERTY(QVector2D vector2Property READ vector2Property WRITE setVector2Property NOTIFY vector2PropertyChanged)
     Q_PROPERTY(QVector3D vectorProperty READ vectorProperty WRITE setVectorProperty NOTIFY vectorPropertyChanged)
     Q_PROPERTY(QVector4D vector4Property READ vector4Property WRITE setVector4Property NOTIFY vector4PropertyChanged)
+    Q_PROPERTY(QQuaternion quaternionProperty READ quaternionProperty WRITE setQuaternionProperty NOTIFY quaternionPropertyChanged)
     Q_PROPERTY(QUrl urlProperty READ urlProperty WRITE setUrlProperty NOTIFY urlPropertyChanged)
 
     Q_PROPERTY(QQmlScriptString scriptProperty READ scriptProperty WRITE setScriptProperty)
@@ -305,7 +296,7 @@ public:
         emit flagPropertyChanged();
     }
 
-    enum MyEnum { EnumVal1, EnumVal2 };
+    enum MyEnum { EnumVal1, EnumVal2, lowercaseEnumVal };
     MyEnum enumPropertyValue;
     MyEnum enumProperty() const {
         return enumPropertyValue;
@@ -537,6 +528,15 @@ public:
         emit vectorPropertyChanged();
     }
 
+    QVector2D vector2PropertyValue;
+    QVector2D vector2Property() const {
+        return vector2PropertyValue;
+    }
+    void setVector2Property(const QVector2D &v) {
+        vector2PropertyValue = v;
+        emit vector2PropertyChanged();
+    }
+
     QVector4D vector4PropertyValue;
     QVector4D vector4Property() const {
         return vector4PropertyValue;
@@ -546,6 +546,15 @@ public:
         emit vector4PropertyChanged();
     }
 
+    QQuaternion quaternionPropertyValue;
+    QQuaternion quaternionProperty() const {
+        return quaternionPropertyValue;
+    }
+    void setQuaternionProperty(const QQuaternion &v) {
+        quaternionPropertyValue = v;
+        emit quaternionPropertyChanged();
+    }
+
     QUrl urlPropertyValue;
     QUrl urlProperty() const {
         return urlPropertyValue;
@@ -599,12 +608,20 @@ signals:
     void boolPropertyChanged();
     void variantPropertyChanged();
     void vectorPropertyChanged();
+    void vector2PropertyChanged();
     void vector4PropertyChanged();
+    void quaternionPropertyChanged();
     void urlPropertyChanged();
 
 };
 Q_DECLARE_OPERATORS_FOR_FLAGS(MyTypeObject::MyFlags)
 
+// FIXME: If no subclass is used for the singleton registration with qmlRegisterSingletonType(),
+//        the valueTypes() test will fail.
+class MyTypeObjectSingleton : public MyTypeObject
+{
+    Q_OBJECT
+};
 
 class MyContainer : public QObject
 {
@@ -739,15 +756,15 @@ class MyCustomParserType : public QObject
 class MyCustomParserTypeParser : public QQmlCustomParser
 {
 public:
-    QByteArray compile(const QV4::CompiledData::QmlUnit *, const QList<const QV4::CompiledData::Binding *> &) { return QByteArray(); }
-    void setCustomData(QObject *, const QByteArray &, QQmlCompiledData*) {}
+    virtual void verifyBindings(const QV4::CompiledData::Unit *, const QList<const QV4::CompiledData::Binding *> &) {}
+    virtual void applyBindings(QObject *, QQmlCompiledData *, const QList<const QV4::CompiledData::Binding *> &) {}
 };
 
 class EnumSupportingCustomParser : public QQmlCustomParser
 {
 public:
-    QByteArray compile(const QV4::CompiledData::QmlUnit *qmlUnit, const QList<const QV4::CompiledData::Binding *> &bindings);
-    void setCustomData(QObject *, const QByteArray &, QQmlCompiledData*) {}
+    virtual void verifyBindings(const QV4::CompiledData::Unit *, const QList<const QV4::CompiledData::Binding *> &);
+    virtual void applyBindings(QObject *, QQmlCompiledData *, const QList<const QV4::CompiledData::Binding *> &) {}
 };
 
 class MyParserStatus : public QObject, public QQmlParserStatus
@@ -1087,9 +1104,20 @@ class MyEnumDerivedClass : public MyEnum2Class
     Q_OBJECT
 };
 
+class MyCompositeBaseType : public QObject
+{
+    Q_OBJECT
+    Q_ENUMS(CompositeEnum)
+
+public:
+    enum CompositeEnum { EnumValue0, EnumValue42 = 42 };
+    static QObject *qmlAttachedProperties(QObject *parent) { return new QObject(parent); }
+};
+
 Q_DECLARE_METATYPE(MyEnum2Class::EnumB)
 Q_DECLARE_METATYPE(MyEnum1Class::EnumA)
 Q_DECLARE_METATYPE(Qt::TextFormat)
+Q_DECLARE_METATYPE(MyCompositeBaseType::CompositeEnum)
 
 QML_DECLARE_TYPE(MyRevisionedBaseClassRegistered)
 QML_DECLARE_TYPE(MyRevisionedBaseClassUnregistered)
@@ -1097,6 +1125,8 @@ QML_DECLARE_TYPE(MyRevisionedClass)
 QML_DECLARE_TYPE(MyRevisionedSubclass)
 QML_DECLARE_TYPE(MySubclass)
 QML_DECLARE_TYPE(MyReceiversTestObject)
+QML_DECLARE_TYPE(MyCompositeBaseType)
+QML_DECLARE_TYPEINFO(MyCompositeBaseType, QML_HAS_ATTACHED_PROPERTIES)
 
 class CustomBinding : public QObject, public QQmlParserStatus
 {
@@ -1112,13 +1142,15 @@ public:
     void setTarget(QObject *newTarget) { m_target = newTarget; }
 
     QPointer<QObject> m_target;
+    QQmlRefPointer<QQmlCompiledData> cdata;
+    QList<const QV4::CompiledData::Binding*> bindings;
     QByteArray m_bindingData;
 };
 
 class CustomBindingParser : public QQmlCustomParser
 {
-    virtual QByteArray compile(const QV4::CompiledData::QmlUnit *qmlUnit, const QList<const QV4::CompiledData::Binding *> &bindings);
-    virtual void setCustomData(QObject *object, const QByteArray &data, QQmlCompiledData *);
+    virtual void verifyBindings(const QV4::CompiledData::Unit *, const QList<const QV4::CompiledData::Binding *> &) {}
+    virtual void applyBindings(QObject *, QQmlCompiledData *, const QList<const QV4::CompiledData::Binding *> &);
 };
 
 class SimpleObjectWithCustomParser : public QObject
@@ -1142,10 +1174,29 @@ private:
     int m_customBindingsCount;
 };
 
+class SimpleObjectExtension : public QObject
+{
+    Q_OBJECT
+    Q_PROPERTY(int extendedProperty READ extendedProperty WRITE setExtendedProperty NOTIFY extendedPropertyChanged)
+public:
+    SimpleObjectExtension(QObject *parent = 0)
+        : QObject(parent)
+        , m_extendedProperty(1584)
+    {}
+
+    void setExtendedProperty(int extendedProperty) { m_extendedProperty = extendedProperty; emit extendedPropertyChanged(); }
+    int extendedProperty() const { return m_extendedProperty; }
+
+signals:
+   void extendedPropertyChanged();
+private:
+   int m_extendedProperty;
+};
+
 class SimpleObjectCustomParser : public QQmlCustomParser
 {
-    virtual QByteArray compile(const QV4::CompiledData::QmlUnit *, const QList<const QV4::CompiledData::Binding *> &bindings);
-    virtual void setCustomData(QObject *object, const QByteArray &data, QQmlCompiledData *);
+    virtual void verifyBindings(const QV4::CompiledData::Unit *, const QList<const QV4::CompiledData::Binding *> &) {}
+    virtual void applyBindings(QObject *, QQmlCompiledData *, const QList<const QV4::CompiledData::Binding *> &);
 };
 
 class RootObjectInCreationTester : public QObject
