@@ -1,31 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -75,6 +70,9 @@ private slots:
     void importStrictModule();
     void importStrictModule_data();
     void importProtectedModule();
+    void importsChildPlugin();
+    void importsChildPlugin2();
+    void importsChildPlugin21();
 
 private:
     QString m_importsDirectory;
@@ -580,6 +578,57 @@ void tst_qqmlmoduleplugin::importProtectedModule()
     QVERIFY(object != 0);
 }
 
+void tst_qqmlmoduleplugin::importsChildPlugin()
+{
+    QQmlEngine engine;
+    engine.addImportPath(m_importsDirectory);
+    QTest::ignoreMessage(QtWarningMsg, "child plugin created");
+    QTest::ignoreMessage(QtWarningMsg, "child import worked");
+    QTest::ignoreMessage(QtWarningMsg, "Module 'org.qtproject.AutoTestQmlPluginType.ChildPlugin' does not contain a module identifier directive - it cannot be protected from external registrations.");
+    QQmlComponent component(&engine, testFileUrl(QStringLiteral("child.qml")));
+    foreach (QQmlError err, component.errors())
+        qWarning() << err;
+    VERIFY_ERRORS(0);
+    QObject *object = component.create();
+    QVERIFY(object != 0);
+    QCOMPARE(object->property("value").toInt(),123);
+    delete object;
+}
+
+void tst_qqmlmoduleplugin::importsChildPlugin2()
+{
+    QQmlEngine engine;
+    engine.addImportPath(m_importsDirectory);
+    QTest::ignoreMessage(QtWarningMsg, "child plugin2 created");
+    QTest::ignoreMessage(QtWarningMsg, "child import2 worked");
+    QTest::ignoreMessage(QtWarningMsg, "Module 'org.qtproject.AutoTestQmlPluginType.ChildPlugin' does not contain a module identifier directive - it cannot be protected from external registrations.");
+    QQmlComponent component(&engine, testFileUrl(QStringLiteral("child2.qml")));
+    foreach (QQmlError err, component.errors())
+        qWarning() << err;
+    VERIFY_ERRORS(0);
+    QObject *object = component.create();
+    QVERIFY(object != 0);
+    QCOMPARE(object->property("value").toInt(),123);
+    delete object;
+}
+
+void tst_qqmlmoduleplugin::importsChildPlugin21()
+{
+    QQmlEngine engine;
+    engine.addImportPath(m_importsDirectory);
+    QTest::ignoreMessage(QtWarningMsg, "child plugin2.1 created");
+    QTest::ignoreMessage(QtWarningMsg, "child import2.1 worked");
+    QTest::ignoreMessage(QtWarningMsg, "Module 'org.qtproject.AutoTestQmlPluginType.ChildPlugin' does not contain a module identifier directive - it cannot be protected from external registrations.");
+    QQmlComponent component(&engine, testFileUrl(QStringLiteral("child21.qml")));
+    foreach (QQmlError err, component.errors())
+        qWarning() << err;
+    VERIFY_ERRORS(0);
+    QObject *object = component.create();
+    QVERIFY(object != 0);
+    QCOMPARE(object->property("value").toInt(),123);
+    delete object;
+}
+
 QTEST_MAIN(tst_qqmlmoduleplugin)
 
 #include "tst_qqmlmoduleplugin.moc"
