@@ -1,31 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -140,7 +135,6 @@ private slots:
 
     // Bugs
     void crashOnValueProperty();
-    void aliasPropertyBindings_data();
     void aliasPropertyBindings();
     void noContext();
     void assignEmptyVariantMap();
@@ -148,6 +142,8 @@ private slots:
     void registeredCompositeTypeProperty();
     void deeplyNestedObject();
     void readOnlyDynamicProperties();
+
+    void floatToStringPrecision_data();
     void floatToStringPrecision();
 
     void copy();
@@ -1824,40 +1820,23 @@ void tst_qqmlproperty::crashOnValueProperty()
     QCOMPARE(p.read(), QVariant(20));
 }
 
-void tst_qqmlproperty::aliasPropertyBindings_data()
-{
-    QTest::addColumn<QString>("file");
-    QTest::addColumn<QString>("subObject");
-
-    QTest::newRow("same object") << "aliasPropertyBindings.qml" << "";
-    QTest::newRow("different objects") << "aliasPropertyBindings2.qml" << "innerObject";
-}
-
-// QTBUG-13719, QTBUG-58271
+// QTBUG-13719
 void tst_qqmlproperty::aliasPropertyBindings()
 {
-    QFETCH(QString, file);
-    QFETCH(QString, subObject);
-
-    QQmlComponent component(&engine, testFileUrl(file));
+    QQmlComponent component(&engine, testFileUrl("aliasPropertyBindings.qml"));
 
     QObject *object = component.create();
     QVERIFY(object != 0);
 
-    // the object where realProperty lives
-    QObject *realPropertyObject = object;
-    if (!subObject.isEmpty())
-       realPropertyObject = object->property(subObject.toLatin1()).value<QObject*>();
-
-    QCOMPARE(realPropertyObject->property("realProperty").toReal(), 90.);
+    QCOMPARE(object->property("realProperty").toReal(), 90.);
     QCOMPARE(object->property("aliasProperty").toReal(), 90.);
 
     object->setProperty("test", 10);
 
-    QCOMPARE(realPropertyObject->property("realProperty").toReal(), 110.);
+    QCOMPARE(object->property("realProperty").toReal(), 110.);
     QCOMPARE(object->property("aliasProperty").toReal(), 110.);
 
-    QQmlProperty realProperty(realPropertyObject, QLatin1String("realProperty"));
+    QQmlProperty realProperty(object, QLatin1String("realProperty"));
     QQmlProperty aliasProperty(object, QLatin1String("aliasProperty"));
 
     // Check there is a binding on these two properties
@@ -1876,18 +1855,18 @@ void tst_qqmlproperty::aliasPropertyBindings()
     QCOMPARE(QQmlPropertyPrivate::binding(realProperty),
              QQmlPropertyPrivate::binding(aliasProperty));
 
-    QCOMPARE(realPropertyObject->property("realProperty").toReal(), 96.);
+    QCOMPARE(object->property("realProperty").toReal(), 96.);
     QCOMPARE(object->property("aliasProperty").toReal(), 96.);
 
     // Check the old binding really has not effect any more
     object->setProperty("test", 4);
 
-    QCOMPARE(realPropertyObject->property("realProperty").toReal(), 96.);
+    QCOMPARE(object->property("realProperty").toReal(), 96.);
     QCOMPARE(object->property("aliasProperty").toReal(), 96.);
 
     object->setProperty("test2", 9);
 
-    QCOMPARE(realPropertyObject->property("realProperty").toReal(), 288.);
+    QCOMPARE(object->property("realProperty").toReal(), 288.);
     QCOMPARE(object->property("aliasProperty").toReal(), 288.);
 
     // Revert
@@ -1898,12 +1877,12 @@ void tst_qqmlproperty::aliasPropertyBindings()
     QCOMPARE(QQmlPropertyPrivate::binding(realProperty),
              QQmlPropertyPrivate::binding(aliasProperty));
 
-    QCOMPARE(realPropertyObject->property("realProperty").toReal(), 20.);
+    QCOMPARE(object->property("realProperty").toReal(), 20.);
     QCOMPARE(object->property("aliasProperty").toReal(), 20.);
 
     object->setProperty("test2", 3);
 
-    QCOMPARE(realPropertyObject->property("realProperty").toReal(), 20.);
+    QCOMPARE(object->property("realProperty").toReal(), 20.);
     QCOMPARE(object->property("aliasProperty").toReal(), 20.);
 
     delete object;
@@ -2004,7 +1983,7 @@ void tst_qqmlproperty::warnOnInvalidBinding()
     QTest::ignoreMessage(QtWarningMsg, expectedWarning.toLatin1().constData());
 
     // V8 error message for invalid binding to anchor
-    expectedWarning = testUrl.toString() + QString::fromLatin1(":14:33: Unable to assign QQuickItem_QML_8 to QQuickAnchorLine");
+    expectedWarning = testUrl.toString() + QString::fromLatin1(":14:33: Unable to assign QQuickItem_QML_6 to QQuickAnchorLine");
     QTest::ignoreMessage(QtWarningMsg, expectedWarning.toLatin1().constData());
 
     QQmlComponent component(&engine, testUrl);
@@ -2037,23 +2016,43 @@ void tst_qqmlproperty::readOnlyDynamicProperties()
     delete obj;
 }
 
+void tst_qqmlproperty::floatToStringPrecision_data()
+{
+    QTest::addColumn<QString>("propertyName");
+    QTest::addColumn<double>("number");
+    QTest::addColumn<QString>("qtString");
+    QTest::addColumn<QString>("jsString");
+
+    QTest::newRow("3.4")           << "a" << 3.4           << "3.4"         << "3.4";
+    QTest::newRow("0.035003945")   << "b" << 0.035003945   << "0.035003945" << "0.035003945";
+    QTest::newRow("0.0000012345")  << "c" << 0.0000012345  << "1.2345e-6"   << "0.0000012345";
+    QTest::newRow("0.00000012345") << "d" << 0.00000012345 << "1.2345e-7"   << "1.2345e-7";
+    QTest::newRow("1e20")          << "e" << 1e20          << "1e+20"       << "100000000000000000000";
+    QTest::newRow("1e21")          << "f" << 1e21          << "1e+21"       << "1e+21";
+}
+
 void tst_qqmlproperty::floatToStringPrecision()
 {
     QQmlComponent comp(&engine, testFileUrl("floatToStringPrecision.qml"));
     QObject *obj = comp.create();
     QVERIFY(obj != 0);
 
-    QCOMPARE(obj->property("a").toDouble(), 3.4);
-    QEXPECT_FAIL("", "QVariant's double-to-string conversion is worse than V4's.", Continue);
-    QCOMPARE(obj->property("a").toString(), QLatin1String("3.4"));
-    QCOMPARE(obj->property("b").toDouble(), 3.4);
-    QCOMPARE(obj->property("b").toString(), QLatin1String("3.4"));
+    QFETCH(QString, propertyName);
+    QFETCH(double, number);
+    QFETCH(QString, qtString);
+    QFETCH(QString, jsString);
 
-    QCOMPARE(obj->property("c").toDouble(), 0.035003945);
-    QEXPECT_FAIL("", "QVariant's double-to-string conversion is worse than V4's.", Continue);
-    QCOMPARE(obj->property("c").toString(), QLatin1String("0.035003945"));
-    QCOMPARE(obj->property("d").toDouble(), 0.035003945);
-    QCOMPARE(obj->property("d").toString(), QLatin1String("0.035003945"));
+    QByteArray name = propertyName.toLatin1();
+    QCOMPARE(obj->property(name).toDouble(), number);
+    QCOMPARE(obj->property(name).toString(), qtString);
+
+    QByteArray name1 = (propertyName + QLatin1Char('1')).toLatin1();
+    QCOMPARE(obj->property(name1).toDouble(), number);
+    QCOMPARE(obj->property(name1).toString(), qtString);
+
+    QByteArray name2 = (propertyName + QLatin1Char('2')).toLatin1();
+    QCOMPARE(obj->property(name2).toDouble(), number);
+    QCOMPARE(obj->property(name2).toString(), jsString);
 
     delete obj;
 }
