@@ -1,39 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/legal
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
 ** GNU General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -54,7 +41,6 @@
 
 // From qquickfolderlistmodel.h
 const int FileNameRole = Qt::UserRole+1;
-const int FilePathRole = Qt::UserRole+2;
 enum SortField { Unsorted, Name, Time, Size, Type };
 
 class tst_qquickfolderlistmodel : public QQmlDataTest
@@ -83,6 +69,7 @@ private slots:
     void showDotAndDotDot();
     void showDotAndDotDot_data();
     void sortReversed();
+    void introspectQrc();
 
 private:
     void checkNoErrors(const QQmlComponent& component);
@@ -101,8 +88,8 @@ void tst_qquickfolderlistmodel::checkNoErrors(const QQmlComponent& component)
         QList<QQmlError> errors = component.errors();
         for (int ii = 0; ii < errors.count(); ++ii) {
             const QQmlError &error = errors.at(ii);
-            QByteArray errorStr = QByteArray::number(error.line()) + ":" +
-                                  QByteArray::number(error.column()) + ":" +
+            QByteArray errorStr = QByteArray::number(error.line()) + ':' +
+                                  QByteArray::number(error.column()) + ':' +
                                   error.description().toUtf8();
             qWarning() << errorStr;
         }
@@ -127,7 +114,7 @@ void tst_qquickfolderlistmodel::basicProperties()
     QSignalSpy folderChangedSpy(flm, SIGNAL(folderChanged()));
     flm->setProperty("folder", dataDirectoryUrl());
     QVERIFY(folderChangedSpy.wait());
-    QCOMPARE(flm->property("count").toInt(), 6);
+    QCOMPARE(flm->property("count").toInt(), 8);
     QCOMPARE(flm->property("folder").toUrl(), dataDirectoryUrl());
     QCOMPARE(flm->property("parentFolder").toUrl(), QUrl::fromLocalFile(QDir(directory()).canonicalPath()));
     QCOMPARE(flm->property("sortField").toInt(), int(Name));
@@ -153,12 +140,12 @@ void tst_qquickfolderlistmodel::showFiles()
     QVERIFY(flm != 0);
 
     flm->setProperty("folder", dataDirectoryUrl());
-    QTRY_COMPARE(flm->property("count").toInt(), 6); // wait for refresh
+    QTRY_COMPARE(flm->property("count").toInt(), 8); // wait for refresh
     QCOMPARE(flm->property("showFiles").toBool(), true);
 
     flm->setProperty("showFiles", false);
     QCOMPARE(flm->property("showFiles").toBool(), false);
-    QTRY_COMPARE(flm->property("count").toInt(), 1); // wait for refresh
+    QTRY_COMPARE(flm->property("count").toInt(), 2); // wait for refresh
 }
 
 void tst_qquickfolderlistmodel::resetFiltering()
@@ -225,7 +212,7 @@ void tst_qquickfolderlistmodel::refresh()
     QVERIFY(flm != 0);
 
     flm->setProperty("folder", dataDirectoryUrl());
-    QTRY_COMPARE(flm->property("count").toInt(),6); // wait for refresh
+    QTRY_COMPARE(flm->property("count").toInt(),8); // wait for refresh
 
     int count = flm->rowCount();
 
@@ -303,11 +290,11 @@ void tst_qquickfolderlistmodel::changeDrive()
 
     flm->setProperty("folder",QUrl::fromLocalFile(dataDir));
     QCOMPARE(flm->property("folder").toUrl(), QUrl::fromLocalFile(dataDir));
-    QTRY_VERIFY(folderChangeSpy.count() == 1);
+    QTRY_COMPARE(folderChangeSpy.count(), 1);
 
     flm->setProperty("folder",QUrl::fromLocalFile("X:/resetfiltering/"));
     QCOMPARE(flm->property("folder").toUrl(), QUrl::fromLocalFile("X:/resetfiltering/"));
-    QTRY_VERIFY(folderChangeSpy.count() == 2);
+    QTRY_COMPARE(folderChangeSpy.count(), 2);
 }
 #endif
 
@@ -329,7 +316,7 @@ void tst_qquickfolderlistmodel::showDotAndDotDot()
     flm->setProperty("rootFolder", rootFolder);
     flm->setProperty("showDotAndDotDot", showDotAndDotDot);
 
-    int count = 6;
+    int count = 9;
     if (showDot) count++;
     if (showDotDot) count++;
     QTRY_COMPARE(flm->property("count").toInt(), count); // wait for refresh
@@ -360,10 +347,18 @@ void tst_qquickfolderlistmodel::sortReversed()
     QAbstractListModel *flm = qobject_cast<QAbstractListModel*>(component.create());
     QVERIFY(flm != 0);
     flm->setProperty("folder", dataDirectoryUrl());
+    QTRY_COMPARE(flm->property("count").toInt(), 9); // wait for refresh
+    QCOMPARE(flm->data(flm->index(0),FileNameRole).toString(), QLatin1String("txtdir"));
+}
 
-    int count = 6;
-    QTRY_COMPARE(flm->property("count").toInt(), count); // wait for refresh
-    QCOMPARE(flm->data(flm->index(0),FileNameRole).toString(), QLatin1String("sortReversed.qml"));
+void tst_qquickfolderlistmodel::introspectQrc()
+{
+    QQmlComponent component(&engine, testFileUrl("qrc.qml"));
+    checkNoErrors(component);
+    QAbstractListModel *flm = qobject_cast<QAbstractListModel*>(component.create());
+    QVERIFY(flm != 0);
+    QTRY_COMPARE(flm->property("count").toInt(), 1); // wait for refresh
+    QCOMPARE(flm->data(flm->index(0),FileNameRole).toString(), QLatin1String("hello.txt"));
 }
 
 QTEST_MAIN(tst_qquickfolderlistmodel)
