@@ -1,31 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -79,15 +74,14 @@ private:
 tst_examples::tst_examples()
 {
     // Add files to exclude here
-    excludedFiles << "examples/quick/canvas/tiger/tiger.qml"; // QTBUG-26528
     excludedFiles << "snippets/qml/listmodel/listmodel.qml"; //Just a ListModel, no root QQuickItem
     excludedFiles << "examples/quick/demos/photosurface/photosurface.qml"; // root item is Window rather than Item
 
-    // Add directories you want excluded here (don't add examples/, because they install to examples/qtdeclarative/)
+    // Add directories you want excluded here
     excludedDirs << "shared"; //Not an example
-    excludedDirs << "quick/text/fonts"; // QTBUG-29004
     excludedDirs << "snippets/qml/path"; //No root QQuickItem
-    excludedDirs << "tutorials/gettingStartedQml"; //C++ example, but no cpp files in root dir
+    excludedDirs << "examples/qml/qmlextensionplugins"; //Requires special import search path
+    excludedDirs << "examples/quick/tutorials/gettingStartedQml"; //C++ example, but no cpp files in root dir
 
     // These snippets are not expected to run on their own.
     excludedDirs << "snippets/qml/visualdatamodel_rootindex";
@@ -175,9 +169,8 @@ void tst_examples::namingConvention(const QDir &d)
 void tst_examples::namingConvention()
 {
     QStringList examplesLocations;
-    examplesLocations << QLibraryInfo::location(QLibraryInfo::ExamplesPath) + QLatin1String("/qtdeclarative");
-    examplesLocations << QLibraryInfo::location(QLibraryInfo::ExamplesPath) + QLatin1String("/qtquick");
-    examplesLocations << QLibraryInfo::location(QLibraryInfo::ExamplesPath) + QLatin1String("/qtqml");
+    examplesLocations << QLibraryInfo::location(QLibraryInfo::ExamplesPath) + QLatin1String("/qml");
+    examplesLocations << QLibraryInfo::location(QLibraryInfo::ExamplesPath) + QLatin1String("/quick");
 
     foreach(const QString &examples, examplesLocations) {
         QDir d(examples);
@@ -296,7 +289,6 @@ void tst_examples::sgsnippets_data()
 
 void tst_examples::sgsnippets()
 {
-    QQuickWindow window;
 
     QFETCH(QString, file);
 
@@ -306,19 +298,26 @@ void tst_examples::sgsnippets()
     QCOMPARE(component.status(), QQmlComponent::Ready);
 
     QScopedPointer<QObject> object(component.beginCreate(engine.rootContext()));
+    QQuickWindow *window = qobject_cast<QQuickWindow*>(object.data());
     QQuickItem *root = qobject_cast<QQuickItem *>(object.data());
-    if (!root)
+    if (!root && !window) {
         component.completeCreate();
-    QVERIFY(root);
+        QVERIFY(false);
+    }
+    if (!window)
+        window = new QQuickWindow;
 
-    window.resize(240, 320);
-    window.show();
-    QVERIFY(QTest::qWaitForWindowExposed(&window));
+    window->resize(240, 320);
+    window->show();
+    QVERIFY(QTest::qWaitForWindowExposed(window));
 
-    root->setParentItem(window.contentItem());
+    if (root)
+        root->setParentItem(window->contentItem());
     component.completeCreate();
 
     qApp->processEvents();
+    if (root)
+        delete window;
 }
 
 QTEST_MAIN(tst_examples)
