@@ -1,31 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -123,7 +118,7 @@ void tst_qquickborderimage::imageSource_data()
         << "<Unknown File>:2:1: QML BorderImage: Cannot open: " + testFileUrl("no-such-file.png").toString();
     QTest::newRow("remote") << "/colors.png" << true << "";
     QTest::newRow("remote not found") << "/no-such-file.png" << true
-        << "<Unknown File>:2:1: QML BorderImage: Error downloading {{ServerBaseUrl}}/no-such-file.png - server replied: Not found";
+        << "<Unknown File>:2:1: QML BorderImage: Error transferring {{ServerBaseUrl}}/no-such-file.png - server replied: Not found";
 }
 
 void tst_qquickborderimage::imageSource()
@@ -132,6 +127,13 @@ void tst_qquickborderimage::imageSource()
     QFETCH(bool, remote);
     QFETCH(QString, error);
 
+#if defined(Q_OS_LINUX) && !defined(Q_OS_ANDROID)
+    if (qstrcmp(QTest::currentDataTag(), "remote") == 0
+        || qstrcmp(QTest::currentDataTag(), "remote not found") == 0) {
+        QSKIP("Remote tests cause occasional hangs in the CI system -- QTBUG-45655");
+    }
+#endif
+
     TestHTTPServer server;
     if (remote) {
         QVERIFY2(server.listen(), qPrintable(server.errorString()));
@@ -150,12 +152,12 @@ void tst_qquickborderimage::imageSource()
     QVERIFY(obj != 0);
 
     if (remote)
-        QTRY_VERIFY(obj->status() == QQuickBorderImage::Loading);
+        QTRY_COMPARE(obj->status(), QQuickBorderImage::Loading);
 
     QCOMPARE(obj->source(), remote ? source : QUrl(source));
 
     if (error.isEmpty()) {
-        QTRY_VERIFY(obj->status() == QQuickBorderImage::Ready);
+        QTRY_COMPARE(obj->status(), QQuickBorderImage::Ready);
         QCOMPARE(obj->width(), 120.);
         QCOMPARE(obj->height(), 120.);
         QCOMPARE(obj->sourceSize().width(), 120);
@@ -163,7 +165,7 @@ void tst_qquickborderimage::imageSource()
         QCOMPARE(obj->horizontalTileMode(), QQuickBorderImage::Stretch);
         QCOMPARE(obj->verticalTileMode(), QQuickBorderImage::Stretch);
     } else {
-        QTRY_VERIFY(obj->status() == QQuickBorderImage::Error);
+        QTRY_COMPARE(obj->status(), QQuickBorderImage::Error);
     }
 
     delete obj;
@@ -178,13 +180,13 @@ void tst_qquickborderimage::clearSource()
     component.setData(componentStr.toLatin1(), QUrl::fromLocalFile(""));
     QQuickBorderImage *obj = qobject_cast<QQuickBorderImage*>(component.create());
     QVERIFY(obj != 0);
-    QVERIFY(obj->status() == QQuickBorderImage::Ready);
+    QCOMPARE(obj->status(), QQuickBorderImage::Ready);
     QCOMPARE(obj->width(), 120.);
     QCOMPARE(obj->height(), 120.);
 
     ctxt->setContextProperty("srcImage", "");
     QVERIFY(obj->source().isEmpty());
-    QVERIFY(obj->status() == QQuickBorderImage::Null);
+    QCOMPARE(obj->status(), QQuickBorderImage::Null);
     QCOMPARE(obj->width(), 0.);
     QCOMPARE(obj->height(), 0.);
 
@@ -296,14 +298,14 @@ void tst_qquickborderimage::sciSource()
     QVERIFY(obj != 0);
 
     if (remote)
-        QTRY_VERIFY(obj->status() == QQuickBorderImage::Loading);
+        QTRY_COMPARE(obj->status(), QQuickBorderImage::Loading);
 
     QCOMPARE(obj->source(), remote ? source : QUrl(source));
     QCOMPARE(obj->width(), 300.);
     QCOMPARE(obj->height(), 300.);
 
     if (valid) {
-        QTRY_VERIFY(obj->status() == QQuickBorderImage::Ready);
+        QTRY_COMPARE(obj->status(), QQuickBorderImage::Ready);
         QCOMPARE(obj->border()->left(), 10);
         QCOMPARE(obj->border()->top(), 20);
         QCOMPARE(obj->border()->right(), 30);
@@ -311,7 +313,7 @@ void tst_qquickborderimage::sciSource()
         QCOMPARE(obj->horizontalTileMode(), QQuickBorderImage::Round);
         QCOMPARE(obj->verticalTileMode(), QQuickBorderImage::Repeat);
     } else {
-        QTRY_VERIFY(obj->status() == QQuickBorderImage::Error);
+        QTRY_COMPARE(obj->status(), QQuickBorderImage::Error);
     }
 
     delete obj;
@@ -443,7 +445,7 @@ void tst_qquickborderimage::statusChanges()
     obj->setSource(source);
     if (remote)
         server.sendDelayedItem();
-    QTRY_VERIFY(obj->status() == finalStatus);
+    QTRY_COMPARE(obj->status(), finalStatus);
     QCOMPARE(spy.count(), emissions);
 
     delete obj;
@@ -529,8 +531,8 @@ void tst_qquickborderimage::progressAndStatusChanges()
     component.setData(componentStr.toLatin1(), QUrl::fromLocalFile(""));
     QQuickBorderImage *obj = qobject_cast<QQuickBorderImage*>(component.create());
     QVERIFY(obj != 0);
-    QVERIFY(obj->status() == QQuickBorderImage::Ready);
-    QTRY_VERIFY(obj->progress() == 1.0);
+    QCOMPARE(obj->status(), QQuickBorderImage::Ready);
+    QTRY_COMPARE(obj->progress(), 1.0);
 
     qRegisterMetaType<QQuickBorderImage::Status>();
     QSignalSpy sourceSpy(obj, SIGNAL(sourceChanged(QUrl)));
@@ -539,33 +541,33 @@ void tst_qquickborderimage::progressAndStatusChanges()
 
     // Same file
     ctxt->setContextProperty("srcImage", testFileUrl("heart200.png"));
-    QTRY_VERIFY(obj->status() == QQuickBorderImage::Ready);
-    QTRY_VERIFY(obj->progress() == 1.0);
+    QTRY_COMPARE(obj->status(), QQuickBorderImage::Ready);
+    QTRY_COMPARE(obj->progress(), 1.0);
     QTRY_COMPARE(sourceSpy.count(), 0);
     QTRY_COMPARE(progressSpy.count(), 0);
     QTRY_COMPARE(statusSpy.count(), 0);
 
     // Loading local file
     ctxt->setContextProperty("srcImage", testFileUrl("colors.png"));
-    QTRY_VERIFY(obj->status() == QQuickBorderImage::Ready);
-    QTRY_VERIFY(obj->progress() == 1.0);
+    QTRY_COMPARE(obj->status(), QQuickBorderImage::Ready);
+    QTRY_COMPARE(obj->progress(), 1.0);
     QTRY_COMPARE(sourceSpy.count(), 1);
     QTRY_COMPARE(progressSpy.count(), 0);
     QTRY_COMPARE(statusSpy.count(), 1);
 
     // Loading remote file
     ctxt->setContextProperty("srcImage", server.url("/heart200.png"));
-    QTRY_VERIFY(obj->status() == QQuickBorderImage::Loading);
-    QTRY_VERIFY(obj->progress() == 0.0);
-    QTRY_VERIFY(obj->status() == QQuickBorderImage::Ready);
-    QTRY_VERIFY(obj->progress() == 1.0);
+    QTRY_COMPARE(obj->status(), QQuickBorderImage::Loading);
+    QTRY_COMPARE(obj->progress(), 0.0);
+    QTRY_COMPARE(obj->status(), QQuickBorderImage::Ready);
+    QTRY_COMPARE(obj->progress(), 1.0);
     QTRY_COMPARE(sourceSpy.count(), 2);
     QTRY_VERIFY(progressSpy.count() > 1);
     QTRY_COMPARE(statusSpy.count(), 3);
 
     ctxt->setContextProperty("srcImage", "");
-    QTRY_VERIFY(obj->status() == QQuickBorderImage::Null);
-    QTRY_VERIFY(obj->progress() == 0.0);
+    QTRY_COMPARE(obj->status(), QQuickBorderImage::Null);
+    QTRY_COMPARE(obj->progress(), 0.0);
     QTRY_COMPARE(sourceSpy.count(), 3);
     QTRY_VERIFY(progressSpy.count() > 2);
     QTRY_COMPARE(statusSpy.count(), 4);
