@@ -1,31 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -53,7 +48,7 @@ public:
     TestItem(QQuickItem *parent = 0)
         : QQuickItem(parent), focused(false), pressCount(0), releaseCount(0)
         , wheelCount(0), acceptIncomingTouchEvents(true)
-        , touchEventReached(false), timestamp(0) {}
+        , touchEventReached(false) {}
 
     bool focused;
     int pressCount;
@@ -61,7 +56,6 @@ public:
     int wheelCount;
     bool acceptIncomingTouchEvents;
     bool touchEventReached;
-    ulong timestamp;
 protected:
     virtual void focusInEvent(QFocusEvent *) { Q_ASSERT(!focused); focused = true; }
     virtual void focusOutEvent(QFocusEvent *) { Q_ASSERT(focused); focused = false; }
@@ -71,7 +65,7 @@ protected:
         touchEventReached = true;
         event->setAccepted(acceptIncomingTouchEvents);
     }
-    virtual void wheelEvent(QWheelEvent *event) { event->accept(); ++wheelCount; timestamp = event->timestamp(); }
+    virtual void wheelEvent(QWheelEvent *event) { event->accept(); ++wheelCount; }
 };
 
 class TestWindow: public QQuickWindow
@@ -173,6 +167,8 @@ private slots:
     void contains_data();
     void contains();
 
+    void childAt();
+
     void ignoreButtonPressNotInAcceptedMouseButtons();
 
 private:
@@ -1436,14 +1432,12 @@ void tst_qquickitem::wheelEvent()
     item->setVisible(visible);
 
     QWheelEvent event(QPoint(100, 50), -120, Qt::NoButton, Qt::NoModifier, Qt::Vertical);
-    event.setTimestamp(123456UL);
     event.setAccepted(false);
     QGuiApplication::sendEvent(&window, &event);
 
     if (shouldReceiveWheelEvents) {
         QVERIFY(event.isAccepted());
         QCOMPARE(item->wheelCount, 1);
-        QCOMPARE(item->timestamp, 123456UL);
     } else {
         QVERIFY(!event.isAccepted());
         QCOMPARE(item->wheelCount, 0);
@@ -1976,6 +1970,47 @@ void tst_qquickitem::contains()
     QCOMPARE(result.toBool(), contains);
 }
 
+void tst_qquickitem::childAt()
+{
+    QQuickView view;
+    view.setSource(testFileUrl("childAtRectangle.qml"));
+    QQuickItem *root = qobject_cast<QQuickItem*>(view.rootObject());
+
+    int found = 0;
+    for (int i = 0; i < 16; i++)
+    {
+        if (root->childAt(i, 0))
+            found++;
+    }
+    QCOMPARE(found, 16);
+
+    found = 0;
+    for (int i = 0; i < 16; i++)
+    {
+        if (root->childAt(0, i))
+            found++;
+    }
+    QCOMPARE(found, 16);
+
+    found = 0;
+    for (int i = 0; i < 2; i++)
+    {
+        if (root->childAt(18 + i, 0))
+            found++;
+    }
+    QCOMPARE(found, 1);
+
+    found = 0;
+    for (int i = 0; i < 16; i++)
+    {
+        if (root->childAt(18, i))
+            found++;
+    }
+    QCOMPARE(found, 1);
+
+    QVERIFY(!root->childAt(19,19));
+}
+
 void tst_qquickitem::ignoreButtonPressNotInAcceptedMouseButtons()
 {
     // Verify the fix for QTBUG-31861
