@@ -1,31 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the QtQml module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -591,13 +586,10 @@ void tst_TouchMouse::buttonOnFlickable()
     QPoint p2 = p1 + QPoint(0, -10);
     QPoint p3 = p2 + QPoint(0, -10);
     QQuickTouchUtils::flush(window);
-    QTest::qWait(1); // because Flickable pays attention to velocity, we need some time between movements
     QTest::touchEvent(window, device).move(0, p1, window);
     QQuickTouchUtils::flush(window);
-    QTest::qWait(1);
     QTest::touchEvent(window, device).move(0, p2, window);
     QQuickTouchUtils::flush(window);
-    QTest::qWait(1);
     QTest::touchEvent(window, device).move(0, p3, window);
     QQuickTouchUtils::flush(window);
 
@@ -679,13 +671,10 @@ void tst_TouchMouse::buttonOnDelayedPressFlickable()
     QPoint p2 = p1 + QPoint(0, -10);
     QPoint p3 = p2 + QPoint(0, -10);
     QQuickTouchUtils::flush(window);
-    QTest::qWait(1);
     QTest::touchEvent(window, device).move(0, p1, window);
     QQuickTouchUtils::flush(window);
-    QTest::qWait(1);
     QTest::touchEvent(window, device).move(0, p2, window);
     QQuickTouchUtils::flush(window);
-    QTest::qWait(1);
     QTest::touchEvent(window, device).move(0, p3, window);
     QQuickTouchUtils::flush(window);
     QVERIFY(flickable->isMovingVertically());
@@ -877,19 +866,15 @@ void tst_TouchMouse::pinchOnFlickable()
     QQuickTouchUtils::flush(window);
     QCOMPARE(rect->position(), QPointF(200.0, 200.0));
     p -= QPoint(10, 0);
-    QTest::qWait(1);
     QTest::touchEvent(window, device).move(0, p, window);
     QQuickTouchUtils::flush(window);
     p -= QPoint(10, 0);
-    QTest::qWait(1);
     QTest::touchEvent(window, device).move(0, p, window);
     QQuickTouchUtils::flush(window);
     p -= QPoint(10, 0);
-    QTest::qWait(1);
     QTest::touchEvent(window, device).move(0, p, window);
     QQuickTouchUtils::flush(window);
     p -= QPoint(10, 0);
-    QTest::qWait(1);
     QTest::touchEvent(window, device).move(0, p, window);
     QQuickTouchUtils::flush(window);
     QTest::touchEvent(window, device).release(0, p, window);
@@ -1102,16 +1087,13 @@ void tst_TouchMouse::mouseOnFlickableOnPinch()
     QQuickTouchUtils::flush(window);
     QCOMPARE(rect->position(), QPointF(200.0, 200.0));
     p -= QPoint(10, 0);
-    QTest::qWait(1);
     pinchSequence.move(0, p, window).commit();
     QQuickTouchUtils::flush(window);
     p -= QPoint(10, 0);
-    QTest::qWait(1);
     pinchSequence.move(0, p, window).commit();
     QQuickTouchUtils::flush(window);
     QGuiApplication::processEvents();
     p -= QPoint(10, 0);
-    QTest::qWait(1);
     pinchSequence.move(0, p, window).commit();
     QQuickTouchUtils::flush(window);
 
