@@ -1,31 +1,26 @@
 /****************************************************************************
 **
-** Copyright (C) 2015 The Qt Company Ltd.
-** Contact: http://www.qt.io/licensing/
+** Copyright (C) 2016 The Qt Company Ltd.
+** Contact: https://www.qt.io/licensing/
 **
 ** This file is part of the tools applications of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL21$
+** $QT_BEGIN_LICENSE:GPL-EXCEPT$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
 ** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see http://www.qt.io/terms-conditions. For further
-** information use the contact form at http://www.qt.io/contact-us.
+** and conditions see https://www.qt.io/terms-conditions. For further
+** information use the contact form at https://www.qt.io/contact-us.
 **
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 or version 3 as published by the Free
-** Software Foundation and appearing in the file LICENSE.LGPLv21 and
-** LICENSE.LGPLv3 included in the packaging of this file. Please review the
-** following information to ensure the GNU Lesser General Public License
-** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
-** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
-**
-** As a special exception, The Qt Company gives you certain additional
-** rights. These rights are described in The Qt Company LGPL Exception
-** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
+** GNU General Public License Usage
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3 as published by the Free Software
+** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+** included in the packaging of this file. Please review the following
+** information to ensure the GNU General Public License requirements will
+** be met: https://www.gnu.org/licenses/gpl-3.0.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -156,7 +151,7 @@ struct Options
         applicationAttributes.append(Qt::AA_ShareOpenGLContexts);
     }
 
-    QUrl file;
+    QUrl url;
     bool originalQml;
     bool originalQmlRaster;
     bool maximized;
@@ -204,7 +199,7 @@ static int displayOptionsDialog(Options *options)
     QFileInfoList fileInfos = findQmlFiles(":/bundle") + findQmlFiles("./qmlscene-resources");
 
     foreach (QFileInfo fileInfo, fileInfos)
-        qmlFileComboBox->addItem(fileInfo.dir().dirName() + "/" + fileInfo.fileName(), QVariant::fromValue(fileInfo));
+        qmlFileComboBox->addItem(fileInfo.dir().dirName() + QLatin1Char('/') + fileInfo.fileName(), QVariant::fromValue(fileInfo));
 
     QCheckBox *originalCheckBox = new QCheckBox(&dialog);
     originalCheckBox->setText("Use original QML viewer");
@@ -235,7 +230,7 @@ static int displayOptionsDialog(Options *options)
         QVariant variant = qmlFileComboBox->itemData(qmlFileComboBox->currentIndex());
         QFileInfo fileInfo = variant.value<QFileInfo>();
 
-        if (fileInfo.canonicalFilePath().startsWith(":"))
+        if (fileInfo.canonicalFilePath().startsWith(QLatin1Char(':')))
             options->file = QUrl("qrc" + fileInfo.canonicalFilePath());
         else
             options->file = QUrl::fromLocalFile(fileInfo.canonicalFilePath());
@@ -252,12 +247,10 @@ static bool checkVersion(const QUrl &url)
     if (!qgetenv("QMLSCENE_IMPORT_NAME").isEmpty())
         fprintf(stderr, "QMLSCENE_IMPORT_NAME is no longer supported.\n");
 
-    QString fileName = url.toLocalFile();
-    if (fileName.isEmpty()) {
-        fprintf(stderr, "qmlscene: filename required.\n");
-        return false;
-    }
+    if (!url.isLocalFile())
+        return true;
 
+    const QString fileName = url.toLocalFile();
     QFile f(fileName);
     if (!f.open(QFile::ReadOnly | QFile::Text)) {
         fprintf(stderr, "qmlscene: failed to check version of file '%s', could not open...\n",
@@ -272,7 +265,7 @@ static bool checkVersion(const QUrl &url)
     bool codeFound= false;
     while (!codeFound) {
         QString line = stream.readLine();
-        if (line.contains("{")) {
+        if (line.contains(QLatin1Char('{'))) {
             codeFound = true;
         } else {
             QString import;
@@ -300,7 +293,7 @@ static void displayFileDialog(Options *options)
     QString fileName = QFileDialog::getOpenFileName(0, "Open QML file", QString(), "QML Files (*.qml)");
     if (!fileName.isEmpty()) {
         QFileInfo fi(fileName);
-        options->file = QUrl::fromLocalFile(fi.canonicalFilePath());
+        options->url = QUrl::fromLocalFile(fi.canonicalFilePath());
     }
 #else
     Q_UNUSED(options);
@@ -358,6 +351,8 @@ static void usage()
     puts("  --desktop..........................Force use of desktop GL (AA_UseDesktopOpenGL)");
     puts("  --gles.............................Force use of GLES (AA_UseOpenGLES)");
     puts("  --software.........................Force use of software rendering (AA_UseOpenGLES)");
+    puts("  --scaling..........................Enable High DPI scaling (AA_EnableHighDpiScaling)");
+    puts("  --no-scaling.......................Disable High DPI scaling (AA_DisableHighDpiScaling)");
     puts("  --verbose..........................Print version and graphical diagnostics for the run-time");
     puts("  -I <path> ........................ Add <path> to the list of import paths");
     puts("  -P <path> ........................ Add <path> to the list of plugin paths");
@@ -416,17 +411,65 @@ static void setWindowTitle(bool verbose, const QObject *topLevel, QWindow *windo
         window->setTitle(newTitle);
 }
 
+static QUrl parseUrlArgument(const QString &arg)
+{
+    const QUrl url = QUrl::fromUserInput(arg, QDir::currentPath(), QUrl::AssumeLocalFile);
+    if (!url.isValid()) {
+        fprintf(stderr, "Invalid URL: \"%s\"\n", qPrintable(arg));
+        return QUrl();
+    }
+    if (url.isLocalFile()) {
+        const QFileInfo fi(url.toLocalFile());
+        if (!fi.exists()) {
+            fprintf(stderr, "\"%s\" does not exist.\n",
+                    qPrintable(QDir::toNativeSeparators(fi.absoluteFilePath())));
+            return QUrl();
+        }
+    }
+    return url;
+}
+
 int main(int argc, char ** argv)
 {
     Options options;
 
     QStringList imports;
     QStringList pluginPaths;
+
+    // Parse arguments for application attributes to be applied before Q[Gui]Application creation.
     for (int i = 1; i < argc; ++i) {
-        if (*argv[i] != '-' && QFileInfo(QFile::decodeName(argv[i])).exists()) {
-            options.file = QUrl::fromLocalFile(argv[i]);
+        const char *arg = argv[i];
+        if (!qstrcmp(arg, "--disable-context-sharing"))
+            options.applicationAttributes.removeAll(Qt::AA_ShareOpenGLContexts);
+        else if (!qstrcmp(arg, "--gles"))
+            options.applicationAttributes.append(Qt::AA_UseOpenGLES);
+        else if (!qstrcmp(arg, "--software"))
+            options.applicationAttributes.append(Qt::AA_UseSoftwareOpenGL);
+        else if (!qstrcmp(arg, "--desktop"))
+            options.applicationAttributes.append(Qt::AA_UseDesktopOpenGL);
+        else if (!qstrcmp(arg, "--scaling"))
+            options.applicationAttributes.append(Qt::AA_EnableHighDpiScaling);
+        else if (!qstrcmp(arg, "--no-scaling"))
+            options.applicationAttributes.append(Qt::AA_DisableHighDpiScaling);
+    }
+
+    foreach (Qt::ApplicationAttribute a, options.applicationAttributes)
+        QCoreApplication::setAttribute(a);
+#ifdef QT_WIDGETS_LIB
+    QApplication app(argc, argv);
+#else
+    QGuiApplication app(argc, argv);
+#endif
+    app.setApplicationName("QtQmlViewer");
+    app.setOrganizationName("QtProject");
+    app.setOrganizationDomain("qt-project.org");
+
+    const QStringList arguments = QCoreApplication::arguments();
+    for (int i = 1, size = arguments.size(); i < size; ++i) {
+        if (!arguments.at(i).startsWith(QLatin1Char('-'))) {
+            options.url = parseUrlArgument(arguments.at(i));
         } else {
-            const QString lowerArgument = QString::fromLatin1(argv[i]).toLower();
+            const QString lowerArgument = arguments.at(i).toLower();
             if (lowerArgument == QLatin1String("--maximized"))
                 options.maximized = true;
             else if (lowerArgument == QLatin1String("--fullscreen"))
@@ -447,20 +490,12 @@ int main(int argc, char ** argv)
                 options.resizeViewToRootItem = true;
             else if (lowerArgument == QLatin1String("--multisample"))
                 options.multisample = true;
-            else if (lowerArgument == QLatin1String("--disable-context-sharing"))
-                options.applicationAttributes.removeAll(Qt::AA_ShareOpenGLContexts);
-            else if (lowerArgument == QLatin1String("--gles"))
-                options.applicationAttributes.append(Qt::AA_UseOpenGLES);
-            else if (lowerArgument == QLatin1String("--software"))
-                options.applicationAttributes.append(Qt::AA_UseSoftwareOpenGL);
-            else if (lowerArgument == QLatin1String("--desktop"))
-                options.applicationAttributes.append(Qt::AA_UseDesktopOpenGL);
             else if (lowerArgument == QLatin1String("--verbose"))
                 options.verbose = true;
-            else if (lowerArgument == QLatin1String("-i") && i + 1 < argc)
-                imports.append(QString::fromLatin1(argv[++i]));
-            else if (lowerArgument == QLatin1String("-p") && i + 1 < argc)
-                pluginPaths.append(QString::fromLatin1(argv[++i]));
+            else if (lowerArgument == QLatin1String("-i") && i + 1 < size)
+                imports.append(arguments.at(++i));
+            else if (lowerArgument == QLatin1String("-p") && i + 1 < size)
+                pluginPaths.append(arguments.at(++i));
             else if (lowerArgument == QLatin1String("--help")
                      || lowerArgument == QLatin1String("-help")
                      || lowerArgument == QLatin1String("--h")
@@ -469,17 +504,6 @@ int main(int argc, char ** argv)
         }
     }
 
-    foreach (Qt::ApplicationAttribute a, options.applicationAttributes)
-        QCoreApplication::setAttribute(a);
-#ifdef QT_WIDGETS_LIB
-    QApplication app(argc, argv);
-#else
-    QGuiApplication app(argc, argv);
-#endif
-    app.setApplicationName("QtQmlViewer");
-    app.setOrganizationName("QtProject");
-    app.setOrganizationDomain("qt-project.org");
-
 #ifndef QT_NO_TRANSLATION
     QTranslator translator;
     QTranslator qtTranslator;
@@ -502,7 +526,7 @@ int main(int argc, char ** argv)
 
     QUnifiedTimer::instance()->setSlowModeEnabled(options.slowAnimations);
 
-    if (options.file.isEmpty())
+    if (options.url.isEmpty())
 #if defined(QMLSCENE_BUNDLE)
         displayOptionsDialog(&options);
 #else
@@ -514,8 +538,8 @@ int main(int argc, char ** argv)
     if (options.verbose)
         puts(QLibraryInfo::build());
 
-    if (!options.file.isEmpty()) {
-        if (!options.versionDetection || checkVersion(options.file)) {
+    if (!options.url.isEmpty()) {
+        if (!options.versionDetection || checkVersion(options.url)) {
 #ifndef QT_NO_TRANSLATION
             QTranslator translator;
 #endif
@@ -528,15 +552,17 @@ int main(int argc, char ** argv)
                 engine.addImportPath(imports.at(i));
             for (int i = 0; i < pluginPaths.size(); ++i)
                 engine.addPluginPath(pluginPaths.at(i));
-            if (options.file.isLocalFile()) {
-                QFileInfo fi(options.file.toLocalFile());
+            if (options.url.isLocalFile()) {
+                QFileInfo fi(options.url.toLocalFile());
 #ifndef QT_NO_TRANSLATION
                 loadTranslationFile(translator, fi.path());
 #endif
                 loadDummyDataFiles(engine, fi.path());
             }
             QObject::connect(&engine, SIGNAL(quit()), QCoreApplication::instance(), SLOT(quit()));
-            component->loadUrl(options.file);
+            component->loadUrl(options.url);
+            while (component->isLoading())
+                QCoreApplication::processEvents();
             if ( !component->isReady() ) {
                 fprintf(stderr, "%s\n", qPrintable(component->errorString()));
                 return -1;
@@ -560,7 +586,7 @@ int main(int argc, char ** argv)
                         qxView->setResizeMode(QQuickView::SizeViewToRootObject);
                     else
                         qxView->setResizeMode(QQuickView::SizeRootObjectToView);
-                    qxView->setContent(options.file, component, contentItem);
+                    qxView->setContent(options.url, component, contentItem);
                 }
             }
 
